@@ -18,12 +18,11 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
+#include "SDL_internal.h"
 
 #ifndef SDL_assert_c_h_
 #define SDL_assert_c_h_
 
 extern void SDL_AssertionsQuit(void);
 
-#endif /* SDL_assert_c_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_assert_c_h_
