@@ -18,41 +18,40 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../SDL_internal.h"
+#include "SDL_internal.h"
 
-/* Get the name of the audio device we use for output */
+// Get the name of the audio device we use for output
 
-#if defined(SDL_AUDIO_DRIVER_NETBSD) || defined(SDL_AUDIO_DRIVER_OSS) || defined(SDL_AUDIO_DRIVER_SUNAUDIO)
+#if defined(SDL_AUDIO_DRIVER_NETBSD) || defined(SDL_AUDIO_DRIVER_OSS)
 
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h> /* For close() */
+#include <unistd.h> // For close()
 
-#include "SDL_stdinc.h"
 #include "SDL_audiodev_c.h"
 
-#ifndef _PATH_DEV_DSP
-#if defined(__NETBSD__) || defined(__OPENBSD__)
-#define _PATH_DEV_DSP "/dev/audio"
+#ifndef SDL_PATH_DEV_DSP
+#if defined(SDL_PLATFORM_NETBSD) || defined(SDL_PLATFORM_OPENBSD)
+#define SDL_PATH_DEV_DSP "/dev/audio"
 #else
-#define _PATH_DEV_DSP "/dev/dsp"
+#define SDL_PATH_DEV_DSP "/dev/dsp"
 #endif
 #endif
-#ifndef _PATH_DEV_DSP24
-#define _PATH_DEV_DSP24 "/dev/sound/dsp"
+#ifndef SDL_PATH_DEV_DSP24
+#define SDL_PATH_DEV_DSP24 "/dev/sound/dsp"
 #endif
-#ifndef _PATH_DEV_AUDIO
-#define _PATH_DEV_AUDIO "/dev/audio"
+#ifndef SDL_PATH_DEV_AUDIO
+#define SDL_PATH_DEV_AUDIO "/dev/audio"
 #endif
 
-static void test_device(const int iscapture, const char *fname, int flags, int (*test)(int fd))
+static void test_device(const bool recording, const char *fname, int flags, bool (*test)(int fd))
 {
     struct stat sb;
-    if ((stat(fname, &sb) == 0) && (S_ISCHR(sb.st_mode))) {
-        const int audio_fd = open(fname, flags | O_CLOEXEC, 0);
-        if (audio_fd >= 0) {
-            const int okay = test(audio_fd);
+    const int audio_fd = open(fname, flags | O_CLOEXEC, 0);
+    if (audio_fd >= 0) {
+        if ((fstat(audio_fd, &sb) == 0) && (S_ISCHR(sb.st_mode))) {
+            const bool okay = test(audio_fd);
             close(audio_fd);
             if (okay) {
                 static size_t dummyhandle = 0;
@@ -64,20 +63,22 @@ static void test_device(const int iscapture, const char *fname, int flags, int (
                  * information,  making this information inaccessible at
                  * enumeration time
                  */
-                SDL_AddAudioDevice(iscapture, fname, NULL, (void *)(uintptr_t)dummyhandle);
+                SDL_AddAudioDevice(recording, fname, NULL, (void *)(uintptr_t)dummyhandle);
             }
+        } else {
+            close(audio_fd);
         }
     }
 }
 
-static int test_stub(int fd)
+static bool test_stub(int fd)
 {
-    return 1;
+    return true;
 }
 
-static void SDL_EnumUnixAudioDevices_Internal(const int iscapture, const int classic, int (*test)(int))
+static void SDL_EnumUnixAudioDevices_Internal(const bool recording, const bool classic, bool (*test)(int))
 {
-    const int flags = iscapture ? OPEN_FLAGS_INPUT : OPEN_FLAGS_OUTPUT;
+    const int flags = recording ? OPEN_FLAGS_INPUT : OPEN_FLAGS_OUTPUT;
     const char *audiodev;
     char audiopath[1024];
 
@@ -85,26 +86,23 @@ static void SDL_EnumUnixAudioDevices_Internal(const int iscapture, const int cla
         test = test_stub;
     }
 
-    /* Figure out what our audio device is */
-    audiodev = SDL_getenv("SDL_PATH_DSP");
-    if (!audiodev) {
-        audiodev = SDL_getenv("AUDIODEV");
-    }
+    // Figure out what our audio device is
+    audiodev = SDL_getenv("AUDIODEV");
     if (!audiodev) {
         if (classic) {
-            audiodev = _PATH_DEV_AUDIO;
+            audiodev = SDL_PATH_DEV_AUDIO;
         } else {
             struct stat sb;
 
-            /* Added support for /dev/sound/\* in Linux 2.4 */
-            if (((stat("/dev/sound", &sb) == 0) && S_ISDIR(sb.st_mode)) && ((stat(_PATH_DEV_DSP24, &sb) == 0) && S_ISCHR(sb.st_mode))) {
-                audiodev = _PATH_DEV_DSP24;
+            // Added support for /dev/sound/\* in Linux 2.4
+            if (((stat("/dev/sound", &sb) == 0) && S_ISDIR(sb.st_mode)) && ((stat(SDL_PATH_DEV_DSP24, &sb) == 0) && S_ISCHR(sb.st_mode))) {
+                audiodev = SDL_PATH_DEV_DSP24;
             } else {
-                audiodev = _PATH_DEV_DSP;
+                audiodev = SDL_PATH_DEV_DSP;
             }
         }
     }
-    test_device(iscapture, audiodev, flags, test);
+    test_device(recording, audiodev, flags, test);
 
     if (SDL_strlen(audiodev) < (sizeof(audiopath) - 3)) {
         int instance = 0;
@@ -112,17 +110,15 @@ static void SDL_EnumUnixAudioDevices_Internal(const int iscapture, const int cla
             (void)SDL_snprintf(audiopath, SDL_arraysize(audiopath),
                                "%s%d", audiodev, instance);
             instance++;
-            test_device(iscapture, audiopath, flags, test);
+            test_device(recording, audiopath, flags, test);
         }
     }
 }
 
-void SDL_EnumUnixAudioDevices(const int classic, int (*test)(int))
+void SDL_EnumUnixAudioDevices(const bool classic, bool (*test)(int))
 {
-    SDL_EnumUnixAudioDevices_Internal(SDL_TRUE, classic, test);
-    SDL_EnumUnixAudioDevices_Internal(SDL_FALSE, classic, test);
+    SDL_EnumUnixAudioDevices_Internal(true, classic, test);
+    SDL_EnumUnixAudioDevices_Internal(false, classic, test);
 }
 
-#endif /* Audio driver selection */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // Audio device selection
