@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2019 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,24 +18,19 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_diskaudio_h_
 #define SDL_diskaudio_h_
 
-#include "SDL_rwops.h"
 #include "../SDL_sysaudio.h"
 
-/* Hidden "this" pointer for the audio functions */
-#define _THIS   SDL_AudioDevice *this
-
 struct SDL_PrivateAudioData
 {
-    /* The file descriptor for the audio device */
-    SDL_RWops *io;
+    // The file descriptor for the audio device
+    SDL_IOStream *io;
     Uint32 io_delay;
     Uint8 *mixbuf;
 };
 
-#endif /* SDL_diskaudio_h_ */
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_diskaudio_h_
