@@ -19,32 +19,22 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#ifndef _SDL_n3dsaudio_h_
-#define _SDL_n3dsaudio_h_
+#ifndef SDL_n3dsaudio_h
+#define SDL_n3dsaudio_h
 
 #include <3ds.h>
 
-/* Hidden "this" pointer for the audio functions */
-#define _THIS SDL_AudioDevice *this
-
-#define NUM_BUFFERS 3 /* -- Minimum 2! */
+#define NUM_BUFFERS 3 // -- Minimum 2!
 
 struct SDL_PrivateAudioData
 {
-    /* Speaker data */
+    // Speaker data
     Uint8 *mixbuf;
-    Uint32 mixlen;
-    Uint32 format;
-    Uint32 samplerate;
-    Uint32 channels;
-    Uint8 bytePerSample;
-    Uint32 isSigned;
     Uint32 nextbuf;
     ndspWaveBuf waveBuf[NUM_BUFFERS];
     LightLock lock;
     CondVar cv;
-    SDL_bool isCancelled;
+    bool isCancelled;
 };
 
-#endif /* _SDL_n3dsaudio_h_ */
-/* vi: set sts=4 ts=4 sw=4 expandtab: */
+#endif // SDL_n3dsaudio_h
