@@ -18,31 +18,27 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_netbsdaudio_h_
 #define SDL_netbsdaudio_h_
 
 #include "../SDL_sysaudio.h"
 
-#define _THIS SDL_AudioDevice *this
-
 struct SDL_PrivateAudioData
 {
-    /* The file descriptor for the audio device */
+    // The file descriptor for the audio device
     int audio_fd;
 
-    /* Raw mixing buffer */
+    // Raw mixing buffer
     Uint8 *mixbuf;
     int mixlen;
 
-    /* Support for audio timing using a timer, in addition to SDL_IOReady() */
+    // Support for audio timing using a timer, in addition to SDL_IOReady()
     float frame_ticks;
     float next_frame;
 };
 
-#define FUDGE_TICKS 10 /* The scheduler overhead ticks per frame */
+#define FUDGE_TICKS 10 // The scheduler overhead ticks per frame
 
-#endif /* SDL_netbsdaudio_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_netbsdaudio_h_
