@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -24,23 +24,18 @@
 
 #include "../SDL_sysaudio.h"
 
-/* Hidden "this" pointer for the audio functions */
-#define _THIS SDL_AudioDevice *this
-
 #define NUM_BUFFERS 2
 
 struct SDL_PrivateAudioData
 {
-    /* The hardware output channel. */
+    // The hardware output channel.
     int channel;
-    /* The raw allocated mixing buffer. */
+    // The raw allocated mixing buffer.
     Uint8 *rawbuf;
-    /* Individual mixing buffers. */
+    // Individual mixing buffers.
     Uint8 *mixbufs[NUM_BUFFERS];
-    /* Index of the next available mixing buffer. */
+    // Index of the next available mixing buffer.
     int next_buffer;
 };
 
-#endif /* SDL_pspaudio_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_pspaudio_h_
