@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -22,19 +22,14 @@
 #ifndef SDL_ime_h_
 #define SDL_ime_h_
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#include "SDL_stdinc.h"
-#include "SDL_rect.h"
-
-extern SDL_bool SDL_IME_Init(void);
+extern bool SDL_IME_Init(void);
 extern void SDL_IME_Quit(void);
-extern void SDL_IME_SetFocus(SDL_bool focused);
+extern void SDL_IME_SetFocus(bool focused);
 extern void SDL_IME_Reset(void);
-extern SDL_bool SDL_IME_ProcessKeyEvent(Uint32 keysym, Uint32 keycode, Uint8 state);
-extern void SDL_IME_UpdateTextRect(const SDL_Rect *rect);
+extern bool SDL_IME_ProcessKeyEvent(Uint32 keysym, Uint32 keycode, bool down);
+extern void SDL_IME_UpdateTextInputArea(SDL_Window *window);
 extern void SDL_IME_PumpEvents(void);
 
-#endif /* SDL_ime_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_ime_h_
