@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,8 +19,7 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "../../SDL_internal.h"
-#include "SDL_events.h"
+#include "SDL_internal.h"
 #include <sys/time.h>
 #include <dev/wscons/wsconsio.h>
 #include <unistd.h>
@@ -33,90 +32,95 @@
 typedef struct SDL_WSCONS_mouse_input_data
 {
     int fd;
+    SDL_MouseID mouseID;
 } SDL_WSCONS_mouse_input_data;
 
-SDL_WSCONS_mouse_input_data *SDL_WSCONS_Init_Mouse()
+SDL_WSCONS_mouse_input_data *SDL_WSCONS_Init_Mouse(void)
 {
 #ifdef WSMOUSEIO_SETVERSION
     int version = WSMOUSE_EVENT_VERSION;
 #endif
-    SDL_WSCONS_mouse_input_data *mouseInputData = SDL_calloc(1, sizeof(SDL_WSCONS_mouse_input_data));
+    SDL_WSCONS_mouse_input_data *input = SDL_calloc(1, sizeof(SDL_WSCONS_mouse_input_data));
 
-    if (mouseInputData == NULL) {
+    if (!input) {
         return NULL;
     }
-    mouseInputData->fd = open("/dev/wsmouse", O_RDWR | O_NONBLOCK | O_CLOEXEC);
-    if (mouseInputData->fd == -1) {
-        free(mouseInputData);
+    input->fd = open("/dev/wsmouse", O_RDWR | O_NONBLOCK | O_CLOEXEC);
+    if (input->fd == -1) {
+        SDL_free(input);
         return NULL;
     }
+
+    input->mouseID = SDL_GetNextObjectID();
+    SDL_AddMouse(input->mouseID, NULL, false);
+
 #ifdef WSMOUSEIO_SETMODE
-    ioctl(mouseInputData->fd, WSMOUSEIO_SETMODE, WSMOUSE_COMPAT);
+    ioctl(input->fd, WSMOUSEIO_SETMODE, WSMOUSE_COMPAT);
 #endif
 #ifdef WSMOUSEIO_SETVERSION
-    ioctl(mouseInputData->fd, WSMOUSEIO_SETVERSION, &version);
+    ioctl(input->fd, WSMOUSEIO_SETVERSION, &version);
 #endif
-    return mouseInputData;
+    return input;
 }
 
-void updateMouse(SDL_WSCONS_mouse_input_data *inputData)
+void updateMouse(SDL_WSCONS_mouse_input_data *input)
 {
     struct wscons_event events[64];
-    int type;
-    int n, i;
+    int n;
     SDL_Mouse *mouse = SDL_GetMouse();
 
-    if ((n = read(inputData->fd, events, sizeof(events))) > 0) {
+    if ((n = read(input->fd, events, sizeof(events))) > 0) {
+        int i;
         n /= sizeof(struct wscons_event);
         for (i = 0; i < n; i++) {
-            type = events[i].type;
+            int type = events[i].type;
             switch (type) {
             case WSCONS_EVENT_MOUSE_DOWN:
             {
                 switch (events[i].value) {
-                case 0: /* Left Mouse Button. */
-                    SDL_SendMouseButton(mouse->focus, mouse->mouseID, SDL_PRESSED, SDL_BUTTON_LEFT);
+                case 0: // Left Mouse Button.
+                    SDL_SendMouseButton(0, mouse->focus, input->mouseID, SDL_BUTTON_LEFT, true);
                     break;
-                case 1: /* Middle Mouse Button. */
-                    SDL_SendMouseButton(mouse->focus, mouse->mouseID, SDL_PRESSED, SDL_BUTTON_MIDDLE);
+                case 1: // Middle Mouse Button.
+                    SDL_SendMouseButton(0, mouse->focus, input->mouseID, SDL_BUTTON_MIDDLE, true);
                     break;
-                case 2: /* Right Mouse Button. */
-                    SDL_SendMouseButton(mouse->focus, mouse->mouseID, SDL_PRESSED, SDL_BUTTON_RIGHT);
+                case 2: // Right Mouse Button.
+                    SDL_SendMouseButton(0, mouse->focus, input->mouseID, SDL_BUTTON_RIGHT, true);
                     break;
                 }
             } break;
             case WSCONS_EVENT_MOUSE_UP:
             {
                 switch (events[i].value) {
-                case 0: /* Left Mouse Button. */
-                    SDL_SendMouseButton(mouse->focus, mouse->mouseID, SDL_RELEASED, SDL_BUTTON_LEFT);
+                case 0: // Left Mouse Button.
+                    SDL_SendMouseButton(0, mouse->focus, input->mouseID, SDL_BUTTON_LEFT, false);
                     break;
-                case 1: /* Middle Mouse Button. */
-                    SDL_SendMouseButton(mouse->focus, mouse->mouseID, SDL_RELEASED, SDL_BUTTON_MIDDLE);
+                case 1: // Middle Mouse Button.
+                    SDL_SendMouseButton(0, mouse->focus, input->mouseID, SDL_BUTTON_MIDDLE, false);
                     break;
-                case 2: /* Right Mouse Button. */
-                    SDL_SendMouseButton(mouse->focus, mouse->mouseID, SDL_RELEASED, SDL_BUTTON_RIGHT);
+                case 2: // Right Mouse Button.
+                    SDL_SendMouseButton(0, mouse->focus, input->mouseID, SDL_BUTTON_RIGHT, false);
                     break;
                 }
             } break;
             case WSCONS_EVENT_MOUSE_DELTA_X:
             {
-                SDL_SendMouseMotion(mouse->focus, mouse->mouseID, 1, events[i].value, 0);
+                SDL_SendMouseMotion(0, mouse->focus, input->mouseID, 1, (float)events[i].value, 0.0f);
                 break;
             }
             case WSCONS_EVENT_MOUSE_DELTA_Y:
             {
-                SDL_SendMouseMotion(mouse->focus, mouse->mouseID, 1, 0, -events[i].value);
+                SDL_SendMouseMotion(0, mouse->focus, input->mouseID, 1, 0.0f, -(float)events[i].value);
                 break;
             }
             case WSCONS_EVENT_MOUSE_DELTA_W:
             {
-                SDL_SendMouseWheel(mouse->focus, mouse->mouseID, events[i].value, 0, SDL_MOUSEWHEEL_NORMAL);
+                SDL_SendMouseWheel(0, mouse->focus, input->mouseID, events[i].value, 0, SDL_MOUSEWHEEL_NORMAL);
                 break;
             }
             case WSCONS_EVENT_MOUSE_DELTA_Z:
             {
-                SDL_SendMouseWheel(mouse->focus, mouse->mouseID, 0, -events[i].value, SDL_MOUSEWHEEL_NORMAL);
+                SDL_SendMouseWheel(0, mouse->focus, input->mouseID, 0, -events[i].value, SDL_MOUSEWHEEL_NORMAL);
                 break;
             }
             }
@@ -124,11 +128,11 @@ void updateMouse(SDL_WSCONS_mouse_input_data *inputData)
     }
 }
 
-void SDL_WSCONS_Quit_Mouse(SDL_WSCONS_mouse_input_data *inputData)
+void SDL_WSCONS_Quit_Mouse(SDL_WSCONS_mouse_input_data *input)
 {
-    if (inputData == NULL) {
+    if (!input) {
         return;
     }
-    close(inputData->fd);
-    free(inputData);
+    close(input->fd);
+    SDL_free(input);
 }
