@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,25 +19,28 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "SDL_config.h"
+#include "SDL_build_config.h"
 #include "SDL_dynapi.h"
+#include "SDL_dynapi_unsupported.h"
 
 #if SDL_DYNAMIC_API
 
-#define SDL_DYNAMIC_API_ENVVAR "SDL_DYNAMIC_API"
+#define SDL_DYNAMIC_API_ENVVAR "SDL3_DYNAMIC_API"
+#define SDL_SLOW_MEMCPY
+#define SDL_SLOW_MEMMOVE
+#define SDL_SLOW_MEMSET
 
-#if defined(__OS2__)
-#define INCL_DOS
-#define INCL_DOSERRORS
-#include <os2.h>
-#include <dos.h>
+#ifdef HAVE_STDIO_H
+#include <stdio.h>
 #endif
 
-#include "SDL.h"
+#include <SDL3/SDL.h>
+#define SDL_MAIN_NOIMPL // don't drag in header-only implementation of SDL_main
+#include <SDL3/SDL_main.h>
 
-/* These headers have system specific definitions, so aren't included above */
-#include "SDL_syswm.h"
-#include "SDL_vulkan.h"
+
+// These headers have system specific definitions, so aren't included above
+#include <SDL3/SDL_vulkan.h>
 
 /* This is the version of the dynamic API. This doesn't match the SDL version
    and should not change until there's been a major revamp in API/ABI.
@@ -48,7 +51,7 @@
    we'll forget to bump every time we add a function, so this is the
    failsafe switch for major API change decisions. Respect it and use it
    sparingly. */
-#define SDL_DYNAPI_VERSION 1
+#define SDL_DYNAPI_VERSION 2
 
 #ifdef __cplusplus
 extern "C" {
@@ -57,15 +60,11 @@ extern "C" {
 static void SDL_InitDynamicAPI(void);
 
 /* BE CAREFUL CALLING ANY SDL CODE IN HERE, IT WILL BLOW UP.
-   Even self-contained stuff might call SDL_Error and break everything. */
+   Even self-contained stuff might call SDL_SetError() and break everything. */
 
-/* behold, the macro salsa! */
+// behold, the macro salsa!
 
-/* !!! FIXME: ...disabled...until we write it.  :) */
-#define DISABLE_JUMP_MAGIC 1
-
-#if DISABLE_JUMP_MAGIC
-/* Can't use the macro for varargs nonsense. This is atrocious. */
+// Can't use the macro for varargs nonsense. This is atrocious.
 #define SDL_DYNAPI_VARARGS_LOGFN(_static, name, initcall, logname, prio)                                     \
     _static void SDLCALL SDL_Log##logname##name(int category, SDL_PRINTF_FORMAT_STRING const char *fmt, ...) \
     {                                                                                                        \
@@ -77,7 +76,7 @@ static void SDL_InitDynamicAPI(void);
     }
 
 #define SDL_DYNAPI_VARARGS(_static, name, initcall)                                                                                       \
-    _static int SDLCALL SDL_SetError##name(SDL_PRINTF_FORMAT_STRING const char *fmt, ...)                                                 \
+    _static bool SDLCALL SDL_SetError##name(SDL_PRINTF_FORMAT_STRING const char *fmt, ...)                                            \
     {                                                                                                                                     \
         char buf[128], *str = buf;                                                                                                        \
         int result;                                                                                                                       \
@@ -96,42 +95,62 @@ static void SDL_InitDynamicAPI(void);
             }                                                                                                                             \
         }                                                                                                                                 \
         if (result >= 0) {                                                                                                                \
-            result = jump_table.SDL_SetError("%s", str);                                                                                  \
+            jump_table.SDL_SetError("%s", str);                                                                                           \
         }                                                                                                                                 \
         if (str != buf) {                                                                                                                 \
             jump_table.SDL_free(str);                                                                                                     \
         }                                                                                                                                 \
-        return result;                                                                                                                    \
+        return false;                                                                                                                 \
     }                                                                                                                                     \
     _static int SDLCALL SDL_sscanf##name(const char *buf, SDL_SCANF_FORMAT_STRING const char *fmt, ...)                                   \
     {                                                                                                                                     \
-        int retval;                                                                                                                       \
+        int result;                                                                                                                       \
         va_list ap;                                                                                                                       \
         initcall;                                                                                                                         \
         va_start(ap, fmt);                                                                                                                \
-        retval = jump_table.SDL_vsscanf(buf, fmt, ap);                                                                                    \
+        result = jump_table.SDL_vsscanf(buf, fmt, ap);                                                                                    \
         va_end(ap);                                                                                                                       \
-        return retval;                                                                                                                    \
+        return result;                                                                                                                    \
     }                                                                                                                                     \
     _static int SDLCALL SDL_snprintf##name(SDL_OUT_Z_CAP(maxlen) char *buf, size_t maxlen, SDL_PRINTF_FORMAT_STRING const char *fmt, ...) \
     {                                                                                                                                     \
-        int retval;                                                                                                                       \
+        int result;                                                                                                                       \
         va_list ap;                                                                                                                       \
         initcall;                                                                                                                         \
         va_start(ap, fmt);                                                                                                                \
-        retval = jump_table.SDL_vsnprintf(buf, maxlen, fmt, ap);                                                                          \
+        result = jump_table.SDL_vsnprintf(buf, maxlen, fmt, ap);                                                                          \
         va_end(ap);                                                                                                                       \
-        return retval;                                                                                                                    \
+        return result;                                                                                                                    \
+    }                                                                                                                                     \
+    _static int SDLCALL SDL_swprintf##name(SDL_OUT_Z_CAP(maxlen) wchar_t *buf, size_t maxlen, SDL_PRINTF_FORMAT_STRING const wchar_t *fmt, ...) \
+    {                                                                                                                                     \
+        int result;                                                                                                                       \
+        va_list ap;                                                                                                                       \
+        initcall;                                                                                                                         \
+        va_start(ap, fmt);                                                                                                                \
+        result = jump_table.SDL_vswprintf(buf, maxlen, fmt, ap);                                                                          \
+        va_end(ap);                                                                                                                       \
+        return result;                                                                                                                    \
     }                                                                                                                                     \
     _static int SDLCALL SDL_asprintf##name(char **strp, SDL_PRINTF_FORMAT_STRING const char *fmt, ...)                                    \
     {                                                                                                                                     \
-        int retval;                                                                                                                       \
+        int result;                                                                                                                       \
         va_list ap;                                                                                                                       \
         initcall;                                                                                                                         \
         va_start(ap, fmt);                                                                                                                \
-        retval = jump_table.SDL_vasprintf(strp, fmt, ap);                                                                                 \
+        result = jump_table.SDL_vasprintf(strp, fmt, ap);                                                                                 \
         va_end(ap);                                                                                                                       \
-        return retval;                                                                                                                    \
+        return result;                                                                                                                    \
+    }                                                                                                                                     \
+    _static size_t SDLCALL SDL_IOprintf##name(SDL_IOStream *context, SDL_PRINTF_FORMAT_STRING const char *fmt, ...)                          \
+    {                                                                                                                                     \
+        size_t result;                                                                                                                    \
+        va_list ap;                                                                                                                       \
+        initcall;                                                                                                                         \
+        va_start(ap, fmt);                                                                                                                \
+        result = jump_table.SDL_IOvprintf(context, fmt, ap);                                                                              \
+        va_end(ap);                                                                                                                       \
+        return result;                                                                                                                    \
     }                                                                                                                                     \
     _static void SDLCALL SDL_Log##name(SDL_PRINTF_FORMAT_STRING const char *fmt, ...)                                                     \
     {                                                                                                                                     \
@@ -149,17 +168,17 @@ static void SDL_InitDynamicAPI(void);
         jump_table.SDL_LogMessageV(category, priority, fmt, ap);                                                                          \
         va_end(ap);                                                                                                                       \
     }                                                                                                                                     \
+    SDL_DYNAPI_VARARGS_LOGFN(_static, name, initcall, Trace, TRACE)                                                                   \
     SDL_DYNAPI_VARARGS_LOGFN(_static, name, initcall, Verbose, VERBOSE)                                                                   \
     SDL_DYNAPI_VARARGS_LOGFN(_static, name, initcall, Debug, DEBUG)                                                                       \
     SDL_DYNAPI_VARARGS_LOGFN(_static, name, initcall, Info, INFO)                                                                         \
     SDL_DYNAPI_VARARGS_LOGFN(_static, name, initcall, Warn, WARN)                                                                         \
     SDL_DYNAPI_VARARGS_LOGFN(_static, name, initcall, Error, ERROR)                                                                       \
     SDL_DYNAPI_VARARGS_LOGFN(_static, name, initcall, Critical, CRITICAL)
-#endif
 
-/* Typedefs for function pointers for jump table, and predeclare funcs */
-/* The DEFAULT funcs will init jump table and then call real function. */
-/* The REAL funcs are the actual functions, name-mangled to not clash. */
+// Typedefs for function pointers for jump table, and predeclare funcs
+// The DEFAULT funcs will init jump table and then call real function.
+// The REAL funcs are the actual functions, name-mangled to not clash.
 #define SDL_DYNAPI_PROC(rc, fn, params, args, ret) \
     typedef rc (SDLCALL *SDL_DYNAPIFN_##fn) params;\
     static rc SDLCALL fn##_DEFAULT params;         \
@@ -167,7 +186,7 @@ static void SDL_InitDynamicAPI(void);
 #include "SDL_dynapi_procs.h"
 #undef SDL_DYNAPI_PROC
 
-/* The jump table! */
+// The jump table!
 typedef struct
 {
 #define SDL_DYNAPI_PROC(rc, fn, params, args, ret) SDL_DYNAPIFN_##fn fn;
@@ -175,60 +194,45 @@ typedef struct
 #undef SDL_DYNAPI_PROC
 } SDL_DYNAPI_jump_table;
 
-/* Predeclare the default functions for initializing the jump table. */
-#define SDL_DYNAPI_PROC(rc, fn, params, args, ret) static rc SDLCALL fn##_DEFAULT params;
-#include "SDL_dynapi_procs.h"
-#undef SDL_DYNAPI_PROC
-
-/* The actual jump table. */
+// The actual jump table.
 static SDL_DYNAPI_jump_table jump_table = {
 #define SDL_DYNAPI_PROC(rc, fn, params, args, ret) fn##_DEFAULT,
 #include "SDL_dynapi_procs.h"
 #undef SDL_DYNAPI_PROC
 };
 
-/* Default functions init the function table then call right thing. */
-#if DISABLE_JUMP_MAGIC
+// Default functions init the function table then call right thing.
 #define SDL_DYNAPI_PROC(rc, fn, params, args, ret) \
     static rc SDLCALL fn##_DEFAULT params          \
     {                                              \
         SDL_InitDynamicAPI();                      \
         ret jump_table.fn args;                    \
     }
-#define SDL_DYNAPI_PROC_NO_VARARGS
+#define SDL_DYNAPI_PROC_NO_VARARGS 1
 #include "SDL_dynapi_procs.h"
 #undef SDL_DYNAPI_PROC
 #undef SDL_DYNAPI_PROC_NO_VARARGS
 SDL_DYNAPI_VARARGS(static, _DEFAULT, SDL_InitDynamicAPI())
-#else
-/* !!! FIXME: need the jump magic. */
-#error Write me.
-#endif
 
-/* Public API functions to jump into the jump table. */
-#if DISABLE_JUMP_MAGIC
+// Public API functions to jump into the jump table.
 #define SDL_DYNAPI_PROC(rc, fn, params, args, ret) \
     rc SDLCALL fn params                           \
     {                                              \
         ret jump_table.fn args;                    \
     }
-#define SDL_DYNAPI_PROC_NO_VARARGS
+#define SDL_DYNAPI_PROC_NO_VARARGS 1
 #include "SDL_dynapi_procs.h"
 #undef SDL_DYNAPI_PROC
 #undef SDL_DYNAPI_PROC_NO_VARARGS
 SDL_DYNAPI_VARARGS(, , )
-#else
-/* !!! FIXME: need the jump magic. */
-#error Write me.
-#endif
 
 #define ENABLE_SDL_CALL_LOGGING 0
 #if ENABLE_SDL_CALL_LOGGING
-static int SDLCALL SDL_SetError_LOGSDLCALLS(SDL_PRINTF_FORMAT_STRING const char *fmt, ...)
+static bool SDLCALL SDL_SetError_LOGSDLCALLS(SDL_PRINTF_FORMAT_STRING const char *fmt, ...)
 {
-    char buf[512]; /* !!! FIXME: dynamic allocation */
+    char buf[512]; // !!! FIXME: dynamic allocation
     va_list ap;
-    SDL_Log_REAL("SDL2CALL SDL_SetError");
+    SDL_Log_REAL("SDL3CALL SDL_SetError");
     va_start(ap, fmt);
     SDL_vsnprintf_REAL(buf, sizeof(buf), fmt, ap);
     va_end(ap);
@@ -236,38 +240,58 @@ static int SDLCALL SDL_SetError_LOGSDLCALLS(SDL_PRINTF_FORMAT_STRING const char
 }
 static int SDLCALL SDL_sscanf_LOGSDLCALLS(const char *buf, SDL_SCANF_FORMAT_STRING const char *fmt, ...)
 {
-    int retval;
+    int result;
     va_list ap;
-    SDL_Log_REAL("SDL2CALL SDL_sscanf");
+    SDL_Log_REAL("SDL3CALL SDL_sscanf");
     va_start(ap, fmt);
-    retval = SDL_vsscanf_REAL(buf, fmt, ap);
+    result = SDL_vsscanf_REAL(buf, fmt, ap);
     va_end(ap);
-    return retval;
+    return result;
 }
 static int SDLCALL SDL_snprintf_LOGSDLCALLS(SDL_OUT_Z_CAP(maxlen) char *buf, size_t maxlen, SDL_PRINTF_FORMAT_STRING const char *fmt, ...)
 {
-    int retval;
+    int result;
     va_list ap;
-    SDL_Log_REAL("SDL2CALL SDL_snprintf");
+    SDL_Log_REAL("SDL3CALL SDL_snprintf");
     va_start(ap, fmt);
-    retval = SDL_vsnprintf_REAL(buf, maxlen, fmt, ap);
+    result = SDL_vsnprintf_REAL(buf, maxlen, fmt, ap);
     va_end(ap);
-    return retval;
+    return result;
 }
 static int SDLCALL SDL_asprintf_LOGSDLCALLS(char **strp, SDL_PRINTF_FORMAT_STRING const char *fmt, ...)
 {
-    int retval;
+    int result;
     va_list ap;
-    SDL_Log_REAL("SDL2CALL SDL_asprintf");
+    SDL_Log_REAL("SDL3CALL SDL_asprintf");
     va_start(ap, fmt);
-    retval = SDL_vasprintf_REAL(strp, fmt, ap);
+    result = SDL_vasprintf_REAL(strp, fmt, ap);
     va_end(ap);
-    return retval;
+    return result;
+}
+static int SDLCALL SDL_swprintf_LOGSDLCALLS(SDL_OUT_Z_CAP(maxlen) wchar_t *buf, size_t maxlen, SDL_PRINTF_FORMAT_STRING const wchar_t *fmt, ...)
+{
+    int result;
+    va_list ap;
+    SDL_Log_REAL("SDL3CALL SDL_swprintf");
+    va_start(ap, fmt);
+    result = SDL_vswprintf_REAL(buf, maxlen, fmt, ap);
+    va_end(ap);
+    return result;
+}
+static size_t SDLCALL SDL_IOprintf_LOGSDLCALLS(SDL_IOStream *context, SDL_PRINTF_FORMAT_STRING const char *fmt, ...)
+{
+    size_t result;
+    va_list ap;
+    SDL_Log_REAL("SDL3CALL SDL_IOprintf");
+    va_start(ap, fmt);
+    result = SDL_IOvprintf_REAL(context, fmt, ap);
+    va_end(ap);
+    return result;
 }
 static void SDLCALL SDL_Log_LOGSDLCALLS(SDL_PRINTF_FORMAT_STRING const char *fmt, ...)
 {
     va_list ap;
-    SDL_Log_REAL("SDL2CALL SDL_Log");
+    SDL_Log_REAL("SDL3CALL SDL_Log");
     va_start(ap, fmt);
     SDL_LogMessageV_REAL(SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_PRIORITY_INFO, fmt, ap);
     va_end(ap);
@@ -275,27 +299,34 @@ static void SDLCALL SDL_Log_LOGSDLCALLS(SDL_PRINTF_FORMAT_STRING const char *fmt
 static void SDLCALL SDL_LogMessage_LOGSDLCALLS(int category, SDL_LogPriority priority, SDL_PRINTF_FORMAT_STRING const char *fmt, ...)
 {
     va_list ap;
-    SDL_Log_REAL("SDL2CALL SDL_LogMessage");
+    SDL_Log_REAL("SDL3CALL SDL_LogMessage");
     va_start(ap, fmt);
     SDL_LogMessageV_REAL(category, priority, fmt, ap);
     va_end(ap);
 }
-#define SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(logname, prio) \
-    static void SDLCALL SDL_Log##logname##_LOGSDLCALLS(int category, SDL_PRINTF_FORMAT_STRING const char *fmt, ...) { \
-        va_list ap; va_start(ap, fmt); \
-        SDL_Log_REAL("SDL2CALL SDL_Log%s", #logname); \
-        SDL_LogMessageV_REAL(category, SDL_LOG_PRIORITY_##prio, fmt, ap); \
-        va_end(ap); \
+#define SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(logname, prio)                                                         \
+    static void SDLCALL SDL_Log##logname##_LOGSDLCALLS(int category, SDL_PRINTF_FORMAT_STRING const char *fmt, ...) \
+    {                                                                                                               \
+        va_list ap;                                                                                                 \
+        va_start(ap, fmt);                                                                                          \
+        SDL_Log_REAL("SDL3CALL SDL_Log%s", #logname);                                                               \
+        SDL_LogMessageV_REAL(category, SDL_LOG_PRIORITY_##prio, fmt, ap);                                           \
+        va_end(ap);                                                                                                 \
     }
+SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(Trace, TRACE)
 SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(Verbose, VERBOSE)
 SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(Debug, DEBUG)
 SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(Info, INFO)
 SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(Warn, WARN)
 SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(Error, ERROR)
 SDL_DYNAPI_VARARGS_LOGFN_LOGSDLCALLS(Critical, CRITICAL)
-#define SDL_DYNAPI_PROC(rc,fn,params,args,ret) \
-    rc SDLCALL fn##_LOGSDLCALLS params { SDL_Log_REAL("SDL2CALL %s", #fn); ret fn##_REAL args; }
-#define SDL_DYNAPI_PROC_NO_VARARGS
+#define SDL_DYNAPI_PROC(rc, fn, params, args, ret) \
+    rc SDLCALL fn##_LOGSDLCALLS params             \
+    {                                              \
+        SDL_Log_REAL("SDL3CALL %s", #fn);          \
+        ret fn##_REAL args;                        \
+    }
+#define SDL_DYNAPI_PROC_NO_VARARGS 1
 #include "SDL_dynapi_procs.h"
 #undef SDL_DYNAPI_PROC
 #undef SDL_DYNAPI_PROC_NO_VARARGS
@@ -308,17 +339,17 @@ static Sint32 initialize_jumptable(Uint32 apiver, void *table, Uint32 tablesize)
     SDL_DYNAPI_jump_table *output_jump_table = (SDL_DYNAPI_jump_table *)table;
 
     if (apiver != SDL_DYNAPI_VERSION) {
-        /* !!! FIXME: can maybe handle older versions? */
-        return -1; /* not compatible. */
+        // !!! FIXME: can maybe handle older versions?
+        return -1; // not compatible.
     } else if (tablesize > sizeof(jump_table)) {
-        return -1; /* newer version of SDL with functions we can't provide. */
+        return -1; // newer version of SDL with functions we can't provide.
     }
 
-/* Init our jump table first. */
+// Init our jump table first.
 #if ENABLE_SDL_CALL_LOGGING
     {
-        const char *env = SDL_getenv_REAL("SDL_DYNAPI_LOG_CALLS");
-        const SDL_bool log_calls = (env && SDL_atoi_REAL(env));
+        const char *env = SDL_getenv_unsafe_REAL("SDL_DYNAPI_LOG_CALLS");
+        const bool log_calls = (env && SDL_atoi_REAL(env));
         if (log_calls) {
 #define SDL_DYNAPI_PROC(rc, fn, params, args, ret) jump_table.fn = fn##_LOGSDLCALLS;
 #include "SDL_dynapi_procs.h"
@@ -335,20 +366,20 @@ static Sint32 initialize_jumptable(Uint32 apiver, void *table, Uint32 tablesize)
 #undef SDL_DYNAPI_PROC
 #endif
 
-    /* Then the external table... */
+    // Then the external table...
     if (output_jump_table != &jump_table) {
         jump_table.SDL_memcpy(output_jump_table, &jump_table, tablesize);
     }
 
-    /* Safe to call SDL functions now; jump table is initialized! */
+    // Safe to call SDL functions now; jump table is initialized!
 
-    return 0; /* success! */
+    return 0; // success!
 }
 
-/* Here's the exported entry point that fills in the jump table. */
-/*  Use specific types when an "int" might suffice to keep this sane. */
+// Here's the exported entry point that fills in the jump table.
+// Use specific types when an "int" might suffice to keep this sane.
 typedef Sint32 (SDLCALL *SDL_DYNAPI_ENTRYFN)(Uint32 apiver, void *table, Uint32 tablesize);
-extern DECLSPEC Sint32 SDLCALL SDL_DYNAPI_entry(Uint32, void *, Uint32);
+extern SDL_DECLSPEC Sint32 SDLCALL SDL_DYNAPI_entry(Uint32, void *, Uint32);
 
 Sint32 SDL_DYNAPI_entry(Uint32 apiver, void *table, Uint32 tablesize)
 {
@@ -359,9 +390,9 @@ Sint32 SDL_DYNAPI_entry(Uint32 apiver, void *table, Uint32 tablesize)
 }
 #endif
 
-/* Obviously we can't use SDL_LoadObject() to load SDL.  :)  */
-/* Also obviously, we never close the loaded library. */
-#if defined(WIN32) || defined(_WIN32) || defined(__CYGWIN__)
+// Obviously we can't use SDL_LoadObject() to load SDL.  :)
+// Also obviously, we never close the loaded library.
+#if defined(WIN32) || defined(_WIN32) || defined(SDL_PLATFORM_CYGWIN)
 #ifndef WIN32_LEAN_AND_MEAN
 #define WIN32_LEAN_AND_MEAN 1
 #endif
@@ -369,43 +400,29 @@ Sint32 SDL_DYNAPI_entry(Uint32 apiver, void *table, Uint32 tablesize)
 static SDL_INLINE void *get_sdlapi_entry(const char *fname, const char *sym)
 {
     HMODULE lib = LoadLibraryA(fname);
-    void *retval = NULL;
+    void *result = NULL;
     if (lib) {
-        retval = (void *) GetProcAddress(lib, sym);
-        if (!retval) {
+        result = (void *) GetProcAddress(lib, sym);
+        if (!result) {
             FreeLibrary(lib);
         }
     }
-    return retval;
+    return result;
 }
 
-#elif defined(unix) || defined(__unix__) || defined(__APPLE__) || defined(__HAIKU__) || defined(__QNX__)
+#elif defined(SDL_PLATFORM_UNIX) || defined(SDL_PLATFORM_APPLE) || defined(SDL_PLATFORM_HAIKU)
 #include <dlfcn.h>
 static SDL_INLINE void *get_sdlapi_entry(const char *fname, const char *sym)
 {
     void *lib = dlopen(fname, RTLD_NOW | RTLD_LOCAL);
-    void *retval = NULL;
+    void *result = NULL;
     if (lib) {
-        retval = dlsym(lib, sym);
-        if (!retval) {
+        result = dlsym(lib, sym);
+        if (!result) {
             dlclose(lib);
         }
     }
-    return retval;
-}
-
-#elif defined(__OS2__)
-static SDL_INLINE void *get_sdlapi_entry(const char *fname, const char *sym)
-{
-    HMODULE hmodule;
-    PFN retval = NULL;
-    char error[256];
-    if (DosLoadModule(error, sizeof(error), fname, &hmodule) == NO_ERROR) {
-        if (DosQueryProcAddr(hmodule, 0, sym, &retval) != NO_ERROR) {
-            DosFreeModule(hmodule);
-        }
-    }
-    return (void *)retval;
+    return result;
 }
 
 #else
@@ -415,8 +432,9 @@ static SDL_INLINE void *get_sdlapi_entry(const char *fname, const char *sym)
 static void dynapi_warn(const char *msg)
 {
     const char *caption = "SDL Dynamic API Failure!";
-/* SDL_ShowSimpleMessageBox() is a too heavy for here. */
-#if (defined(WIN32) || defined(_WIN32) || defined(__CYGWIN__)) && !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
+    (void)caption;
+// SDL_ShowSimpleMessageBox() is a too heavy for here.
+#if (defined(WIN32) || defined(_WIN32) || defined(SDL_PLATFORM_CYGWIN)) && !defined(SDL_PLATFORM_XBOXONE) && !defined(SDL_PLATFORM_XBOXSERIES)
     MessageBoxA(NULL, msg, caption, MB_OK | MB_ICONERROR);
 #elif defined(HAVE_STDIO_H)
     fprintf(stderr, "\n\n%s\n%s\n\n", caption, msg);
@@ -431,7 +449,7 @@ static void dynapi_warn(const char *msg)
 extern "C" {
 #endif
 extern SDL_NORETURN void SDL_ExitProcess(int exitcode);
-#if defined(__WATCOMC__)
+#ifdef __WATCOMC__
 #pragma aux SDL_ExitProcess aborts;
 #endif
 #ifdef __cplusplus
@@ -440,24 +458,16 @@ extern SDL_NORETURN void SDL_ExitProcess(int exitcode);
 
 static void SDL_InitDynamicAPILocked(void)
 {
-    /* this can't use SDL_getenv_REAL, because it might allocate memory before the app can set their allocator */
-#if defined(WIN32) || defined(_WIN32) || defined(__CYGWIN__)
-    /* We've always used LoadLibraryA for this, so this has never worked with Unicode paths on Windows. Sorry. */
-    char envbuf[512];  /* overflows will just report as environment variable being unset, but LoadLibraryA has a MAX_PATH of 260 anyhow, apparently. */
-    const DWORD rc = GetEnvironmentVariableA(SDL_DYNAMIC_API_ENVVAR, envbuf, (DWORD) sizeof (envbuf));
-    char *libname = ((rc != 0) && (rc < sizeof (envbuf))) ? envbuf : NULL;
-#else
-    char *libname = getenv(SDL_DYNAMIC_API_ENVVAR);
-#endif
-
-    SDL_DYNAPI_ENTRYFN entry = NULL; /* funcs from here by default. */
-    SDL_bool use_internal = SDL_TRUE;
+    const char *libname = SDL_getenv_unsafe_REAL(SDL_DYNAMIC_API_ENVVAR);
+    SDL_DYNAPI_ENTRYFN entry = NULL; // funcs from here by default.
+    bool use_internal = true;
 
     if (libname) {
         while (*libname && !entry) {
-            char *ptr = libname;
-            while (SDL_TRUE) {
-                const char ch = *ptr;
+            // This is evil, but we're not making any permanent changes...
+            char *ptr = (char *)libname;
+            while (true) {
+                char ch = *ptr;
                 if ((ch == ',') || (ch == '\0')) {
                     *ptr = '\0';
                     entry = (SDL_DYNAPI_ENTRYFN)get_sdlapi_entry(libname, "SDL_DYNAPI_entry");
@@ -471,29 +481,29 @@ static void SDL_InitDynamicAPILocked(void)
         }
         if (!entry) {
             dynapi_warn("Couldn't load an overriding SDL library. Please fix or remove the " SDL_DYNAMIC_API_ENVVAR " environment variable. Using the default SDL.");
-            /* Just fill in the function pointers from this library, later. */
+            // Just fill in the function pointers from this library, later.
         }
     }
 
     if (entry) {
         if (entry(SDL_DYNAPI_VERSION, &jump_table, sizeof(jump_table)) < 0) {
             dynapi_warn("Couldn't override SDL library. Using a newer SDL build might help. Please fix or remove the " SDL_DYNAMIC_API_ENVVAR " environment variable. Using the default SDL.");
-            /* Just fill in the function pointers from this library, later. */
+            // Just fill in the function pointers from this library, later.
         } else {
-            use_internal = SDL_FALSE; /* We overrode SDL! Don't use the internal version! */
+            use_internal = false; // We overrode SDL! Don't use the internal version!
         }
     }
 
-    /* Just fill in the function pointers from this library. */
+    // Just fill in the function pointers from this library.
     if (use_internal) {
         if (initialize_jumptable(SDL_DYNAPI_VERSION, &jump_table, sizeof(jump_table)) < 0) {
-            /* Now we're screwed. Should definitely abort now. */
+            // Now we're screwed. Should definitely abort now.
             dynapi_warn("Failed to initialize internal SDL dynapi. As this would otherwise crash, we have to abort now.");
             SDL_ExitProcess(86);
         }
     }
 
-    /* we intentionally never close the newly-loaded lib, of course. */
+    // we intentionally never close the newly-loaded lib, of course.
 }
 
 static void SDL_InitDynamicAPI(void)
@@ -509,26 +519,30 @@ static void SDL_InitDynamicAPI(void)
      *  SDL_CreateThread() would also call this function before building the
      *  new thread).
      */
-    static SDL_bool already_initialized = SDL_FALSE;
+    static bool already_initialized = false;
 
-    /* SDL_AtomicLock calls SDL mutex functions to emulate if
-       SDL_ATOMIC_DISABLED, which we can't do here, so in such a
-       configuration, you're on your own. */
-    #ifndef SDL_ATOMIC_DISABLED
     static SDL_SpinLock lock = 0;
-    SDL_AtomicLock_REAL(&lock);
-    #endif
+    SDL_LockSpinlock_REAL(&lock);
 
     if (!already_initialized) {
         SDL_InitDynamicAPILocked();
-        already_initialized = SDL_TRUE;
+        already_initialized = true;
     }
 
-    #ifndef SDL_ATOMIC_DISABLED
-    SDL_AtomicUnlock_REAL(&lock);
-    #endif
+    SDL_UnlockSpinlock_REAL(&lock);
 }
 
-#endif /* SDL_DYNAMIC_API */
+#else // SDL_DYNAMIC_API
 
-/* vi: set ts=4 sw=4 expandtab: */
+#include <SDL3/SDL.h>
+
+Sint32 SDL_DYNAPI_entry(Uint32 apiver, void *table, Uint32 tablesize);
+Sint32 SDL_DYNAPI_entry(Uint32 apiver, void *table, Uint32 tablesize)
+{
+    (void)apiver;
+    (void)table;
+    (void)tablesize;
+    return -1; // not compatible.
+}
+
+#endif // SDL_DYNAMIC_API
