@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,15 +18,14 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_dropevents_c_h_
 #define SDL_dropevents_c_h_
 
-extern int SDL_SendDropFile(SDL_Window *window, const char *file);
-extern int SDL_SendDropText(SDL_Window *window, const char *text);
-extern int SDL_SendDropComplete(SDL_Window *window);
+extern bool SDL_SendDropFile(SDL_Window *window, const char *source, const char *file);
+extern bool SDL_SendDropPosition(SDL_Window *window, float x, float y);
+extern bool SDL_SendDropText(SDL_Window *window, const char *text);
+extern bool SDL_SendDropComplete(SDL_Window *window);
 
-#endif /* SDL_dropevents_c_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_dropevents_c_h_
