@@ -18,10 +18,13 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../SDL_internal.h"
+#include "SDL_internal.h"
+
+
+/* Return true if the HIDAPI should ignore a device during enumeration */
+extern bool SDL_HIDAPI_ShouldIgnoreDevice(int bus_type, Uint16 vendor_id, Uint16 product_id, Uint16 usage_page, Uint16 usage);
 
 #ifdef SDL_JOYSTICK_HIDAPI
-
 #ifdef HAVE_LIBUSB
 #define HAVE_ENABLE_GAMECUBE_ADAPTORS
 #endif
@@ -29,7 +32,4 @@
 #ifdef HAVE_ENABLE_GAMECUBE_ADAPTORS
 extern void SDL_EnableGameCubeAdaptors(void);
 #endif
-
 #endif /* SDL_JOYSTICK_HIDAPI */
-
-/* vi: set sts=4 ts=4 sw=4 expandtab: */
