@@ -19,7 +19,7 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_JOYSTICK_ANDROID
 
@@ -28,32 +28,30 @@
 
 #include "../SDL_sysjoystick.h"
 
-extern int Android_OnPadDown(int device_id, int keycode);
-extern int Android_OnPadUp(int device_id, int keycode);
-extern int Android_OnJoy(int device_id, int axisnum, float value);
-extern int Android_OnHat(int device_id, int hat_id, int x, int y);
-extern int Android_AddJoystick(int device_id, const char *name, const char *desc, int vendor_id, int product_id, SDL_bool is_accelerometer, int button_mask, int naxes, int axis_mask, int nhats, int nballs);
-extern int Android_RemoveJoystick(int device_id);
+extern bool Android_OnPadDown(int device_id, int keycode);
+extern bool Android_OnPadUp(int device_id, int keycode);
+extern bool Android_OnJoy(int device_id, int axisnum, float value);
+extern bool Android_OnHat(int device_id, int hat_id, int x, int y);
+extern void Android_AddJoystick(int device_id, const char *name, const char *desc, int vendor_id, int product_id, int button_mask, int naxes, int axis_mask, int nhats, bool can_rumble);
+extern void Android_RemoveJoystick(int device_id);
 
-/* A linked list of available joysticks */
+// A linked list of available joysticks
 typedef struct SDL_joylist_item
 {
     int device_instance;
-    int device_id; /* Android's device id */
-    char *name;    /* "SideWinder 3D Pro" or whatever */
-    SDL_JoystickGUID guid;
-    SDL_bool is_accelerometer;
+    int device_id; // Android's device id
+    char *name;    // "SideWinder 3D Pro" or whatever
+    SDL_GUID guid;
     SDL_Joystick *joystick;
-    int nbuttons, naxes, nhats, nballs;
+    int nbuttons, naxes, nhats;
     int dpad_state;
+    bool can_rumble;
 
     struct SDL_joylist_item *next;
 } SDL_joylist_item;
 
 typedef SDL_joylist_item joystick_hwdata;
 
-#endif /* SDL_sysjoystick_c_h_ */
+#endif // SDL_sysjoystick_c_h_
 
-#endif /* SDL_JOYSTICK_ANDROID */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_JOYSTICK_ANDROID
