@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -22,7 +22,7 @@
 #ifndef usb_ids_h_
 #define usb_ids_h_
 
-/* Definitions of useful USB VID/PID values */
+// Definitions of useful USB VID/PID values
 
 #define USB_VENDOR_8BITDO       0x2dc8
 #define USB_VENDOR_AMAZON       0x1949
@@ -34,6 +34,7 @@
 #define USB_VENDOR_DRAGONRISE   0x0079
 #define USB_VENDOR_GOOGLE       0x18d1
 #define USB_VENDOR_HORI         0x0f0d
+#define USB_VENDOR_HP           0x03f0
 #define USB_VENDOR_HYPERKIN     0x2e24
 #define USB_VENDOR_LOGITECH     0x046d
 #define USB_VENDOR_MADCATZ      0x0738
@@ -58,8 +59,8 @@
 #define USB_VENDOR_VALVE        0x28de
 #define USB_VENDOR_ZEROPLUS     0x0c12
 
-#define USB_PRODUCT_8BITDO_XBOX_CONTROLLER1               0x2002 /* Ultimate Wired Controller for Xbox */
-#define USB_PRODUCT_8BITDO_XBOX_CONTROLLER2               0x3106 /* Ultimate Wireless / Pro 2 Wired Controller */
+#define USB_PRODUCT_8BITDO_XBOX_CONTROLLER1               0x2002 // Ultimate Wired Controller for Xbox
+#define USB_PRODUCT_8BITDO_XBOX_CONTROLLER2               0x3106 // Ultimate Wireless / Pro 2 Wired Controller
 #define USB_PRODUCT_AMAZON_LUNA_CONTROLLER                0x0419
 #define USB_PRODUCT_ASTRO_C40_XBOX360                     0x0024
 #define USB_PRODUCT_BACKBONE_ONE_IOS                      0x0103
@@ -72,6 +73,9 @@
 #define USB_PRODUCT_HORI_HORIPAD_PRO_SERIES_X             0x014f
 #define USB_PRODUCT_HORI_FIGHTING_STICK_ALPHA_PS4         0x011c
 #define USB_PRODUCT_HORI_FIGHTING_STICK_ALPHA_PS5         0x0184
+#define USB_PRODUCT_HORI_FIGHTING_STICK_ALPHA_PS5         0x0184
+#define USB_PRODUCT_HORI_STEAM_CONTROLLER                 0x01AB
+#define USB_PRODUCT_HORI_STEAM_CONTROLLER_BT              0x0196
 #define USB_PRODUCT_LOGITECH_F310                         0xc216
 #define USB_PRODUCT_LOGITECH_CHILLSTREAM                  0xcad1
 #define USB_PRODUCT_MADCATZ_SAITEK_SIDE_PANEL_CONTROL_DECK 0x2218
@@ -85,7 +89,7 @@
 #define USB_PRODUCT_NINTENDO_SNES_CONTROLLER              0x2017
 #define USB_PRODUCT_NINTENDO_SWITCH_JOYCON_GRIP           0x200e
 #define USB_PRODUCT_NINTENDO_SWITCH_JOYCON_LEFT           0x2006
-#define USB_PRODUCT_NINTENDO_SWITCH_JOYCON_PAIR           0x2008 /* Used by joycond */
+#define USB_PRODUCT_NINTENDO_SWITCH_JOYCON_PAIR           0x2008 // Used by joycond
 #define USB_PRODUCT_NINTENDO_SWITCH_JOYCON_RIGHT          0x2007
 #define USB_PRODUCT_NINTENDO_SWITCH_PRO                   0x2009
 #define USB_PRODUCT_NINTENDO_WII_REMOTE                   0x0306
@@ -123,9 +127,11 @@
 #define USB_PRODUCT_TURTLE_BEACH_SERIES_X_RECON           0x7009
 #define USB_PRODUCT_VICTRIX_FS_PRO                        0x0203
 #define USB_PRODUCT_VICTRIX_FS_PRO_V2                     0x0207
-#define USB_PRODUCT_XBOX360_XUSB_CONTROLLER               0x02a1 /* XUSB driver software PID */
+#define USB_PRODUCT_XBOX360_XUSB_CONTROLLER               0x02a1 // XUSB driver software PID
 #define USB_PRODUCT_XBOX360_WIRED_CONTROLLER              0x028e
 #define USB_PRODUCT_XBOX360_WIRELESS_RECEIVER             0x0719
+#define USB_PRODUCT_XBOX360_WIRELESS_RECEIVER_THIRDPARTY1 0x02a9
+#define USB_PRODUCT_XBOX360_WIRELESS_RECEIVER_THIRDPARTY2 0x0291
 #define USB_PRODUCT_XBOX_ONE_ADAPTIVE                     0x0b0a
 #define USB_PRODUCT_XBOX_ONE_ADAPTIVE_BLUETOOTH           0x0b0c
 #define USB_PRODUCT_XBOX_ONE_ADAPTIVE_BLE                 0x0b21
@@ -139,19 +145,22 @@
 #define USB_PRODUCT_XBOX_ONE_S_REV2_BLE                   0x0b20
 #define USB_PRODUCT_XBOX_SERIES_X                         0x0b12
 #define USB_PRODUCT_XBOX_SERIES_X_BLE                     0x0b13
-#define USB_PRODUCT_XBOX_SERIES_X_VICTRIX_GAMBIT          0x02d6
-#define USB_PRODUCT_XBOX_SERIES_X_PDP_BLUE                0x02d9
+#define USB_PRODUCT_XBOX_SERIES_X_HP_HYPERX               0x08b6
+#define USB_PRODUCT_XBOX_SERIES_X_HP_HYPERX_RGB           0x07a0
 #define USB_PRODUCT_XBOX_SERIES_X_PDP_AFTERGLOW           0x02da
+#define USB_PRODUCT_XBOX_SERIES_X_PDP_BLUE                0x02d9
 #define USB_PRODUCT_XBOX_SERIES_X_POWERA_FUSION_PRO2      0x4001
+#define USB_PRODUCT_XBOX_SERIES_X_POWERA_MOGA_XP_ULTRA    0x890b
 #define USB_PRODUCT_XBOX_SERIES_X_POWERA_SPECTRA          0x4002
-#define USB_PRODUCT_XBOX_ONE_XBOXGIP_CONTROLLER           0x02ff /* XBOXGIP driver software PID */
+#define USB_PRODUCT_XBOX_SERIES_X_VICTRIX_GAMBIT          0x02d6
+#define USB_PRODUCT_XBOX_ONE_XBOXGIP_CONTROLLER           0x02ff // XBOXGIP driver software PID
 #define USB_PRODUCT_STEAM_VIRTUAL_GAMEPAD                 0x11ff
 
-/* USB usage pages */
+// USB usage pages
 #define USB_USAGEPAGE_GENERIC_DESKTOP 0x0001
 #define USB_USAGEPAGE_BUTTON          0x0009
 
-/* USB usages for USAGE_PAGE_GENERIC_DESKTOP */
+// USB usages for USAGE_PAGE_GENERIC_DESKTOP
 #define USB_USAGE_GENERIC_POINTER             0x0001
 #define USB_USAGE_GENERIC_MOUSE               0x0002
 #define USB_USAGE_GENERIC_JOYSTICK            0x0004
@@ -176,6 +185,4 @@
 
 #define BLUETOOTH_PRODUCT_LUNA_CONTROLLER 0x0419
 
-#endif /* usb_ids_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // usb_ids_h_
