@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,20 +18,15 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 #include "../../core/windows/SDL_windows.h"
 
-/* Return true if the RawInput driver is enabled */
-extern SDL_bool RAWINPUT_IsEnabled();
+// Return true if the RawInput driver is enabled
+extern bool RAWINPUT_IsEnabled(void);
 
-/* Return true if a RawInput device is present and supported as a joystick */
-extern SDL_bool RAWINPUT_IsDevicePresent(Uint16 vendor_id, Uint16 product_id, Uint16 version, const char *name);
-
-/* Registers for input events */
+// Registers for input events
 extern int RAWINPUT_RegisterNotifications(HWND hWnd);
-extern int RAWINPUT_UnregisterNotifications();
+extern int RAWINPUT_UnregisterNotifications(void);
 
-/* Returns 0 if message was handled */
+// Returns 0 if message was handled
 extern LRESULT CALLBACK RAWINPUT_WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
-
-/* vi: set ts=4 sw=4 expandtab: */
