@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,30 +18,35 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#include "SDL_events.h"
 #include "../SDL_sysjoystick.h"
 #include "../../core/windows/SDL_windows.h"
 #include "../../core/windows/SDL_directx.h"
 
-#define MAX_INPUTS  256     /* each joystick can have up to 256 inputs */
+#define MAX_INPUTS 256 // each joystick can have up to 256 inputs
+
+// Set up for C function definitions, even when using C++
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 typedef struct JoyStick_DeviceData
 {
-    SDL_JoystickGUID guid;
+    SDL_GUID guid;
     char *joystickname;
     Uint8 send_add_event;
     SDL_JoystickID nInstanceID;
-    SDL_bool bXInputDevice;
+    bool bXInputDevice;
     BYTE SubType;
     Uint8 XInputUserId;
     DIDEVICEINSTANCE dxdevice;
-    char hidPath[MAX_PATH];
+    char path[MAX_PATH];
+    int steam_virtual_gamepad_slot;
     struct JoyStick_DeviceData *pNext;
 } JoyStick_DeviceData;
 
-extern JoyStick_DeviceData *SYS_Joystick;    /* array to hold joystick ID values */
+extern JoyStick_DeviceData *SYS_Joystick; // array to hold joystick ID values
 
 typedef enum Type
 {
@@ -52,43 +57,47 @@ typedef enum Type
 
 typedef struct input_t
 {
-    /* DirectInput offset for this input type: */
+    // DirectInput offset for this input type:
     DWORD ofs;
 
-    /* Button, axis or hat: */
+    // Button, axis or hat:
     Type type;
 
-    /* SDL input offset: */
+    // SDL input offset:
     Uint8 num;
 } input_t;
 
-/* The private structure used to keep track of a joystick */
+// The private structure used to keep track of a joystick
 struct joystick_hwdata
 {
-    SDL_JoystickGUID guid;
+    SDL_GUID guid;
 
-#if SDL_JOYSTICK_DINPUT
+#ifdef SDL_JOYSTICK_DINPUT
     LPDIRECTINPUTDEVICE8 InputDevice;
     DIDEVCAPS Capabilities;
-    SDL_bool buffered;
+    bool buffered;
+    bool first_update;
     input_t Inputs[MAX_INPUTS];
     int NumInputs;
     int NumSliders;
-    SDL_bool ff_initialized;
+    bool ff_initialized;
     DIEFFECT *ffeffect;
     LPDIRECTINPUTEFFECT ffeffect_ref;
 #endif
 
-    SDL_bool bXInputDevice; /* SDL_TRUE if this device supports using the xinput API rather than DirectInput */
-    SDL_bool bXInputHaptic; /* Supports force feedback via XInput. */
-    Uint8 userid; /* XInput userid index for this joystick */
+    bool bXInputDevice; // true if this device supports using the xinput API rather than DirectInput
+    bool bXInputHaptic; // Supports force feedback via XInput.
+    Uint8 userid;           // XInput userid index for this joystick
     DWORD dwPacketNumber;
 };
 
-#if SDL_JOYSTICK_DINPUT
+#ifdef SDL_JOYSTICK_DINPUT
 extern const DIDATAFORMAT SDL_c_dfDIJoystick2;
 #endif
 
 extern void WINDOWS_AddJoystickDevice(JoyStick_DeviceData *device);
 
-/* vi: set ts=4 sw=4 expandtab: */
+// Ends C function definitions when using C++
+#ifdef __cplusplus
+}
+#endif
