@@ -19,7 +19,7 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 #include "../../core/windows/SDL_windows.h"
 #include "../SDL_syslocale.h"
 
@@ -31,8 +31,8 @@ typedef BOOL(WINAPI *pfnGetUserPreferredUILanguages)(DWORD, PULONG, WCHAR *, PUL
 static pfnGetUserPreferredUILanguages pGetUserPreferredUILanguages = NULL;
 static HMODULE kernel32 = 0;
 
-/* this is the fallback for WinXP...one language, not a list. */
-static void SDL_SYS_GetPreferredLocales_winxp(char *buf, size_t buflen)
+// this is the fallback for WinXP...one language, not a list.
+static bool SDL_SYS_GetPreferredLocales_winxp(char *buf, size_t buflen)
 {
     char lang[16];
     char country[16];
@@ -47,53 +47,54 @@ static void SDL_SYS_GetPreferredLocales_winxp(char *buf, size_t buflen)
 
     /* Win95 systems will fail, because they don't have LOCALE_SISO*NAME ... */
     if (langrc == 0) {
-        SDL_SetError("Couldn't obtain language info");
+        return SDL_SetError("Couldn't obtain language info");
     } else {
         (void)SDL_snprintf(buf, buflen, "%s%s%s", lang, ctryrc ? "_" : "", ctryrc ? country : "");
+        return true;
     }
 }
 
-/* this works on Windows Vista and later. */
-static void SDL_SYS_GetPreferredLocales_vista(char *buf, size_t buflen)
+// this works on Windows Vista and later.
+static bool SDL_SYS_GetPreferredLocales_vista(char *buf, size_t buflen)
 {
     ULONG numlangs = 0;
     WCHAR *wbuf = NULL;
     ULONG wbuflen = 0;
-    SDL_bool isstack;
+    bool isstack;
 
     SDL_assert(pGetUserPreferredUILanguages != NULL);
     pGetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &numlangs, NULL, &wbuflen);
 
     wbuf = SDL_small_alloc(WCHAR, wbuflen, &isstack);
     if (!wbuf) {
-        SDL_OutOfMemory();
-        return;
+        return false;
     }
 
     if (!pGetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &numlangs, wbuf, &wbuflen)) {
-        SDL_SYS_GetPreferredLocales_winxp(buf, buflen); /* oh well, try the fallback. */
+        SDL_SYS_GetPreferredLocales_winxp(buf, buflen); // oh well, try the fallback.
     } else {
         const ULONG endidx = (ULONG)SDL_min(buflen, wbuflen - 1);
         ULONG str_start = 0;
         ULONG i;
         for (i = 0; i < endidx; i++) {
-            const char ch = (char)wbuf[i]; /* these should all be low-ASCII, safe to cast */
+            const char ch = (char)wbuf[i]; // these should all be low-ASCII, safe to cast
             if (ch == '\0') {
-                buf[i] = ','; /* change null separators to commas */
+                buf[i] = ','; // change null separators to commas
                 str_start = i;
             } else if (ch == '-') {
-                buf[i] = '_'; /* change '-' to '_' */
+                buf[i] = '_'; // change '-' to '_'
             } else {
-                buf[i] = ch; /* copy through as-is. */
+                buf[i] = ch; // copy through as-is.
             }
         }
-        buf[str_start] = '\0'; /* terminate string, chop off final ',' */
+        buf[str_start] = '\0'; // terminate string, chop off final ','
     }
 
     SDL_small_free(wbuf, isstack);
+    return true;
 }
 
-void SDL_SYS_GetPreferredLocales(char *buf, size_t buflen)
+bool SDL_SYS_GetPreferredLocales(char *buf, size_t buflen)
 {
     if (!kernel32) {
         kernel32 = GetModuleHandle(TEXT("kernel32.dll"));
@@ -103,10 +104,8 @@ void SDL_SYS_GetPreferredLocales(char *buf, size_t buflen)
     }
 
     if (!pGetUserPreferredUILanguages) {
-        SDL_SYS_GetPreferredLocales_winxp(buf, buflen); /* this is always available */
+        return SDL_SYS_GetPreferredLocales_winxp(buf, buflen); // this is always available
     } else {
-        SDL_SYS_GetPreferredLocales_vista(buf, buflen); /* available on Vista and later. */
+        return SDL_SYS_GetPreferredLocales_vista(buf, buflen); // available on Vista and later.
     }
 }
-
-/* vi: set ts=4 sw=4 expandtab: */
