@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,18 +18,14 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
+#include "SDL_internal.h"
 
 #include "SDL_sysurl.h"
 
-extern int SDL_SYS_OpenURL(const char *url);
-
-int
-SDL_OpenURL(const char *url)
+bool SDL_OpenURL(const char *url)
 {
     if (!url) {
         return SDL_InvalidParamError("url");
     }
     return SDL_SYS_OpenURL(url);
 }
-
-/* vi: set ts=4 sw=4 expandtab: */
