@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,15 +18,11 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_POWER_UIKIT
-
-#include "SDL_power.h"
+#ifdef SDL_POWER_UIKIT
 
 void SDL_UIKit_UpdateBatteryMonitoring(void);
-SDL_bool SDL_GetPowerInfo_UIKit(SDL_PowerState * state, int *seconds, int *percent);
+bool SDL_GetPowerInfo_UIKit(SDL_PowerState *state, int *seconds, int *percent);
 
-#endif /* SDL_POWER_UIKIT */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_POWER_UIKIT
