@@ -18,26 +18,24 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_POWER_DISABLED
 #ifdef SDL_POWER_UIKIT
 
 #import <UIKit/UIKit.h>
 
-#include "SDL_power.h"
-#include "SDL_timer.h"
 #include "SDL_syspower.h"
 
-#if !TARGET_OS_TV
-/* turn off the battery monitor if it's been more than X ms since last check. */
+#ifndef SDL_PLATFORM_TVOS
+// turn off the battery monitor if it's been more than X ms since last check.
 static const int BATTERY_MONITORING_TIMEOUT = 3000;
-static Uint32 SDL_UIKitLastPowerInfoQuery = 0;
+static Uint64 SDL_UIKitLastPowerInfoQuery = 0;
 
 void SDL_UIKit_UpdateBatteryMonitoring(void)
 {
     if (SDL_UIKitLastPowerInfoQuery) {
-        if (SDL_TICKS_PASSED(SDL_GetTicks(), SDL_UIKitLastPowerInfoQuery + BATTERY_MONITORING_TIMEOUT)) {
+        if (SDL_GetTicks() >= (SDL_UIKitLastPowerInfoQuery + BATTERY_MONITORING_TIMEOUT)) {
             UIDevice *uidev = [UIDevice currentDevice];
             SDL_assert([uidev isBatteryMonitoringEnabled] == YES);
             [uidev setBatteryMonitoringEnabled:NO];
@@ -48,17 +46,17 @@ void SDL_UIKit_UpdateBatteryMonitoring(void)
 #else
 void SDL_UIKit_UpdateBatteryMonitoring(void)
 {
-    /* Do nothing. */
+    // Do nothing.
 }
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 
-SDL_bool SDL_GetPowerInfo_UIKit(SDL_PowerState *state, int *seconds, int *percent)
+bool SDL_GetPowerInfo_UIKit(SDL_PowerState *state, int *seconds, int *percent)
 {
-#if TARGET_OS_TV
+#ifdef SDL_PLATFORM_TVOS
     *state = SDL_POWERSTATE_NO_BATTERY;
     *seconds = -1;
     *percent = -1;
-#else  /* TARGET_OS_TV */
+#else  // SDL_PLATFORM_TVOS
     @autoreleasepool {
         UIDevice *uidev = [UIDevice currentDevice];
 
@@ -74,7 +72,7 @@ SDL_bool SDL_GetPowerInfo_UIKit(SDL_PowerState *state, int *seconds, int *percen
          */
         SDL_UIKitLastPowerInfoQuery = SDL_GetTicks();
 
-        *seconds = -1; /* no API to estimate this in UIKit. */
+        *seconds = -1; // no API to estimate this in UIKit.
 
         switch (uidev.batteryState) {
         case UIDeviceBatteryStateCharging:
@@ -98,12 +96,10 @@ SDL_bool SDL_GetPowerInfo_UIKit(SDL_PowerState *state, int *seconds, int *percen
         const float level = uidev.batteryLevel;
         *percent = ((level < 0.0f) ? -1 : ((int)((level * 100) + 0.5f)));
     }
-#endif /* TARGET_OS_TV */
+#endif // SDL_PLATFORM_TVOS
 
-    return SDL_TRUE; /* always the definitive answer on iOS. */
+    return true; // always the definitive answer on iOS.
 }
 
-#endif /* SDL_POWER_UIKIT */
-#endif /* SDL_POWER_DISABLED */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_POWER_UIKIT
+#endif // SDL_POWER_DISABLED
