@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,18 +18,16 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_RENDER_OGL
+#ifdef SDL_VIDEO_RENDER_OGL
 
-#include "SDL_stdinc.h"
-#include "SDL_opengl.h"
-#include "SDL_video.h"
+#include <SDL3/SDL_opengl.h>
 #include "SDL_shaders_gl.h"
 
-/* OpenGL shader implementation */
+// OpenGL shader implementation
 
-/* #define DEBUG_SHADERS */
+// #define DEBUG_SHADERS
 
 typedef struct
 {
@@ -54,14 +52,16 @@ struct GL_ShaderContext
     PFNGLSHADERSOURCEARBPROC glShaderSourceARB;
     PFNGLUNIFORM1IARBPROC glUniform1iARB;
     PFNGLUNIFORM1FARBPROC glUniform1fARB;
+    PFNGLUNIFORM3FARBPROC glUniform3fARB;
     PFNGLUSEPROGRAMOBJECTARBPROC glUseProgramObjectARB;
 
-    SDL_bool GL_ARB_texture_rectangle_supported;
+    bool GL_ARB_texture_rectangle_supported;
 
     GL_ShaderData shaders[NUM_SHADERS];
+    const float *shader_params[NUM_SHADERS];
 };
 
-/* *INDENT-OFF* */ /* clang-format off */
+/* *INDENT-OFF* */ // clang-format off
 
 #define COLOR_VERTEX_SHADER                                     \
 "varying vec4 v_color;\n"                                       \
@@ -83,39 +83,16 @@ struct GL_ShaderContext
 "    v_texCoord = vec2(gl_MultiTexCoord0);\n"                   \
 "}"                                                             \
 
-#define JPEG_SHADER_CONSTANTS                                   \
-"// YUV offset \n"                                              \
-"const vec3 offset = vec3(0, -0.501960814, -0.501960814);\n"    \
-"\n"                                                            \
-"// RGB coefficients \n"                                        \
-"const vec3 Rcoeff = vec3(1,  0.000,  1.402);\n"                \
-"const vec3 Gcoeff = vec3(1, -0.3441, -0.7141);\n"              \
-"const vec3 Bcoeff = vec3(1,  1.772,  0.000);\n"                \
-
-#define BT601_SHADER_CONSTANTS                                  \
-"// YUV offset \n"                                              \
-"const vec3 offset = vec3(-0.0627451017, -0.501960814, -0.501960814);\n" \
-"\n"                                                            \
-"// RGB coefficients \n"                                        \
-"const vec3 Rcoeff = vec3(1.1644,  0.000,  1.596);\n"           \
-"const vec3 Gcoeff = vec3(1.1644, -0.3918, -0.813);\n"          \
-"const vec3 Bcoeff = vec3(1.1644,  2.0172,  0.000);\n"          \
-
-#define BT709_SHADER_CONSTANTS                                  \
-"// YUV offset \n"                                              \
-"const vec3 offset = vec3(-0.0627451017, -0.501960814, -0.501960814);\n" \
-"\n"                                                            \
-"// RGB coefficients \n"                                        \
-"const vec3 Rcoeff = vec3(1.1644,  0.000,  1.7927);\n"          \
-"const vec3 Gcoeff = vec3(1.1644, -0.2132, -0.5329);\n"         \
-"const vec3 Bcoeff = vec3(1.1644,  2.1124,  0.000);\n"          \
-
 #define YUV_SHADER_PROLOGUE                                     \
 "varying vec4 v_color;\n"                                       \
 "varying vec2 v_texCoord;\n"                                    \
 "uniform sampler2D tex0; // Y \n"                               \
 "uniform sampler2D tex1; // U \n"                               \
 "uniform sampler2D tex2; // V \n"                               \
+"uniform vec3 Yoffset;\n"                                       \
+"uniform vec3 Rcoeff;\n"                                        \
+"uniform vec3 Gcoeff;\n"                                        \
+"uniform vec3 Bcoeff;\n"                                        \
 "\n"                                                            \
 
 #define YUV_SHADER_BODY                                         \
@@ -135,7 +112,7 @@ struct GL_ShaderContext
 "    yuv.z = texture2D(tex2, tcoord).r;\n"                      \
 "\n"                                                            \
 "    // Do the color transform \n"                              \
-"    yuv += offset;\n"                                          \
+"    yuv += Yoffset;\n"                                         \
 "    rgb.r = dot(yuv, Rcoeff);\n"                               \
 "    rgb.g = dot(yuv, Gcoeff);\n"                               \
 "    rgb.b = dot(yuv, Bcoeff);\n"                               \
@@ -149,6 +126,10 @@ struct GL_ShaderContext
 "varying vec2 v_texCoord;\n"                                    \
 "uniform sampler2D tex0; // Y \n"                               \
 "uniform sampler2D tex1; // U/V \n"                             \
+"uniform vec3 Yoffset;\n"                                       \
+"uniform vec3 Rcoeff;\n"                                        \
+"uniform vec3 Gcoeff;\n"                                        \
+"uniform vec3 Bcoeff;\n"                                        \
 "\n"                                                            \
 
 #define NV12_RA_SHADER_BODY                                     \
@@ -167,7 +148,7 @@ struct GL_ShaderContext
 "    yuv.yz = texture2D(tex1, tcoord).ra;\n"                    \
 "\n"                                                            \
 "    // Do the color transform \n"                              \
-"    yuv += offset;\n"                                          \
+"    yuv += Yoffset;\n"                                         \
 "    rgb.r = dot(yuv, Rcoeff);\n"                               \
 "    rgb.g = dot(yuv, Gcoeff);\n"                               \
 "    rgb.b = dot(yuv, Bcoeff);\n"                               \
@@ -192,7 +173,7 @@ struct GL_ShaderContext
 "    yuv.yz = texture2D(tex1, tcoord).rg;\n"                    \
 "\n"                                                            \
 "    // Do the color transform \n"                              \
-"    yuv += offset;\n"                                          \
+"    yuv += Yoffset;\n"                                         \
 "    rgb.r = dot(yuv, Rcoeff);\n"                               \
 "    rgb.g = dot(yuv, Gcoeff);\n"                               \
 "    rgb.b = dot(yuv, Bcoeff);\n"                               \
@@ -201,14 +182,7 @@ struct GL_ShaderContext
 "    gl_FragColor = vec4(rgb, 1.0) * v_color;\n"                \
 "}"                                                             \
 
-#define NV21_SHADER_PROLOGUE                                    \
-"varying vec4 v_color;\n"                                       \
-"varying vec2 v_texCoord;\n"                                    \
-"uniform sampler2D tex0; // Y \n"                               \
-"uniform sampler2D tex1; // U/V \n"                             \
-"\n"                                                            \
-
-#define NV21_SHADER_BODY                                        \
+#define NV21_RA_SHADER_BODY                                     \
 "\n"                                                            \
 "void main()\n"                                                 \
 "{\n"                                                           \
@@ -224,7 +198,32 @@ struct GL_ShaderContext
 "    yuv.yz = texture2D(tex1, tcoord).ar;\n"                    \
 "\n"                                                            \
 "    // Do the color transform \n"                              \
-"    yuv += offset;\n"                                          \
+"    yuv += Yoffset;\n"                                         \
+"    rgb.r = dot(yuv, Rcoeff);\n"                               \
+"    rgb.g = dot(yuv, Gcoeff);\n"                               \
+"    rgb.b = dot(yuv, Bcoeff);\n"                               \
+"\n"                                                            \
+"    // That was easy. :) \n"                                   \
+"    gl_FragColor = vec4(rgb, 1.0) * v_color;\n"                \
+"}"                                                             \
+
+#define NV21_RG_SHADER_BODY                                     \
+"\n"                                                            \
+"void main()\n"                                                 \
+"{\n"                                                           \
+"    vec2 tcoord;\n"                                            \
+"    vec3 yuv, rgb;\n"                                          \
+"\n"                                                            \
+"    // Get the Y value \n"                                     \
+"    tcoord = v_texCoord;\n"                                    \
+"    yuv.x = texture2D(tex0, tcoord).r;\n"                      \
+"\n"                                                            \
+"    // Get the U and V values \n"                              \
+"    tcoord *= UVCoordScale;\n"                                 \
+"    yuv.yz = texture2D(tex1, tcoord).gr;\n"                    \
+"\n"                                                            \
+"    // Do the color transform \n"                              \
+"    yuv += Yoffset;\n"                                         \
 "    rgb.r = dot(yuv, Rcoeff);\n"                               \
 "    rgb.g = dot(yuv, Gcoeff);\n"                               \
 "    rgb.b = dot(yuv, Bcoeff);\n"                               \
@@ -238,14 +237,14 @@ struct GL_ShaderContext
  *  texture_rectangle versions if we choose to use that extension.
  */
 static const char *shader_source[NUM_SHADERS][2] = {
-    /* SHADER_NONE */
+    // SHADER_NONE
     { NULL, NULL },
 
-    /* SHADER_SOLID */
+    // SHADER_SOLID
     {
-        /* vertex shader */
+        // vertex shader
         COLOR_VERTEX_SHADER,
-        /* fragment shader */
+        // fragment shader
 "varying vec4 v_color;\n"
 "\n"
 "void main()\n"
@@ -254,11 +253,11 @@ static const char *shader_source[NUM_SHADERS][2] = {
 "}"
     },
 
-    /* SHADER_RGB */
+    // SHADER_RGB
     {
-        /* vertex shader */
+        // vertex shader
         TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
+        // fragment shader
 "varying vec4 v_color;\n"
 "varying vec2 v_texCoord;\n"
 "uniform sampler2D tex0;\n"
@@ -271,11 +270,11 @@ static const char *shader_source[NUM_SHADERS][2] = {
 "}"
     },
 
-    /* SHADER_RGBA */
+    // SHADER_RGBA
     {
-        /* vertex shader */
+        // vertex shader
         TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
+        // fragment shader
 "varying vec4 v_color;\n"
 "varying vec2 v_texCoord;\n"
 "uniform sampler2D tex0;\n"
@@ -286,111 +285,52 @@ static const char *shader_source[NUM_SHADERS][2] = {
 "}"
     },
 #if SDL_HAVE_YUV
-    /* SHADER_YUV_JPEG */
+    // SHADER_YUV
     {
-        /* vertex shader */
+        // vertex shader
         TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
+        // fragment shader
         YUV_SHADER_PROLOGUE
-        JPEG_SHADER_CONSTANTS
         YUV_SHADER_BODY
     },
-    /* SHADER_YUV_BT601 */
+    // SHADER_NV12_RA
     {
-        /* vertex shader */
+        // vertex shader
         TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
-        YUV_SHADER_PROLOGUE
-        BT601_SHADER_CONSTANTS
-        YUV_SHADER_BODY
-    },
-    /* SHADER_YUV_BT709 */
-    {
-        /* vertex shader */
-        TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
-        YUV_SHADER_PROLOGUE
-        BT709_SHADER_CONSTANTS
-        YUV_SHADER_BODY
-    },
-    /* SHADER_NV12_JPEG */
-    {
-        /* vertex shader */
-        TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
+        // fragment shader
         NV12_SHADER_PROLOGUE
-        JPEG_SHADER_CONSTANTS
         NV12_RA_SHADER_BODY
     },
-    /* SHADER_NV12_RA_BT601 */
+    // SHADER_NV12_RG
     {
-        /* vertex shader */
+        // vertex shader
         TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
+        // fragment shader
         NV12_SHADER_PROLOGUE
-        BT601_SHADER_CONSTANTS
-        NV12_RA_SHADER_BODY
-    },
-    /* SHADER_NV12_RG_BT601 */
-    {
-        /* vertex shader */
-        TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
-        NV12_SHADER_PROLOGUE
-        BT601_SHADER_CONSTANTS
         NV12_RG_SHADER_BODY
     },
-    /* SHADER_NV12_RA_BT709 */
+    // SHADER_NV21_RA
     {
-        /* vertex shader */
+        // vertex shader
         TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
+        // fragment shader
         NV12_SHADER_PROLOGUE
-        BT709_SHADER_CONSTANTS
-        NV12_RA_SHADER_BODY
+        NV21_RA_SHADER_BODY
     },
-    /* SHADER_NV12_RG_BT709 */
+    // SHADER_NV21_RG
     {
-        /* vertex shader */
+        // vertex shader
         TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
+        // fragment shader
         NV12_SHADER_PROLOGUE
-        BT709_SHADER_CONSTANTS
-        NV12_RG_SHADER_BODY
+        NV21_RG_SHADER_BODY
     },
-    /* SHADER_NV21_JPEG */
-    {
-        /* vertex shader */
-        TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
-        NV21_SHADER_PROLOGUE
-        JPEG_SHADER_CONSTANTS
-        NV21_SHADER_BODY
-    },
-    /* SHADER_NV21_BT601 */
-    {
-        /* vertex shader */
-        TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
-        NV21_SHADER_PROLOGUE
-        BT601_SHADER_CONSTANTS
-        NV21_SHADER_BODY
-    },
-    /* SHADER_NV21_BT709 */
-    {
-        /* vertex shader */
-        TEXTURE_VERTEX_SHADER,
-        /* fragment shader */
-        NV21_SHADER_PROLOGUE
-        BT709_SHADER_CONSTANTS
-        NV21_SHADER_BODY
-    },
-#endif /* SDL_HAVE_YUV */
+#endif // SDL_HAVE_YUV
 };
 
-/* *INDENT-ON* */ /* clang-format on */
+/* *INDENT-ON* */ // clang-format on
 
-static SDL_bool CompileShader(GL_ShaderContext *ctx, GLhandleARB shader, const char *defines, const char *source)
+static bool CompileShader(GL_ShaderContext *ctx, GLhandleARB shader, const char *defines, const char *source)
 {
     GLint status;
     const char *sources[2];
@@ -402,28 +342,24 @@ static SDL_bool CompileShader(GL_ShaderContext *ctx, GLhandleARB shader, const c
     ctx->glCompileShaderARB(shader);
     ctx->glGetObjectParameterivARB(shader, GL_OBJECT_COMPILE_STATUS_ARB, &status);
     if (status == 0) {
-        SDL_bool isstack;
+        bool isstack;
         GLint length;
         char *info;
 
         ctx->glGetObjectParameterivARB(shader, GL_OBJECT_INFO_LOG_LENGTH_ARB, &length);
         info = SDL_small_alloc(char, length + 1, &isstack);
-        ctx->glGetInfoLogARB(shader, length, NULL, info);
-        SDL_LogError(SDL_LOG_CATEGORY_RENDER,
-                     "Failed to compile shader:\n%s%s\n%s", defines, source, info);
-#ifdef DEBUG_SHADERS
-        fprintf(stderr,
-                "Failed to compile shader:\n%s%s\n%s", defines, source, info);
-#endif
-        SDL_small_free(info, isstack);
-
-        return SDL_FALSE;
+        if (info) {
+            ctx->glGetInfoLogARB(shader, length, NULL, info);
+            SDL_LogError(SDL_LOG_CATEGORY_RENDER, "Failed to compile shader:\n%s%s\n%s", defines, source, info);
+            SDL_small_free(info, isstack);
+        }
+        return false;
     } else {
-        return SDL_TRUE;
+        return true;
     }
 }
 
-static SDL_bool CompileShaderProgram(GL_ShaderContext *ctx, int index, GL_ShaderData *data)
+static bool CompileShaderProgram(GL_ShaderContext *ctx, int index, GL_ShaderData *data)
 {
     const int num_tmus_bound = 4;
     const char *vert_defines = "";
@@ -432,12 +368,12 @@ static SDL_bool CompileShaderProgram(GL_ShaderContext *ctx, int index, GL_Shader
     GLint location;
 
     if (index == SHADER_NONE) {
-        return SDL_TRUE;
+        return true;
     }
 
     ctx->glGetError();
 
-    /* Make sure we use the correct sampler type for our texture type */
+    // Make sure we use the correct sampler type for our texture type
     if (ctx->GL_ARB_texture_rectangle_supported) {
         frag_defines =
             "#define sampler2D sampler2DRect\n"
@@ -448,27 +384,27 @@ static SDL_bool CompileShaderProgram(GL_ShaderContext *ctx, int index, GL_Shader
             "#define UVCoordScale 1.0\n";
     }
 
-    /* Create one program object to rule them all */
+    // Create one program object to rule them all
     data->program = ctx->glCreateProgramObjectARB();
 
-    /* Create the vertex shader */
+    // Create the vertex shader
     data->vert_shader = ctx->glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
     if (!CompileShader(ctx, data->vert_shader, vert_defines, shader_source[index][0])) {
-        return SDL_FALSE;
+        return false;
     }
 
-    /* Create the fragment shader */
+    // Create the fragment shader
     data->frag_shader = ctx->glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
     if (!CompileShader(ctx, data->frag_shader, frag_defines, shader_source[index][1])) {
-        return SDL_FALSE;
+        return false;
     }
 
-    /* ... and in the darkness bind them */
+    // ... and in the darkness bind them
     ctx->glAttachObjectARB(data->program, data->vert_shader);
     ctx->glAttachObjectARB(data->program, data->frag_shader);
     ctx->glLinkProgramARB(data->program);
 
-    /* Set up some uniform variables */
+    // Set up some uniform variables
     ctx->glUseProgramObjectARB(data->program);
     for (i = 0; i < num_tmus_bound; ++i) {
         char tex_name[10];
@@ -493,7 +429,7 @@ static void DestroyShaderProgram(GL_ShaderContext *ctx, GL_ShaderData *data)
 GL_ShaderContext *GL_CreateShaderContext(void)
 {
     GL_ShaderContext *ctx;
-    SDL_bool shaders_supported;
+    bool shaders_supported;
     int i;
 
     ctx = (GL_ShaderContext *)SDL_calloc(1, sizeof(*ctx));
@@ -504,11 +440,11 @@ GL_ShaderContext *GL_CreateShaderContext(void)
     if (!SDL_GL_ExtensionSupported("GL_ARB_texture_non_power_of_two") &&
         (SDL_GL_ExtensionSupported("GL_ARB_texture_rectangle") ||
          SDL_GL_ExtensionSupported("GL_EXT_texture_rectangle"))) {
-        ctx->GL_ARB_texture_rectangle_supported = SDL_TRUE;
+        ctx->GL_ARB_texture_rectangle_supported = true;
     }
 
-    /* Check for shader support */
-    shaders_supported = SDL_FALSE;
+    // Check for shader support
+    shaders_supported = false;
     if (SDL_GL_ExtensionSupported("GL_ARB_shader_objects") &&
         SDL_GL_ExtensionSupported("GL_ARB_shading_language_100") &&
         SDL_GL_ExtensionSupported("GL_ARB_vertex_shader") &&
@@ -526,6 +462,7 @@ GL_ShaderContext *GL_CreateShaderContext(void)
         ctx->glShaderSourceARB = (PFNGLSHADERSOURCEARBPROC)SDL_GL_GetProcAddress("glShaderSourceARB");
         ctx->glUniform1iARB = (PFNGLUNIFORM1IARBPROC)SDL_GL_GetProcAddress("glUniform1iARB");
         ctx->glUniform1fARB = (PFNGLUNIFORM1FARBPROC)SDL_GL_GetProcAddress("glUniform1fARB");
+        ctx->glUniform3fARB = (PFNGLUNIFORM3FARBPROC)SDL_GL_GetProcAddress("glUniform3fARB");
         ctx->glUseProgramObjectARB = (PFNGLUSEPROGRAMOBJECTARBPROC)SDL_GL_GetProcAddress("glUseProgramObjectARB");
         if (ctx->glGetError &&
             ctx->glAttachObjectARB &&
@@ -540,8 +477,9 @@ GL_ShaderContext *GL_CreateShaderContext(void)
             ctx->glShaderSourceARB &&
             ctx->glUniform1iARB &&
             ctx->glUniform1fARB &&
+            ctx->glUniform3fARB &&
             ctx->glUseProgramObjectARB) {
-            shaders_supported = SDL_TRUE;
+            shaders_supported = true;
         }
     }
 
@@ -550,7 +488,7 @@ GL_ShaderContext *GL_CreateShaderContext(void)
         return NULL;
     }
 
-    /* Compile all the shaders */
+    // Compile all the shaders
     for (i = 0; i < NUM_SHADERS; ++i) {
         if (!CompileShaderProgram(ctx, i, &ctx->shaders[i])) {
             GL_DestroyShaderContext(ctx);
@@ -558,13 +496,37 @@ GL_ShaderContext *GL_CreateShaderContext(void)
         }
     }
 
-    /* We're done! */
+    // We're done!
     return ctx;
 }
 
-void GL_SelectShader(GL_ShaderContext *ctx, GL_Shader shader)
+void GL_SelectShader(GL_ShaderContext *ctx, GL_Shader shader, const float *shader_params)
 {
-    ctx->glUseProgramObjectARB(ctx->shaders[shader].program);
+    GLint location;
+    GLhandleARB program = ctx->shaders[shader].program;
+
+    ctx->glUseProgramObjectARB(program);
+
+    if (shader_params && shader_params != ctx->shader_params[shader]) {
+        // YUV shader params are Yoffset, 0, Rcoeff, 0, Gcoeff, 0, Bcoeff, 0
+        location = ctx->glGetUniformLocationARB(program, "Yoffset");
+        if (location >= 0) {
+            ctx->glUniform3fARB(location, shader_params[0], shader_params[1], shader_params[2]);
+        }
+        location = ctx->glGetUniformLocationARB(program, "Rcoeff");
+        if (location >= 0) {
+            ctx->glUniform3fARB(location, shader_params[4], shader_params[5], shader_params[6]);
+        }
+        location = ctx->glGetUniformLocationARB(program, "Gcoeff");
+        if (location >= 0) {
+            ctx->glUniform3fARB(location, shader_params[8], shader_params[9], shader_params[10]);
+        }
+        location = ctx->glGetUniformLocationARB(program, "Bcoeff");
+        if (location >= 0) {
+            ctx->glUniform3fARB(location, shader_params[12], shader_params[13], shader_params[14]);
+        }
+        ctx->shader_params[shader] = shader_params;
+    }
 }
 
 void GL_DestroyShaderContext(GL_ShaderContext *ctx)
@@ -577,6 +539,4 @@ void GL_DestroyShaderContext(GL_ShaderContext *ctx)
     SDL_free(ctx);
 }
 
-#endif /* SDL_VIDEO_RENDER_OGL */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_RENDER_OGL
