@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -22,9 +22,9 @@
 #ifndef SDL_shaders_gl_h_
 #define SDL_shaders_gl_h_
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-/* OpenGL shader implementation */
+// OpenGL shader implementation
 
 typedef enum
 {
@@ -34,17 +34,11 @@ typedef enum
     SHADER_RGB,
     SHADER_RGBA,
 #if SDL_HAVE_YUV
-    SHADER_YUV_JPEG,
-    SHADER_YUV_BT601,
-    SHADER_YUV_BT709,
-    SHADER_NV12_JPEG,
-    SHADER_NV12_RA_BT601,
-    SHADER_NV12_RG_BT601,
-    SHADER_NV12_RA_BT709,
-    SHADER_NV12_RG_BT709,
-    SHADER_NV21_JPEG,
-    SHADER_NV21_BT601,
-    SHADER_NV21_BT709,
+    SHADER_YUV,
+    SHADER_NV12_RA,
+    SHADER_NV12_RG,
+    SHADER_NV21_RA,
+    SHADER_NV21_RG,
 #endif
     NUM_SHADERS
 } GL_Shader;
@@ -52,9 +46,7 @@ typedef enum
 typedef struct GL_ShaderContext GL_ShaderContext;
 
 extern GL_ShaderContext *GL_CreateShaderContext(void);
-extern void GL_SelectShader(GL_ShaderContext *ctx, GL_Shader shader);
+extern void GL_SelectShader(GL_ShaderContext *ctx, GL_Shader shader, const float *shader_params);
 extern void GL_DestroyShaderContext(GL_ShaderContext *ctx);
 
-#endif /* SDL_shaders_gl_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_shaders_gl_h_
