@@ -18,12 +18,11 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_RENDER_PS2
+#ifdef SDL_VIDEO_RENDER_PS2
 
 #include "../SDL_sysrender.h"
-#include "SDL_hints.h"
 
 #include <kernel.h>
 #include <malloc.h>
@@ -31,14 +30,20 @@
 #include <dmaKit.h>
 #include <gsToolkit.h>
 
+#ifdef HAVE_GCC_DIAGNOSTIC_PRAGMA
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeclaration-after-statement"
-#include <gsInline.h>
-#pragma GCC diagnostic pop
+#endif
 
-/* turn black GS Screen */
+#include <gsInline.h>
+
+#ifdef HAVE_GCC_DIAGNOSTIC_PRAGMA
+#pragma GCC diagnostic pop
+#endif
+
+// turn black GS Screen
 #define GS_BLACK GS_SETREG_RGBA(0x00, 0x00, 0x00, 0x80)
-/* Size of Persistent drawbuffer (Single Buffered) */
+// Size of Persistent drawbuffer (Single Buffered)
 #define RENDER_QUEUE_PER_POOLSIZE 1024 * 256 // 256K of persistent renderqueue
 /* Size of Oneshot drawbuffer (Double Buffered, so it uses this size * 2) */
 #define RENDER_QUEUE_OS_POOLSIZE 1024 * 1024 * 2 // 2048K of oneshot renderqueue
@@ -49,13 +54,13 @@ typedef struct
     uint64_t drawColor;
     SDL_Rect *viewport;
     int32_t vsync_callback_id;
-    uint8_t vsync; /* 0 (Disabled), 1 (Enabled), 2 (Dynamic) */
+    int vsync; // 0 (Disabled), 1 (Enabled), -1 (Dynamic)
 } PS2_RenderData;
 
 static int vsync_sema_id = 0;
 
-/* PRIVATE METHODS */
-static int vsync_handler()
+// PRIVATE METHODS
+static int vsync_handler(void)
 {
     iSignalSema(vsync_sema_id);
 
@@ -63,7 +68,7 @@ static int vsync_handler()
     return 0;
 }
 
-/* Copy of gsKit_sync_flip, but without the 'flip' */
+// Copy of gsKit_sync_flip, but without the 'flip'
 static void gsKit_sync(GSGLOBAL *gsGlobal)
 {
     if (!gsGlobal->FirstFrame) {
@@ -73,7 +78,7 @@ static void gsKit_sync(GSGLOBAL *gsGlobal)
         ;
 }
 
-/* Copy of gsKit_sync_flip, but without the 'sync' */
+// Copy of gsKit_sync_flip, but without the 'sync'
 static void gsKit_flip(GSGLOBAL *gsGlobal)
 {
     if (!gsGlobal->FirstFrame) {
@@ -98,54 +103,74 @@ static int PixelFormatToPS2PSM(Uint32 format)
     }
 }
 
+static gs_rgbaq float_color_to_RGBAQ(const SDL_FColor *color, float color_scale)
+{
+    uint8_t colorR = (uint8_t)SDL_roundf(SDL_clamp(color->r * color_scale, 0.0f, 1.0f) * 255.0f);
+    uint8_t colorG = (uint8_t)SDL_roundf(SDL_clamp(color->g * color_scale, 0.0f, 1.0f) * 255.0f);
+    uint8_t colorB = (uint8_t)SDL_roundf(SDL_clamp(color->b * color_scale, 0.0f, 1.0f) * 255.0f);
+    uint8_t colorA = (uint8_t)SDL_roundf(SDL_clamp(color->a, 0.0f, 1.0f) * 255.0f);
+
+    return color_to_RGBAQ(colorR, colorG, colorB, colorA, 0x00);
+}
+
+static uint64_t float_GS_SETREG_RGBAQ(const SDL_FColor *color, float color_scale)
+{
+    uint8_t colorR = (uint8_t)SDL_roundf(SDL_clamp(color->r * color_scale, 0.0f, 1.0f) * 255.0f);
+    uint8_t colorG = (uint8_t)SDL_roundf(SDL_clamp(color->g * color_scale, 0.0f, 1.0f) * 255.0f);
+    uint8_t colorB = (uint8_t)SDL_roundf(SDL_clamp(color->b * color_scale, 0.0f, 1.0f) * 255.0f);
+    uint8_t colorA = (uint8_t)SDL_roundf(SDL_clamp(color->a, 0.0f, 1.0f) * 255.0f);
+
+    return GS_SETREG_RGBAQ(colorR, colorG, colorB, colorA, 0x00);
+}
+
 static void PS2_WindowEvent(SDL_Renderer *renderer, const SDL_WindowEvent *event)
 {
 }
 
-static int PS2_CreateTexture(SDL_Renderer *renderer, SDL_Texture *texture)
+static bool PS2_CreateTexture(SDL_Renderer *renderer, SDL_Texture *texture, SDL_PropertiesID create_props)
 {
     GSTEXTURE *ps2_tex = (GSTEXTURE *)SDL_calloc(1, sizeof(GSTEXTURE));
 
     if (!ps2_tex) {
-        return SDL_OutOfMemory();
+        return false;
     }
 
     ps2_tex->Width = texture->w;
     ps2_tex->Height = texture->h;
     ps2_tex->PSM = PixelFormatToPS2PSM(texture->format);
-    ps2_tex->Mem = memalign(128, gsKit_texture_size_ee(ps2_tex->Width, ps2_tex->Height, ps2_tex->PSM));
+    ps2_tex->Mem = SDL_aligned_alloc(128, gsKit_texture_size_ee(ps2_tex->Width, ps2_tex->Height, ps2_tex->PSM));
 
     if (!ps2_tex->Mem) {
         SDL_free(ps2_tex);
-        return SDL_OutOfMemory();
+        return false;
     }
 
-    texture->driverdata = ps2_tex;
+    texture->internal = ps2_tex;
 
-    return 0;
+    return true;
 }
 
-static int PS2_LockTexture(SDL_Renderer *renderer, SDL_Texture *texture,
+static bool PS2_LockTexture(SDL_Renderer *renderer, SDL_Texture *texture,
                            const SDL_Rect *rect, void **pixels, int *pitch)
 {
-    GSTEXTURE *ps2_texture = (GSTEXTURE *)texture->driverdata;
+    GSTEXTURE *ps2_texture = (GSTEXTURE *)texture->internal;
 
     *pixels =
         (void *)((Uint8 *)ps2_texture->Mem + rect->y * ps2_texture->Width * SDL_BYTESPERPIXEL(texture->format) +
                  rect->x * SDL_BYTESPERPIXEL(texture->format));
     *pitch = ps2_texture->Width * SDL_BYTESPERPIXEL(texture->format);
-    return 0;
+    return true;
 }
 
 static void PS2_UnlockTexture(SDL_Renderer *renderer, SDL_Texture *texture)
 {
-    GSTEXTURE *ps2_texture = (GSTEXTURE *)texture->driverdata;
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    GSTEXTURE *ps2_texture = (GSTEXTURE *)texture->internal;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
 
     gsKit_TexManager_invalidate(data->gsGlobal, ps2_texture);
 }
 
-static int PS2_UpdateTexture(SDL_Renderer *renderer, SDL_Texture *texture,
+static bool PS2_UpdateTexture(SDL_Renderer *renderer, SDL_Texture *texture,
                              const SDL_Rect *rect, const void *pixels, int pitch)
 {
     const Uint8 *src;
@@ -167,32 +192,32 @@ static int PS2_UpdateTexture(SDL_Renderer *renderer, SDL_Texture *texture,
 
     PS2_UnlockTexture(renderer, texture);
 
-    return 0;
+    return true;
 }
 
 static void PS2_SetTextureScaleMode(SDL_Renderer *renderer, SDL_Texture *texture, SDL_ScaleMode scaleMode)
 {
-    GSTEXTURE *ps2_texture = (GSTEXTURE *)texture->driverdata;
+    GSTEXTURE *ps2_texture = (GSTEXTURE *)texture->internal;
     /*
      set texture filtering according to scaleMode
-     suported hint values are nearest (0, default) or linear (1)
+     supported hint values are nearest (0, default) or linear (1)
      gskit scale mode is either GS_FILTER_NEAREST (good for tile-map)
      or GS_FILTER_LINEAR (good for scaling)
      */
-    uint32_t gsKitScaleMode = (scaleMode == SDL_ScaleModeNearest
+    uint32_t gsKitScaleMode = (scaleMode == SDL_SCALEMODE_NEAREST
                                    ? GS_FILTER_NEAREST
                                    : GS_FILTER_LINEAR);
     ps2_texture->Filter = gsKitScaleMode;
 }
 
-static int PS2_SetRenderTarget(SDL_Renderer *renderer, SDL_Texture *texture)
+static bool PS2_SetRenderTarget(SDL_Renderer *renderer, SDL_Texture *texture)
 {
-    return 0;
+    return true;
 }
 
-static int PS2_QueueSetViewport(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
+static bool PS2_QueueSetViewport(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
 {
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
     const SDL_Rect *viewport = &cmd->data.viewport.rect;
     data->viewport = (SDL_Rect *)viewport;
 
@@ -200,66 +225,62 @@ static int PS2_QueueSetViewport(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
     data->gsGlobal->OffsetY = (int)((2048.0f + (float)viewport->y) * 16.0f);
     gsKit_set_scissor(data->gsGlobal, GS_SETREG_SCISSOR(viewport->x, viewport->x + viewport->w, viewport->y, viewport->y + viewport->h));
 
-    return 0;
+    return true;
 }
 
-static int PS2_QueueSetDrawColor(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
+static bool PS2_QueueNoOp(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
 {
-    return 0; /* nothing to do in this backend. */
+    return true; // nothing to do in this backend.
 }
 
-static int PS2_QueueDrawPoints(SDL_Renderer *renderer, SDL_RenderCommand *cmd, const SDL_FPoint *points, int count)
+static bool PS2_QueueDrawPoints(SDL_Renderer *renderer, SDL_RenderCommand *cmd, const SDL_FPoint *points, int count)
 {
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
     GSPRIMPOINT *vertices = (GSPRIMPOINT *)SDL_AllocateRenderVertices(renderer, count * sizeof(GSPRIMPOINT), 4, &cmd->data.draw.first);
-    uint8_t colorR, colorG, colorB, colorA;
     gs_rgbaq rgbaq;
     int i;
 
     if (!vertices) {
-        return -1;
+        return false;
     }
 
     cmd->data.draw.count = count;
 
-    colorR = cmd->data.draw.r;
-    colorG = cmd->data.draw.g;
-    colorB = cmd->data.draw.b;
-    colorA = cmd->data.draw.a;
-    rgbaq = color_to_RGBAQ(colorR, colorG, colorB, colorA, 0.0f);
+    rgbaq = float_color_to_RGBAQ(&cmd->data.draw.color, cmd->data.draw.color_scale);
 
     for (i = 0; i < count; i++, vertices++, points++) {
         vertices->xyz2 = vertex_to_XYZ2(data->gsGlobal, points->x, points->y, 0);
         vertices->rgbaq = rgbaq;
     }
-    return 0;
+    return true;
 }
 
-static int PS2_QueueGeometry(SDL_Renderer *renderer, SDL_RenderCommand *cmd, SDL_Texture *texture,
-                             const float *xy, int xy_stride, const SDL_Color *color, int color_stride, const float *uv, int uv_stride,
+static bool PS2_QueueGeometry(SDL_Renderer *renderer, SDL_RenderCommand *cmd, SDL_Texture *texture,
+                             const float *xy, int xy_stride, const SDL_FColor *color, int color_stride, const float *uv, int uv_stride,
                              int num_vertices, const void *indices, int num_indices, int size_indices,
                              float scale_x, float scale_y)
 {
     int i;
     int count = indices ? num_indices : num_vertices;
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
+    const float color_scale = cmd->data.draw.color_scale;
 
     cmd->data.draw.count = count;
     size_indices = indices ? size_indices : 0;
 
     if (texture) {
         GSPRIMUVPOINT *vertices = (GSPRIMUVPOINT *) SDL_AllocateRenderVertices(renderer, count * sizeof(GSPRIMUVPOINT), 4, &cmd->data.draw.first);
-        GSTEXTURE *ps2_tex = (GSTEXTURE *) texture->driverdata;
+        GSTEXTURE *ps2_tex = (GSTEXTURE *) texture->internal;
 
         if (!vertices) {
-            return -1;
+            return false;
         }
 
         for (i = 0; i < count; i++) {
             int j;
             float *xy_;
             float *uv_;
-            SDL_Color col_;
+            SDL_FColor *col_;
             if (size_indices == 4) {
                 j = ((const Uint32 *)indices)[i];
             } else if (size_indices == 2) {
@@ -271,11 +292,11 @@ static int PS2_QueueGeometry(SDL_Renderer *renderer, SDL_RenderCommand *cmd, SDL
             }
 
             xy_ = (float *)((char *)xy + j * xy_stride);
-            col_ = *(SDL_Color *)((char *)color + j * color_stride);
+            col_ = (SDL_FColor *)((char *)color + j * color_stride);
             uv_ = (float *)((char *)uv + j * uv_stride);
 
             vertices->xyz2 = vertex_to_XYZ2(data->gsGlobal, xy_[0] * scale_x, xy_[1] * scale_y, 0);
-            vertices->rgbaq = color_to_RGBAQ(col_.r, col_.g, col_.b, col_.a, 0);
+            vertices->rgbaq = float_color_to_RGBAQ(col_, color_scale);
             vertices->uv = vertex_to_UV(ps2_tex, uv_[0] * ps2_tex->Width, uv_[1] * ps2_tex->Height);
 
             vertices++;
@@ -285,13 +306,13 @@ static int PS2_QueueGeometry(SDL_Renderer *renderer, SDL_RenderCommand *cmd, SDL
         GSPRIMPOINT *vertices = (GSPRIMPOINT *)SDL_AllocateRenderVertices(renderer, count * sizeof(GSPRIMPOINT), 4, &cmd->data.draw.first);
 
         if (!vertices) {
-            return -1;
+            return false;
         }
 
         for (i = 0; i < count; i++) {
             int j;
             float *xy_;
-            SDL_Color col_;
+            SDL_FColor *col_;
             if (size_indices == 4) {
                 j = ((const Uint32 *)indices)[i];
             } else if (size_indices == 2) {
@@ -303,32 +324,32 @@ static int PS2_QueueGeometry(SDL_Renderer *renderer, SDL_RenderCommand *cmd, SDL
             }
 
             xy_ = (float *)((char *)xy + j * xy_stride);
-            col_ = *(SDL_Color *)((char *)color + j * color_stride);
+            col_ = (SDL_FColor *)((char *)color + j * color_stride);
 
             vertices->xyz2 = vertex_to_XYZ2(data->gsGlobal, xy_[0] * scale_x, xy_[1] * scale_y, 0);
-            vertices->rgbaq = color_to_RGBAQ(col_.r, col_.g, col_.b, col_.a, 0.0f);
+            vertices->rgbaq = float_color_to_RGBAQ(col_, color_scale);
 
             vertices++;
         }
     }
 
-    return 0;
+    return true;
 }
 
-static int PS2_RenderSetViewPort(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
+static bool PS2_RenderSetViewPort(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
 {
-    return 0; /* nothing to do in this backend. */
+    return true; // nothing to do in this backend.
 }
 
-static int PS2_RenderSetClipRect(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
+static bool PS2_RenderSetClipRect(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
 {
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
     SDL_Rect *viewport = data->viewport;
 
     const SDL_Rect *rect = &cmd->data.cliprect.rect;
 
     if (cmd->data.cliprect.enabled) {
-        /* We need to do it relative to saved viewport */
+        // We need to do it relative to saved viewport
         viewport->x += rect->x;
         viewport->y += rect->y;
         viewport->w = SDL_min(viewport->w, rect->w);
@@ -336,53 +357,42 @@ static int PS2_RenderSetClipRect(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
     }
     gsKit_set_scissor(data->gsGlobal, GS_SETREG_SCISSOR(viewport->x, viewport->x + viewport->w, viewport->y, viewport->y + viewport->h));
 
-    return 0;
+    return true;
 }
 
-static int PS2_RenderSetDrawColor(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
+static bool PS2_RenderSetDrawColor(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
 {
-    int colorR, colorG, colorB, colorA;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
 
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
-
-    colorR = (cmd->data.color.r);
-    colorG = (cmd->data.color.g);
-    colorB = (cmd->data.color.b);
-    colorA = (cmd->data.color.a);
-    data->drawColor = GS_SETREG_RGBAQ(colorR, colorG, colorB, colorA, 0x00);
-    return 0;
+    data->drawColor = float_GS_SETREG_RGBAQ(&cmd->data.color.color, cmd->data.color.color_scale);
+    return true;
 }
 
-static int PS2_RenderClear(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
+static bool PS2_RenderClear(SDL_Renderer *renderer, SDL_RenderCommand *cmd)
 {
-    int colorR, colorG, colorB, colorA, offsetX, offsetY;
+    int offsetX, offsetY;
     SDL_Rect *viewport;
 
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
 
-    colorR = (cmd->data.color.r);
-    colorG = (cmd->data.color.g);
-    colorB = (cmd->data.color.b);
-    colorA = (cmd->data.color.a);
-
-    /* Clear the screen, so let's put default viewport */
+    // Clear the screen, so let's put default viewport
     gsKit_set_scissor(data->gsGlobal, GS_SCISSOR_RESET);
-    /* Put back original offset */
+    // Put back original offset
     offsetX = data->gsGlobal->OffsetX;
     offsetY = data->gsGlobal->OffsetY;
     data->gsGlobal->OffsetX = (int)(2048.0f * 16.0f);
     data->gsGlobal->OffsetY = (int)(2048.0f * 16.0f);
-    gsKit_clear(data->gsGlobal, GS_SETREG_RGBAQ(colorR, colorG, colorB, colorA, 0x00));
+    gsKit_clear(data->gsGlobal, float_GS_SETREG_RGBAQ(&cmd->data.color.color, cmd->data.color.color_scale));
 
-    /* Put back original offset */
+    // Put back original offset
     data->gsGlobal->OffsetX = offsetX;
     data->gsGlobal->OffsetY = offsetY;
 
-    // /* Put back view port */
+    // // Put back view port
     viewport = data->viewport;
     gsKit_set_scissor(data->gsGlobal, GS_SETREG_SCISSOR(viewport->x, viewport->x + viewport->w, viewport->y, viewport->y + viewport->h));
-    
-    return 0;
+
+    return true;
 }
 
 static void PS2_SetBlendMode(PS2_RenderData *data, int blendMode)
@@ -406,16 +416,30 @@ static void PS2_SetBlendMode(PS2_RenderData *data, int blendMode)
         data->gsGlobal->PrimAlphaEnable = GS_SETTING_ON;
         break;
     }
+    case SDL_BLENDMODE_BLEND_PREMULTIPLIED:
+    {
+        // FIXME: What are the settings for this?
+        gsKit_set_primalpha(data->gsGlobal, GS_SETREG_ALPHA(A_COLOR_SOURCE, A_COLOR_DEST, A_ALPHA_SOURCE, A_COLOR_DEST, 0), 0);
+        data->gsGlobal->PrimAlphaEnable = GS_SETTING_ON;
+        break;
+    }
     case SDL_BLENDMODE_ADD:
     {
         gsKit_set_primalpha(data->gsGlobal, GS_SETREG_ALPHA(A_COLOR_SOURCE, A_COLOR_NULL, A_ALPHA_FIX, A_COLOR_DEST, 0x80), 0);
         data->gsGlobal->PrimAlphaEnable = GS_SETTING_ON;
         break;
     }
+    case SDL_BLENDMODE_ADD_PREMULTIPLIED:
+    {
+        // FIXME: What are the settings for this?
+        gsKit_set_primalpha(data->gsGlobal, GS_SETREG_ALPHA(A_COLOR_SOURCE, A_COLOR_NULL, A_ALPHA_FIX, A_COLOR_DEST, 0x80), 0);
+        data->gsGlobal->PrimAlphaEnable = GS_SETTING_ON;
+        break;
+    }
     case SDL_BLENDMODE_MUL:
     case SDL_BLENDMODE_MOD:
     {
-        /* We don't fully support MOD and MUL, however this is the best we can do */
+        // We don't fully support MOD and MUL, however this is the best we can do
         gsKit_set_primalpha(data->gsGlobal, GS_SETREG_ALPHA(A_COLOR_DEST, A_COLOR_NULL, A_ALPHA_SOURCE, A_COLOR_SOURCE, 0x80), 0);
         data->gsGlobal->PrimAlphaEnable = GS_SETTING_ON;
         break;
@@ -423,16 +447,16 @@ static void PS2_SetBlendMode(PS2_RenderData *data, int blendMode)
     }
 }
 
-static int PS2_RenderGeometry(SDL_Renderer *renderer, void *vertices, SDL_RenderCommand *cmd)
+static bool PS2_RenderGeometry(SDL_Renderer *renderer, void *vertices, SDL_RenderCommand *cmd)
 {
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
     const size_t count = cmd->data.draw.count;
 
     PS2_SetBlendMode(data, cmd->data.draw.blend);
 
     if (cmd->data.draw.texture) {
         const GSPRIMUVPOINT *verts = (GSPRIMUVPOINT *) (vertices + cmd->data.draw.first);
-        GSTEXTURE *ps2_tex = (GSTEXTURE *)cmd->data.draw.texture->driverdata;
+        GSTEXTURE *ps2_tex = (GSTEXTURE *)cmd->data.draw.texture->internal;
 
         gsKit_TexManager_bind(data->gsGlobal, ps2_tex);
         gsKit_prim_list_triangle_goraud_texture_uv_3d(data->gsGlobal, ps2_tex, count, verts);
@@ -441,43 +465,48 @@ static int PS2_RenderGeometry(SDL_Renderer *renderer, void *vertices, SDL_Render
         gsKit_prim_list_triangle_gouraud_3d(data->gsGlobal, count, verts);
     }
 
-    return 0;
+    return true;
 }
 
-int PS2_RenderLines(SDL_Renderer *renderer, void *vertices, SDL_RenderCommand *cmd)
+static bool PS2_RenderLines(SDL_Renderer *renderer, void *vertices, SDL_RenderCommand *cmd)
 {
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
     const size_t count = cmd->data.draw.count;
     const GSPRIMPOINT *verts = (GSPRIMPOINT *)(vertices + cmd->data.draw.first);
 
     PS2_SetBlendMode(data, cmd->data.draw.blend);
     gsKit_prim_list_line_goraud_3d(data->gsGlobal, count, verts);
 
-    /* We're done! */
-    return 0;
+    // We're done!
+    return true;
 }
 
-int PS2_RenderPoints(SDL_Renderer *renderer, void *vertices, SDL_RenderCommand *cmd)
+static bool PS2_RenderPoints(SDL_Renderer *renderer, void *vertices, SDL_RenderCommand *cmd)
 {
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
     const size_t count = cmd->data.draw.count;
     const GSPRIMPOINT *verts = (GSPRIMPOINT *)(vertices + cmd->data.draw.first);
 
     PS2_SetBlendMode(data, cmd->data.draw.blend);
     gsKit_prim_list_points(data->gsGlobal, count, verts);
 
-    /* We're done! */
-    return 0;
+    // We're done!
+    return true;
 }
 
-static int PS2_RunCommandQueue(SDL_Renderer *renderer, SDL_RenderCommand *cmd, void *vertices, size_t vertsize)
+static void PS2_InvalidateCachedState(SDL_Renderer *renderer)
+{
+    // currently this doesn't do anything. If this needs to do something (and someone is mixing their own rendering calls in!), update this.
+}
+
+static bool PS2_RunCommandQueue(SDL_Renderer *renderer, SDL_RenderCommand *cmd, void *vertices, size_t vertsize)
 {
     while (cmd) {
         switch (cmd->command) {
         case SDL_RENDERCMD_SETVIEWPORT:
         {
             PS2_RenderSetViewPort(renderer, cmd);
-            /* FIXME: We need to update the clip rect too, see https://github.com/libsdl-org/SDL/issues/9094 */
+            // FIXME: We need to update the clip rect too, see https://github.com/libsdl-org/SDL/issues/9094
             break;
         }
         case SDL_RENDERCMD_SETCLIPRECT:
@@ -505,11 +534,11 @@ static int PS2_RunCommandQueue(SDL_Renderer *renderer, SDL_RenderCommand *cmd, v
             PS2_RenderLines(renderer, vertices, cmd);
             break;
         }
-        case SDL_RENDERCMD_FILL_RECTS: /* unused */
+        case SDL_RENDERCMD_FILL_RECTS: // unused
             break;
-        case SDL_RENDERCMD_COPY: /* unused */
+        case SDL_RENDERCMD_COPY: // unused
             break;
-        case SDL_RENDERCMD_COPY_EX: /* unused */
+        case SDL_RENDERCMD_COPY_EX: // unused
             break;
         case SDL_RENDERCMD_GEOMETRY:
         {
@@ -521,21 +550,15 @@ static int PS2_RunCommandQueue(SDL_Renderer *renderer, SDL_RenderCommand *cmd, v
         }
         cmd = cmd->next;
     }
-    return 0;
+    return true;
 }
 
-static int PS2_RenderReadPixels(SDL_Renderer *renderer, const SDL_Rect *rect,
-                                Uint32 format, void *pixels, int pitch)
+static bool PS2_RenderPresent(SDL_Renderer *renderer)
 {
-    return SDL_Unsupported();
-}
-
-static int PS2_RenderPresent(SDL_Renderer *renderer)
-{
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
 
     if (data->gsGlobal->DoubleBuffering == GS_SETTING_OFF) {
-        if (data->vsync == 2) { // Dynamic
+        if (data->vsync == -1) { // Dynamic
             gsKit_sync(data->gsGlobal);
         } else if (data->vsync == 1) {
             gsKit_vsync_wait();
@@ -544,7 +567,7 @@ static int PS2_RenderPresent(SDL_Renderer *renderer)
     } else {
         gsKit_queue_exec(data->gsGlobal);
         gsKit_finish();
-        if (data->vsync == 2) { // Dynamic
+        if (data->vsync == -1) { // Dynamic
             gsKit_sync(data->gsGlobal);
         } else if (data->vsync == 1) {
             gsKit_vsync_wait();
@@ -553,13 +576,13 @@ static int PS2_RenderPresent(SDL_Renderer *renderer)
     }
     gsKit_TexManager_nextFrame(data->gsGlobal);
     gsKit_clear(data->gsGlobal, GS_BLACK);
-    return 0;
+    return true;
 }
 
 static void PS2_DestroyTexture(SDL_Renderer *renderer, SDL_Texture *texture)
 {
-    GSTEXTURE *ps2_texture = (GSTEXTURE *)texture->driverdata;
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    GSTEXTURE *ps2_texture = (GSTEXTURE *)texture->internal;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
 
     if (!data) {
         return;
@@ -572,14 +595,14 @@ static void PS2_DestroyTexture(SDL_Renderer *renderer, SDL_Texture *texture)
     // Free from vram
     gsKit_TexManager_free(data->gsGlobal, ps2_texture);
 
-    SDL_free(ps2_texture->Mem);
+    SDL_aligned_free(ps2_texture->Mem);
     SDL_free(ps2_texture);
-    texture->driverdata = NULL;
+    texture->internal = NULL;
 }
 
 static void PS2_DestroyRenderer(SDL_Renderer *renderer)
 {
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
 
     if (data) {
         gsKit_clear(data->gsGlobal, GS_BLACK);
@@ -593,40 +616,42 @@ static void PS2_DestroyRenderer(SDL_Renderer *renderer)
     if (vsync_sema_id >= 0) {
         DeleteSema(vsync_sema_id);
     }
-
-    SDL_free(renderer);
 }
 
-static int PS2_SetVSync(SDL_Renderer *renderer, const int vsync)
+static bool PS2_SetVSync(SDL_Renderer *renderer, const int vsync)
 {
-    PS2_RenderData *data = (PS2_RenderData *)renderer->driverdata;
-    SDL_bool dynamicVsync = SDL_GetHintBoolean(SDL_HINT_PS2_DYNAMIC_VSYNC, SDL_FALSE);
-    data->vsync = vsync ? (dynamicVsync ? 2 : 1) : 0;
-    return 0;
+    PS2_RenderData *data = (PS2_RenderData *)renderer->internal;
+    switch (vsync) {
+    case -1:
+    case 0:
+    case 1:
+        // Supported
+        break;
+    default:
+        return SDL_Unsupported();
+    }
+    data->vsync = vsync;
+    return true;
 }
 
-static SDL_Renderer *PS2_CreateRenderer(SDL_Window *window, Uint32 flags)
+static bool PS2_CreateRenderer(SDL_Renderer *renderer, SDL_Window *window, SDL_PropertiesID create_props)
 {
-    SDL_Renderer *renderer;
     PS2_RenderData *data;
     GSGLOBAL *gsGlobal;
     ee_sema_t sema;
-    SDL_bool dynamicVsync;
 
-    renderer = (SDL_Renderer *)SDL_calloc(1, sizeof(*renderer));
-    if (!renderer) {
-        SDL_OutOfMemory();
-        return NULL;
+    SDL_SetupRendererColorspace(renderer, create_props);
+
+    if (renderer->output_colorspace != SDL_COLORSPACE_SRGB) {
+        return SDL_SetError("Unsupported output colorspace");
     }
 
     data = (PS2_RenderData *)SDL_calloc(1, sizeof(*data));
     if (!data) {
-        PS2_DestroyRenderer(renderer);
-        SDL_OutOfMemory();
-        return NULL;
+        return false;
     }
 
-    /* Specific gsKit init */
+    // Specific gsKit init
     sema.init_count = 0;
     sema.max_count = 1;
     sema.option = 0;
@@ -664,8 +689,6 @@ static SDL_Renderer *PS2_CreateRenderer(SDL_Window *window, Uint32 flags)
     gsKit_clear(gsGlobal, GS_BLACK);
 
     data->gsGlobal = gsGlobal;
-    dynamicVsync = SDL_GetHintBoolean(SDL_HINT_PS2_DYNAMIC_VSYNC, SDL_FALSE);
-    data->vsync = flags & SDL_RENDERER_PRESENTVSYNC ? (dynamicVsync ? 2 : 1) : 0;
 
     renderer->WindowEvent = PS2_WindowEvent;
     renderer->CreateTexture = PS2_CreateTexture;
@@ -675,38 +698,30 @@ static SDL_Renderer *PS2_CreateRenderer(SDL_Window *window, Uint32 flags)
     renderer->SetTextureScaleMode = PS2_SetTextureScaleMode;
     renderer->SetRenderTarget = PS2_SetRenderTarget;
     renderer->QueueSetViewport = PS2_QueueSetViewport;
-    renderer->QueueSetDrawColor = PS2_QueueSetDrawColor;
+    renderer->QueueSetDrawColor = PS2_QueueNoOp;
     renderer->QueueDrawPoints = PS2_QueueDrawPoints;
     renderer->QueueDrawLines = PS2_QueueDrawPoints;
     renderer->QueueGeometry = PS2_QueueGeometry;
+    renderer->InvalidateCachedState = PS2_InvalidateCachedState;
     renderer->RunCommandQueue = PS2_RunCommandQueue;
-    renderer->RenderReadPixels = PS2_RenderReadPixels;
     renderer->RenderPresent = PS2_RenderPresent;
     renderer->DestroyTexture = PS2_DestroyTexture;
     renderer->DestroyRenderer = PS2_DestroyRenderer;
     renderer->SetVSync = PS2_SetVSync;
-    renderer->info = PS2_RenderDriver.info;
-    renderer->driverdata = data;
+    renderer->internal = data;
+    PS2_InvalidateCachedState(renderer);
     renderer->window = window;
 
-    return renderer;
+    renderer->name = PS2_RenderDriver.name;
+    SDL_AddSupportedTextureFormat(renderer, SDL_PIXELFORMAT_ABGR1555);
+    SDL_AddSupportedTextureFormat(renderer, SDL_PIXELFORMAT_ABGR8888);
+    SDL_SetNumberProperty(SDL_GetRendererProperties(renderer), SDL_PROP_RENDERER_MAX_TEXTURE_SIZE_NUMBER, 1024);
+
+    return true;
 }
 
 SDL_RenderDriver PS2_RenderDriver = {
-    .CreateRenderer = PS2_CreateRenderer,
-    .info = {
-        .name = "PS2 gsKit",
-        .flags = SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC | SDL_RENDERER_TARGETTEXTURE,
-        .num_texture_formats = 2,
-        .texture_formats = {
-            [0] = SDL_PIXELFORMAT_ABGR1555,
-            [1] = SDL_PIXELFORMAT_ABGR8888,
-        },
-        .max_texture_width = 1024,
-        .max_texture_height = 1024,
-    }
+    PS2_CreateRenderer, "PS2 gsKit"
 };
 
-#endif /* SDL_VIDEO_RENDER_PS2 */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_RENDER_PS2
