@@ -22,11 +22,9 @@
 #ifndef SDL_blendpoint_h_
 #define SDL_blendpoint_h_
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-extern int SDL_BlendPoint(SDL_Surface *dst, int x, int y, SDL_BlendMode blendMode, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
-extern int SDL_BlendPoints(SDL_Surface *dst, const SDL_Point *points, int count, SDL_BlendMode blendMode, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
+extern bool SDL_BlendPoint(SDL_Surface *dst, int x, int y, SDL_BlendMode blendMode, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
+extern bool SDL_BlendPoints(SDL_Surface *dst, const SDL_Point *points, int count, SDL_BlendMode blendMode, Uint8 r, Uint8 g, Uint8 b, Uint8 a);
 
-#endif /* SDL_blendpoint_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_blendpoint_h_
