@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,22 +18,15 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
+#include "SDL_internal.h"
 
-#if defined(__clang_analyzer__) && !defined(SDL_DISABLE_ANALYZE_MACROS)
-#define SDL_DISABLE_ANALYZE_MACROS 1
-#endif
+// This file contains portable stdlib functions for SDL
 
-#include "../SDL_internal.h"
-
-/* This file contains portable stdlib functions for SDL */
-
-#include "SDL_stdinc.h"
 #include "../libm/math_libm.h"
 
-double
-SDL_atan(double x)
+double SDL_atan(double x)
 {
-#if defined(HAVE_ATAN)
+#ifdef HAVE_ATAN
     return atan(x);
 #else
     return SDL_uclibc_atan(x);
@@ -42,17 +35,16 @@ SDL_atan(double x)
 
 float SDL_atanf(float x)
 {
-#if defined(HAVE_ATANF)
+#ifdef HAVE_ATANF
     return atanf(x);
 #else
     return (float)SDL_atan((double)x);
 #endif
 }
 
-double
-SDL_atan2(double y, double x)
+double SDL_atan2(double y, double x)
 {
-#if defined(HAVE_ATAN2)
+#ifdef HAVE_ATAN2
     return atan2(y, x);
 #else
     return SDL_uclibc_atan2(y, x);
@@ -61,27 +53,25 @@ SDL_atan2(double y, double x)
 
 float SDL_atan2f(float y, float x)
 {
-#if defined(HAVE_ATAN2F)
+#ifdef HAVE_ATAN2F
     return atan2f(y, x);
 #else
     return (float)SDL_atan2((double)y, (double)x);
 #endif
 }
 
-double
-SDL_acos(double val)
+double SDL_acos(double val)
 {
-#if defined(HAVE_ACOS)
+#ifdef HAVE_ACOS
     return acos(val);
 #else
     double result;
     if (val == -1.0) {
-        result = M_PI;
+        result = SDL_PI_D;
     } else {
         result = SDL_atan(SDL_sqrt(1.0 - val * val) / val);
-        if (result < 0.0)
-        {
-            result += M_PI;
+        if (result < 0.0) {
+            result += SDL_PI_D;
         }
     }
     return result;
@@ -90,24 +80,23 @@ SDL_acos(double val)
 
 float SDL_acosf(float val)
 {
-#if defined(HAVE_ACOSF)
+#ifdef HAVE_ACOSF
     return acosf(val);
 #else
     return (float)SDL_acos((double)val);
 #endif
 }
 
-double
-SDL_asin(double val)
+double SDL_asin(double val)
 {
-#if defined(HAVE_ASIN)
+#ifdef HAVE_ASIN
     return asin(val);
 #else
     double result;
     if (val == -1.0) {
-        result = -(M_PI / 2.0);
+        result = -(SDL_PI_D / 2.0);
     } else {
-        result = (M_PI / 2.0) - SDL_acos(val);
+        result = (SDL_PI_D / 2.0) - SDL_acos(val);
     }
     return result;
 #endif
@@ -115,17 +104,16 @@ SDL_asin(double val)
 
 float SDL_asinf(float val)
 {
-#if defined(HAVE_ASINF)
+#ifdef HAVE_ASINF
     return asinf(val);
 #else
     return (float)SDL_asin((double)val);
 #endif
 }
 
-double
-SDL_ceil(double x)
+double SDL_ceil(double x)
 {
-#if defined(HAVE_CEIL)
+#ifdef HAVE_CEIL
     return ceil(x);
 #else
     double integer = SDL_floor(x);
@@ -134,49 +122,47 @@ SDL_ceil(double x)
         integer += 1.0;
     }
     return integer;
-#endif /* HAVE_CEIL */
+#endif // HAVE_CEIL
 }
 
 float SDL_ceilf(float x)
 {
-#if defined(HAVE_CEILF)
+#ifdef HAVE_CEILF
     return ceilf(x);
 #else
     return (float)SDL_ceil((double)x);
 #endif
 }
 
-double
-SDL_copysign(double x, double y)
+double SDL_copysign(double x, double y)
 {
-#if defined(HAVE_COPYSIGN)
+#ifdef HAVE_COPYSIGN
     return copysign(x, y);
 #elif defined(HAVE__COPYSIGN)
     return _copysign(x, y);
 #elif defined(__WATCOMC__) && defined(__386__)
-    /* this is nasty as hell, but it works.. */
+    // this is nasty as hell, but it works..
     unsigned int *xi = (unsigned int *)&x,
                  *yi = (unsigned int *)&y;
     xi[1] = (yi[1] & 0x80000000) | (xi[1] & 0x7fffffff);
     return x;
 #else
     return SDL_uclibc_copysign(x, y);
-#endif /* HAVE_COPYSIGN */
+#endif // HAVE_COPYSIGN
 }
 
 float SDL_copysignf(float x, float y)
 {
-#if defined(HAVE_COPYSIGNF)
+#ifdef HAVE_COPYSIGNF
     return copysignf(x, y);
 #else
     return (float)SDL_copysign((double)x, (double)y);
 #endif
 }
 
-double
-SDL_cos(double x)
+double SDL_cos(double x)
 {
-#if defined(HAVE_COS)
+#ifdef HAVE_COS
     return cos(x);
 #else
     return SDL_uclibc_cos(x);
@@ -185,17 +171,16 @@ SDL_cos(double x)
 
 float SDL_cosf(float x)
 {
-#if defined(HAVE_COSF)
+#ifdef HAVE_COSF
     return cosf(x);
 #else
     return (float)SDL_cos((double)x);
 #endif
 }
 
-double
-SDL_exp(double x)
+double SDL_exp(double x)
 {
-#if defined(HAVE_EXP)
+#ifdef HAVE_EXP
     return exp(x);
 #else
     return SDL_uclibc_exp(x);
@@ -204,17 +189,16 @@ SDL_exp(double x)
 
 float SDL_expf(float x)
 {
-#if defined(HAVE_EXPF)
+#ifdef HAVE_EXPF
     return expf(x);
 #else
     return (float)SDL_exp((double)x);
 #endif
 }
 
-double
-SDL_fabs(double x)
+double SDL_fabs(double x)
 {
-#if defined(HAVE_FABS)
+#ifdef HAVE_FABS
     return fabs(x);
 #else
     return SDL_uclibc_fabs(x);
@@ -223,17 +207,16 @@ SDL_fabs(double x)
 
 float SDL_fabsf(float x)
 {
-#if defined(HAVE_FABSF)
+#ifdef HAVE_FABSF
     return fabsf(x);
 #else
     return (float)SDL_fabs((double)x);
 #endif
 }
 
-double
-SDL_floor(double x)
+double SDL_floor(double x)
 {
-#if defined(HAVE_FLOOR)
+#ifdef HAVE_FLOOR
     return floor(x);
 #else
     return SDL_uclibc_floor(x);
@@ -242,17 +225,16 @@ SDL_floor(double x)
 
 float SDL_floorf(float x)
 {
-#if defined(HAVE_FLOORF)
+#ifdef HAVE_FLOORF
     return floorf(x);
 #else
     return (float)SDL_floor((double)x);
 #endif
 }
 
-double
-SDL_trunc(double x)
+double SDL_trunc(double x)
 {
-#if defined(HAVE_TRUNC)
+#ifdef HAVE_TRUNC
     return trunc(x);
 #else
     if (x >= 0.0f) {
@@ -265,17 +247,16 @@ SDL_trunc(double x)
 
 float SDL_truncf(float x)
 {
-#if defined(HAVE_TRUNCF)
+#ifdef HAVE_TRUNCF
     return truncf(x);
 #else
     return (float)SDL_trunc((double)x);
 #endif
 }
 
-double
-SDL_fmod(double x, double y)
+double SDL_fmod(double x, double y)
 {
-#if defined(HAVE_FMOD)
+#ifdef HAVE_FMOD
     return fmod(x, y);
 #else
     return SDL_uclibc_fmod(x, y);
@@ -284,17 +265,56 @@ SDL_fmod(double x, double y)
 
 float SDL_fmodf(float x, float y)
 {
-#if defined(HAVE_FMODF)
+#ifdef HAVE_FMODF
     return fmodf(x, y);
 #else
     return (float)SDL_fmod((double)x, (double)y);
 #endif
 }
 
-double
-SDL_log(double x)
+int SDL_isinf(double x)
 {
-#if defined(HAVE_LOG)
+#ifdef HAVE_ISINF
+    return isinf(x);
+#else
+    return SDL_uclibc_isinf(x);
+#endif
+}
+
+int SDL_isinff(float x)
+{
+#ifdef HAVE_ISINF_FLOAT_MACRO
+    return isinf(x);
+#elif defined(HAVE_ISINFF)
+    return isinff(x);
+#else
+    return SDL_uclibc_isinff(x);
+#endif
+}
+
+int SDL_isnan(double x)
+{
+#ifdef HAVE_ISNAN
+    return isnan(x);
+#else
+    return SDL_uclibc_isnan(x);
+#endif
+}
+
+int SDL_isnanf(float x)
+{
+#ifdef HAVE_ISNAN_FLOAT_MACRO
+    return isnan(x);
+#elif defined(HAVE_ISNANF)
+    return isnanf(x);
+#else
+    return SDL_uclibc_isnanf(x);
+#endif
+}
+
+double SDL_log(double x)
+{
+#ifdef HAVE_LOG
     return log(x);
 #else
     return SDL_uclibc_log(x);
@@ -303,17 +323,16 @@ SDL_log(double x)
 
 float SDL_logf(float x)
 {
-#if defined(HAVE_LOGF)
+#ifdef HAVE_LOGF
     return logf(x);
 #else
     return (float)SDL_log((double)x);
 #endif
 }
 
-double
-SDL_log10(double x)
+double SDL_log10(double x)
 {
-#if defined(HAVE_LOG10)
+#ifdef HAVE_LOG10
     return log10(x);
 #else
     return SDL_uclibc_log10(x);
@@ -322,17 +341,37 @@ SDL_log10(double x)
 
 float SDL_log10f(float x)
 {
-#if defined(HAVE_LOG10F)
+#ifdef HAVE_LOG10F
     return log10f(x);
 #else
     return (float)SDL_log10((double)x);
 #endif
 }
 
-double
-SDL_pow(double x, double y)
+double SDL_modf(double x, double *y)
 {
-#if defined(HAVE_POW)
+#ifdef HAVE_MODF
+    return modf(x, y);
+#else
+    return SDL_uclibc_modf(x, y);
+#endif
+}
+
+float SDL_modff(float x, float *y)
+{
+#ifdef HAVE_MODFF
+    return modff(x, y);
+#else
+    double double_result, double_y;
+    double_result = SDL_modf((double)x, &double_y);
+    *y = (float)double_y;
+    return (float)double_result;
+#endif
+}
+
+double SDL_pow(double x, double y)
+{
+#ifdef HAVE_POW
     return pow(x, y);
 #else
     return SDL_uclibc_pow(x, y);
@@ -341,15 +380,14 @@ SDL_pow(double x, double y)
 
 float SDL_powf(float x, float y)
 {
-#if defined(HAVE_POWF)
+#ifdef HAVE_POWF
     return powf(x, y);
 #else
     return (float)SDL_pow((double)x, (double)y);
 #endif
 }
 
-double
-SDL_round(double arg)
+double SDL_round(double arg)
 {
 #if defined HAVE_ROUND
     return round(arg);
@@ -389,10 +427,9 @@ long SDL_lroundf(float arg)
 #endif
 }
 
-double
-SDL_scalbn(double x, int n)
+double SDL_scalbn(double x, int n)
 {
-#if defined(HAVE_SCALBN)
+#ifdef HAVE_SCALBN
     return scalbn(x, n);
 #elif defined(HAVE__SCALB)
     return _scalb(x, n);
@@ -407,17 +444,16 @@ SDL_scalbn(double x, int n)
 
 float SDL_scalbnf(float x, int n)
 {
-#if defined(HAVE_SCALBNF)
+#ifdef HAVE_SCALBNF
     return scalbnf(x, n);
 #else
     return (float)SDL_scalbn((double)x, n);
 #endif
 }
 
-double
-SDL_sin(double x)
+double SDL_sin(double x)
 {
-#if defined(HAVE_SIN)
+#ifdef HAVE_SIN
     return sin(x);
 #else
     return SDL_uclibc_sin(x);
@@ -426,17 +462,16 @@ SDL_sin(double x)
 
 float SDL_sinf(float x)
 {
-#if defined(HAVE_SINF)
+#ifdef HAVE_SINF
     return sinf(x);
 #else
     return (float)SDL_sin((double)x);
 #endif
 }
 
-double
-SDL_sqrt(double x)
+double SDL_sqrt(double x)
 {
-#if defined(HAVE_SQRT)
+#ifdef HAVE_SQRT
     return sqrt(x);
 #else
     return SDL_uclibc_sqrt(x);
@@ -445,17 +480,16 @@ SDL_sqrt(double x)
 
 float SDL_sqrtf(float x)
 {
-#if defined(HAVE_SQRTF)
+#ifdef HAVE_SQRTF
     return sqrtf(x);
 #else
     return (float)SDL_sqrt((double)x);
 #endif
 }
 
-double
-SDL_tan(double x)
+double SDL_tan(double x)
 {
-#if defined(HAVE_TAN)
+#ifdef HAVE_TAN
     return tan(x);
 #else
     return SDL_uclibc_tan(x);
@@ -464,7 +498,7 @@ SDL_tan(double x)
 
 float SDL_tanf(float x)
 {
-#if defined(HAVE_TANF)
+#ifdef HAVE_TANF
     return tanf(x);
 #else
     return (float)SDL_tan((double)x);
@@ -473,35 +507,14 @@ float SDL_tanf(float x)
 
 int SDL_abs(int x)
 {
-#if defined(HAVE_ABS)
+#ifdef HAVE_ABS
     return abs(x);
 #else
     return (x < 0) ? -x : x;
 #endif
 }
 
-#if defined(HAVE_CTYPE_H)
-int SDL_isalpha(int x)
-{
-    return isalpha(x);
-}
-int SDL_isalnum(int x) { return isalnum(x); }
-int SDL_isdigit(int x) { return isdigit(x); }
-int SDL_isxdigit(int x) { return isxdigit(x); }
-int SDL_ispunct(int x) { return ispunct(x); }
-int SDL_isspace(int x) { return isspace(x); }
-int SDL_isupper(int x) { return isupper(x); }
-int SDL_islower(int x) { return islower(x); }
-int SDL_isprint(int x) { return isprint(x); }
-int SDL_isgraph(int x) { return isgraph(x); }
-int SDL_iscntrl(int x) { return iscntrl(x); }
-int SDL_toupper(int x) { return toupper(x); }
-int SDL_tolower(int x) { return tolower(x); }
-#else
-int SDL_isalpha(int x)
-{
-    return (SDL_isupper(x)) || (SDL_islower(x));
-}
+int SDL_isalpha(int x) { return (SDL_isupper(x)) || (SDL_islower(x)); }
 int SDL_isalnum(int x) { return (SDL_isalpha(x)) || (SDL_isdigit(x)); }
 int SDL_isdigit(int x) { return ((x) >= '0') && ((x) <= '9'); }
 int SDL_isxdigit(int x) { return (((x) >= 'A') && ((x) <= 'F')) || (((x) >= 'a') && ((x) <= 'f')) || (SDL_isdigit(x)); }
@@ -514,120 +527,41 @@ int SDL_isgraph(int x) { return (SDL_isprint(x)) && ((x) != ' '); }
 int SDL_iscntrl(int x) { return (((x) >= '\0') && ((x) <= '\x1f')) || ((x) == '\x7f'); }
 int SDL_toupper(int x) { return ((x) >= 'a') && ((x) <= 'z') ? ('A' + ((x) - 'a')) : (x); }
 int SDL_tolower(int x) { return ((x) >= 'A') && ((x) <= 'Z') ? ('a' + ((x) - 'A')) : (x); }
-#endif
+int SDL_isblank(int x) { return ((x) == ' ') || ((x) == '\t'); }
 
-/* This file contains a portable memcpy manipulation function for SDL */
-
-void *SDL_memcpy(SDL_OUT_BYTECAP(len) void *dst, SDL_IN_BYTECAP(len) const void *src, size_t len)
+void *SDL_aligned_alloc(size_t alignment, size_t size)
 {
-#ifdef __GNUC__
-    /* Presumably this is well tuned for speed.
-       On my machine this is twice as fast as the C code below.
-     */
-    return __builtin_memcpy(dst, src, len);
-#elif defined(HAVE_MEMCPY)
-    return memcpy(dst, src, len);
-#elif defined(HAVE_BCOPY)
-    bcopy(src, dst, len);
-    return dst;
-#else
-    /* GCC 4.9.0 with -O3 will generate movaps instructions with the loop
-       using Uint32* pointers, so we need to make sure the pointers are
-       aligned before we loop using them.
-     */
-    if (((uintptr_t)src & 0x3) || ((uintptr_t)dst & 0x3)) {
-        /* Do an unaligned byte copy */
-        Uint8 *srcp1 = (Uint8 *)src;
-        Uint8 *dstp1 = (Uint8 *)dst;
+    size_t padding;
+    Uint8 *result = NULL;
 
-        while (len--) {
-            *dstp1++ = *srcp1++;
-        }
-    } else {
-        size_t left = (len % 4);
-        Uint32 *srcp4, *dstp4;
-        Uint8 *srcp1, *dstp1;
+    if (alignment < sizeof(void*)) {
+        alignment = sizeof(void*);
+    }
+    padding = (alignment - (size % alignment));
 
-        srcp4 = (Uint32 *)src;
-        dstp4 = (Uint32 *)dst;
-        len /= 4;
-        while (len--) {
-            *dstp4++ = *srcp4++;
-        }
+    if (SDL_size_add_check_overflow(size, alignment, &size) &&
+        SDL_size_add_check_overflow(size, sizeof(void *), &size) &&
+        SDL_size_add_check_overflow(size, padding, &size)) {
+        void *original = SDL_malloc(size);
+        if (original) {
+            // Make sure we have enough space to store the original pointer
+            result = (Uint8 *)original + sizeof(original);
 
-        srcp1 = (Uint8 *)srcp4;
-        dstp1 = (Uint8 *)dstp4;
-        switch (left) {
-        case 3:
-            *dstp1++ = *srcp1++;
-        case 2:
-            *dstp1++ = *srcp1++;
-        case 1:
-            *dstp1++ = *srcp1++;
+            // Align the pointer we're going to return
+            result += alignment - (((size_t)result) % alignment);
+
+            // Store the original pointer right before the returned value
+            SDL_memcpy(result - sizeof(original), &original, sizeof(original));
         }
     }
-    return dst;
-#endif /* __GNUC__ */
+    return result;
 }
 
-void *SDL_memset(SDL_OUT_BYTECAP(len) void *dst, int c, size_t len)
+void SDL_aligned_free(void *mem)
 {
-#if defined(HAVE_MEMSET)
-    return memset(dst, c, len);
-#else
-    size_t left;
-    Uint32 *dstp4;
-    Uint8 *dstp1 = (Uint8 *)dst;
-    Uint8 value1;
-    Uint32 value4;
-
-    /* The value used in memset() is a byte, passed as an int */
-    c &= 0xff;
-
-    /* The destination pointer needs to be aligned on a 4-byte boundary to
-     * execute a 32-bit set. Set first bytes manually if needed until it is
-     * aligned. */
-    value1 = (Uint8)c;
-    while ((uintptr_t)dstp1 & 0x3) {
-        if (len--) {
-            *dstp1++ = value1;
-        } else {
-            return dst;
-        }
+    if (mem) {
+        void *original;
+        SDL_memcpy(&original, ((Uint8 *)mem - sizeof(original)), sizeof(original));
+        SDL_free(original);
     }
-
-    value4 = ((Uint32)c | ((Uint32)c << 8) | ((Uint32)c << 16) | ((Uint32)c << 24));
-    dstp4 = (Uint32 *)dstp1;
-    left = (len % 4);
-    len /= 4;
-    while (len--) {
-        *dstp4++ = value4;
-    }
-
-    dstp1 = (Uint8 *)dstp4;
-    switch (left) {
-    case 3:
-        *dstp1++ = value1;
-    case 2:
-        *dstp1++ = value1;
-    case 1:
-        *dstp1++ = value1;
-    }
-
-    return dst;
-#endif /* HAVE_MEMSET */
 }
-
-#if defined(HAVE_CTYPE_H) && defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
-int SDL_isblank(int x)
-{
-    return isblank(x);
-}
-#else
-int SDL_isblank(int x)
-{
-    return ((x) == ' ') || ((x) == '\t');
-}
-#endif
-
-/* vi: set ts=4 sw=4 expandtab: */
