@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,20 +18,16 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_sysmutex_c_h_
 #define SDL_sysmutex_c_h_
 
 #include <3ds.h>
 
-#include "SDL_mutex.h"
-
-struct SDL_mutex
+struct SDL_Mutex
 {
     RecursiveLock lock;
 };
 
-#endif /* SDL_sysmutex_c_h */
-
-/* vi: set sts=4 ts=4 sw=4 expandtab: */
+#endif // SDL_sysmutex_c_h
