@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #include <sys/time.h>
 #include <time.h>
@@ -26,20 +26,19 @@
 #include <errno.h>
 #include <pthread.h>
 
-#include "SDL_thread.h"
 #include "SDL_sysmutex_c.h"
 
-struct SDL_cond
+struct SDL_Condition
 {
     pthread_cond_t cond;
 };
 
-/* Create a condition variable */
-SDL_cond *SDL_CreateCond(void)
+// Create a condition variable
+SDL_Condition *SDL_CreateCondition(void)
 {
-    SDL_cond *cond;
+    SDL_Condition *cond;
 
-    cond = (SDL_cond *)SDL_malloc(sizeof(SDL_cond));
+    cond = (SDL_Condition *)SDL_malloc(sizeof(SDL_Condition));
     if (cond) {
         if (pthread_cond_init(&cond->cond, NULL) != 0) {
             SDL_SetError("pthread_cond_init() failed");
@@ -50,8 +49,8 @@ SDL_cond *SDL_CreateCond(void)
     return cond;
 }
 
-/* Destroy a condition variable */
-void SDL_DestroyCond(SDL_cond *cond)
+// Destroy a condition variable
+void SDL_DestroyCondition(SDL_Condition *cond)
 {
     if (cond) {
         pthread_cond_destroy(&cond->cond);
@@ -59,94 +58,71 @@ void SDL_DestroyCond(SDL_cond *cond)
     }
 }
 
-/* Restart one of the threads that are waiting on the condition variable */
-int SDL_CondSignal(SDL_cond *cond)
+// Restart one of the threads that are waiting on the condition variable
+void SDL_SignalCondition(SDL_Condition *cond)
 {
-    int retval;
-
     if (!cond) {
-        return SDL_InvalidParamError("cond");
+        return;
     }
 
-    retval = 0;
-    if (pthread_cond_signal(&cond->cond) != 0) {
-        return SDL_SetError("pthread_cond_signal() failed");
-    }
-    return retval;
+    pthread_cond_signal(&cond->cond);
 }
 
-/* Restart all threads that are waiting on the condition variable */
-int SDL_CondBroadcast(SDL_cond *cond)
+// Restart all threads that are waiting on the condition variable
+void SDL_BroadcastCondition(SDL_Condition *cond)
 {
-    int retval;
-
     if (!cond) {
-        return SDL_InvalidParamError("cond");
+        return;
     }
 
-    retval = 0;
-    if (pthread_cond_broadcast(&cond->cond) != 0) {
-        return SDL_SetError("pthread_cond_broadcast() failed");
-    }
-    return retval;
+    pthread_cond_broadcast(&cond->cond);
 }
 
-int SDL_CondWaitTimeout(SDL_cond *cond, SDL_mutex *mutex, Uint32 ms)
+bool SDL_WaitConditionTimeoutNS(SDL_Condition *cond, SDL_Mutex *mutex, Sint64 timeoutNS)
 {
-    int retval;
 #ifndef HAVE_CLOCK_GETTIME
     struct timeval delta;
 #endif
     struct timespec abstime;
 
-    if (!cond) {
-        return SDL_InvalidParamError("cond");
+    if (!cond || !mutex) {
+        return true;
+    }
+
+    if (timeoutNS < 0) {
+        return (pthread_cond_wait(&cond->cond, &mutex->id) == 0);
     }
 
 #ifdef HAVE_CLOCK_GETTIME
     clock_gettime(CLOCK_REALTIME, &abstime);
 
-    abstime.tv_nsec += (ms % 1000) * 1000000;
-    abstime.tv_sec += ms / 1000;
+    abstime.tv_sec += (timeoutNS / SDL_NS_PER_SECOND);
+    abstime.tv_nsec += (timeoutNS % SDL_NS_PER_SECOND);
 #else
     gettimeofday(&delta, NULL);
 
-    abstime.tv_sec = delta.tv_sec + (ms / 1000);
-    abstime.tv_nsec = (long)(delta.tv_usec + (ms % 1000) * 1000) * 1000;
+    abstime.tv_sec = delta.tv_sec + (timeoutNS / SDL_NS_PER_SECOND);
+    abstime.tv_nsec = SDL_US_TO_NS(delta.tv_usec) + (timeoutNS % SDL_NS_PER_SECOND);
 #endif
-    if (abstime.tv_nsec >= 1000000000) {
+    while (abstime.tv_nsec >= 1000000000) {
         abstime.tv_sec += 1;
         abstime.tv_nsec -= 1000000000;
     }
 
+    bool result;
+    int rc;
 tryagain:
-    retval = pthread_cond_timedwait(&cond->cond, &mutex->id, &abstime);
-    switch (retval) {
+    rc = pthread_cond_timedwait(&cond->cond, &mutex->id, &abstime);
+    switch (rc) {
     case EINTR:
         goto tryagain;
-        /* break; -Wunreachable-code-break */
+        // break; -Wunreachable-code-break
     case ETIMEDOUT:
-        retval = SDL_MUTEX_TIMEDOUT;
-        break;
-    case 0:
+        result = false;
         break;
     default:
-        retval = SDL_SetError("pthread_cond_timedwait() failed");
+        result = true;
+        break;
     }
-    return retval;
+    return result;
 }
-
-/* Wait on the condition variable, unlocking the provided mutex.
-   The mutex must be locked before entering this function!
- */
-int SDL_CondWait(SDL_cond *cond, SDL_mutex *mutex)
-{
-    if (!cond) {
-        return SDL_InvalidParamError("cond");
-    } else if (pthread_cond_wait(&cond->cond, &mutex->id) != 0) {
-        return SDL_SetError("pthread_cond_wait() failed");
-    }
-    return 0;
-}
-
-/* vi: set ts=4 sw=4 expandtab: */
