@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,9 +18,9 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_THREAD_WINDOWS
+#ifdef SDL_THREAD_WINDOWS
 
 /**
  * Mutex functions using the Win32 API
@@ -30,57 +30,44 @@
  * which are chosen at runtime.
  */
 
-
-#include "SDL_hints.h"
-
 #include "SDL_sysmutex_c.h"
 
-/* Implementation will be chosen at runtime based on available Kernel features */
+// Implementation will be chosen at runtime based on available Kernel features
 SDL_mutex_impl_t SDL_mutex_impl_active = { 0 };
 
 /**
  * Implementation based on Slim Reader/Writer (SRW) Locks for Win 7 and newer.
  */
 
-#if __WINRT__
-/* Functions are guaranteed to be available */
-#define pReleaseSRWLockExclusive    ReleaseSRWLockExclusive
-#define pAcquireSRWLockExclusive    AcquireSRWLockExclusive
-#define pTryAcquireSRWLockExclusive TryAcquireSRWLockExclusive
-#else
+typedef VOID(WINAPI *pfnInitializeSRWLock)(PSRWLOCK);
 typedef VOID(WINAPI *pfnReleaseSRWLockExclusive)(PSRWLOCK);
 typedef VOID(WINAPI *pfnAcquireSRWLockExclusive)(PSRWLOCK);
 typedef BOOLEAN(WINAPI *pfnTryAcquireSRWLockExclusive)(PSRWLOCK);
+static pfnInitializeSRWLock pInitializeSRWLock = NULL;
 static pfnReleaseSRWLockExclusive pReleaseSRWLockExclusive = NULL;
 static pfnAcquireSRWLockExclusive pAcquireSRWLockExclusive = NULL;
 static pfnTryAcquireSRWLockExclusive pTryAcquireSRWLockExclusive = NULL;
-#endif
 
-static SDL_mutex *SDL_CreateMutex_srw(void)
+static SDL_Mutex *SDL_CreateMutex_srw(void)
 {
-    SDL_mutex_srw *mutex;
-
-    /* Relies on SRWLOCK_INIT == 0. */
-    mutex = (SDL_mutex_srw *)SDL_calloc(1, sizeof(*mutex));
-    if (mutex == NULL) {
-        SDL_OutOfMemory();
+    SDL_mutex_srw *mutex = (SDL_mutex_srw *)SDL_calloc(1, sizeof(*mutex));
+    if (mutex) {
+        pInitializeSRWLock(&mutex->srw);
     }
-
-    return (SDL_mutex *)mutex;
+    return (SDL_Mutex *)mutex;
 }
 
-static void SDL_DestroyMutex_srw(SDL_mutex *mutex)
+static void SDL_DestroyMutex_srw(SDL_Mutex *mutex)
 {
-    /* There are no kernel allocated resources */
+    // There are no kernel allocated resources
     SDL_free(mutex);
 }
 
-static int SDL_LockMutex_srw(SDL_mutex *_mutex) SDL_NO_THREAD_SAFETY_ANALYSIS /* clang doesn't know about NULL mutexes */
+static void SDL_LockMutex_srw(SDL_Mutex *_mutex) SDL_NO_THREAD_SAFETY_ANALYSIS  // clang doesn't know about NULL mutexes
 {
     SDL_mutex_srw *mutex = (SDL_mutex_srw *)_mutex;
-    DWORD this_thread;
+    const DWORD this_thread = GetCurrentThreadId();
 
-    this_thread = GetCurrentThreadId();
     if (mutex->owner == this_thread) {
         ++mutex->count;
     } else {
@@ -93,16 +80,14 @@ static int SDL_LockMutex_srw(SDL_mutex *_mutex) SDL_NO_THREAD_SAFETY_ANALYSIS /*
         mutex->owner = this_thread;
         mutex->count = 1;
     }
-    return 0;
 }
 
-static int SDL_TryLockMutex_srw(SDL_mutex *_mutex)
+static bool SDL_TryLockMutex_srw(SDL_Mutex *_mutex)
 {
     SDL_mutex_srw *mutex = (SDL_mutex_srw *)_mutex;
-    DWORD this_thread;
-    int retval = 0;
+    const DWORD this_thread = GetCurrentThreadId();
+    bool retval = true;
 
-    this_thread = GetCurrentThreadId();
     if (mutex->owner == this_thread) {
         ++mutex->count;
     } else {
@@ -111,13 +96,13 @@ static int SDL_TryLockMutex_srw(SDL_mutex *_mutex)
             mutex->owner = this_thread;
             mutex->count = 1;
         } else {
-            retval = SDL_MUTEX_TIMEDOUT;
+            retval = false;
         }
     }
     return retval;
 }
 
-static int SDL_UnlockMutex_srw(SDL_mutex *_mutex) SDL_NO_THREAD_SAFETY_ANALYSIS /* clang doesn't know about NULL mutexes */
+static void SDL_UnlockMutex_srw(SDL_Mutex *_mutex) SDL_NO_THREAD_SAFETY_ANALYSIS  // clang doesn't know about NULL mutexes
 {
     SDL_mutex_srw *mutex = (SDL_mutex_srw *)_mutex;
 
@@ -127,10 +112,8 @@ static int SDL_UnlockMutex_srw(SDL_mutex *_mutex) SDL_NO_THREAD_SAFETY_ANALYSIS
             pReleaseSRWLockExclusive(&mutex->srw);
         }
     } else {
-        return SDL_SetError("mutex not owned by this thread");
+        SDL_assert(!"mutex not owned by this thread");  // undefined behavior...!
     }
-
-    return 0;
 }
 
 static const SDL_mutex_impl_t SDL_mutex_impl_srw = {
@@ -146,64 +129,41 @@ static const SDL_mutex_impl_t SDL_mutex_impl_srw = {
  * Fallback Mutex implementation using Critical Sections (before Win 7)
  */
 
-/* Create a mutex */
-static SDL_mutex *SDL_CreateMutex_cs(void)
+static SDL_Mutex *SDL_CreateMutex_cs(void)
 {
-    SDL_mutex_cs *mutex;
-
-    /* Allocate mutex memory */
-    mutex = (SDL_mutex_cs *)SDL_malloc(sizeof(*mutex));
-    if (mutex != NULL) {
-        /* Initialize */
-        /* On SMP systems, a non-zero spin count generally helps performance */
-#if __WINRT__
-        InitializeCriticalSectionEx(&mutex->cs, 2000, 0);
-#else
-        InitializeCriticalSectionAndSpinCount(&mutex->cs, 2000);
-#endif
-    } else {
-        SDL_OutOfMemory();
+    SDL_mutex_cs *mutex = (SDL_mutex_cs *)SDL_malloc(sizeof(*mutex));
+    if (mutex) {
+        // Initialize
+        // On SMP systems, a non-zero spin count generally helps performance
+        // This function always succeeds
+        (void)InitializeCriticalSectionAndSpinCount(&mutex->cs, 2000);
     }
-    return (SDL_mutex *)mutex;
+    return (SDL_Mutex *)mutex;
 }
 
-/* Free the mutex */
-static void SDL_DestroyMutex_cs(SDL_mutex *mutex_)
+static void SDL_DestroyMutex_cs(SDL_Mutex *mutex_)
 {
     SDL_mutex_cs *mutex = (SDL_mutex_cs *)mutex_;
-
     DeleteCriticalSection(&mutex->cs);
     SDL_free(mutex);
 }
 
-/* Lock the mutex */
-static int SDL_LockMutex_cs(SDL_mutex *mutex_) SDL_NO_THREAD_SAFETY_ANALYSIS /* clang doesn't know about NULL mutexes */
+static void SDL_LockMutex_cs(SDL_Mutex *mutex_) SDL_NO_THREAD_SAFETY_ANALYSIS  // clang doesn't know about NULL mutexes
 {
     SDL_mutex_cs *mutex = (SDL_mutex_cs *)mutex_;
-
     EnterCriticalSection(&mutex->cs);
-    return 0;
 }
 
-/* TryLock the mutex */
-static int SDL_TryLockMutex_cs(SDL_mutex *mutex_)
+static bool SDL_TryLockMutex_cs(SDL_Mutex *mutex_)
 {
     SDL_mutex_cs *mutex = (SDL_mutex_cs *)mutex_;
-    int retval = 0;
-
-    if (TryEnterCriticalSection(&mutex->cs) == 0) {
-        retval = SDL_MUTEX_TIMEDOUT;
-    }
-    return retval;
+    return (TryEnterCriticalSection(&mutex->cs) == TRUE);
 }
 
-/* Unlock the mutex */
-static int SDL_UnlockMutex_cs(SDL_mutex *mutex_) SDL_NO_THREAD_SAFETY_ANALYSIS /* clang doesn't know about NULL mutexes */
+static void SDL_UnlockMutex_cs(SDL_Mutex *mutex_) SDL_NO_THREAD_SAFETY_ANALYSIS  // clang doesn't know about NULL mutexes
 {
     SDL_mutex_cs *mutex = (SDL_mutex_cs *)mutex_;
-
     LeaveCriticalSection(&mutex->cs);
-    return 0;
 }
 
 static const SDL_mutex_impl_t SDL_mutex_impl_cs = {
@@ -219,72 +179,60 @@ static const SDL_mutex_impl_t SDL_mutex_impl_cs = {
  * Runtime selection and redirection
  */
 
-SDL_mutex *SDL_CreateMutex(void)
+SDL_Mutex *SDL_CreateMutex(void)
 {
-    if (SDL_mutex_impl_active.Create == NULL) {
-        /* Default to fallback implementation */
+    if (!SDL_mutex_impl_active.Create) {
         const SDL_mutex_impl_t *impl = &SDL_mutex_impl_cs;
 
-        if (!SDL_GetHintBoolean(SDL_HINT_WINDOWS_FORCE_MUTEX_CRITICAL_SECTIONS, SDL_FALSE)) {
-#if __WINRT__
-            /* Link statically on this platform */
-            impl = &SDL_mutex_impl_srw;
-#else
-            /* Try faster implementation for Windows 7 and newer */
-            HMODULE kernel32 = GetModuleHandle(TEXT("kernel32.dll"));
-            if (kernel32) {
-                /* Requires Vista: */
-                pReleaseSRWLockExclusive = (pfnReleaseSRWLockExclusive)GetProcAddress(kernel32, "ReleaseSRWLockExclusive");
-                pAcquireSRWLockExclusive = (pfnAcquireSRWLockExclusive)GetProcAddress(kernel32, "AcquireSRWLockExclusive");
-                /* Requires 7: */
-                pTryAcquireSRWLockExclusive = (pfnTryAcquireSRWLockExclusive)GetProcAddress(kernel32, "TryAcquireSRWLockExclusive");
-                if (pReleaseSRWLockExclusive && pAcquireSRWLockExclusive && pTryAcquireSRWLockExclusive) {
-                    impl = &SDL_mutex_impl_srw;
-                }
+        // Try faster implementation for Windows 7 and newer
+        HMODULE kernel32 = GetModuleHandle(TEXT("kernel32.dll"));
+        if (kernel32) {
+            // Requires Vista:
+            pInitializeSRWLock = (pfnInitializeSRWLock)GetProcAddress(kernel32, "InitializeSRWLock");
+            pReleaseSRWLockExclusive = (pfnReleaseSRWLockExclusive)GetProcAddress(kernel32, "ReleaseSRWLockExclusive");
+            pAcquireSRWLockExclusive = (pfnAcquireSRWLockExclusive)GetProcAddress(kernel32, "AcquireSRWLockExclusive");
+            // Requires 7:
+            pTryAcquireSRWLockExclusive = (pfnTryAcquireSRWLockExclusive)GetProcAddress(kernel32, "TryAcquireSRWLockExclusive");
+            if (pInitializeSRWLock && pReleaseSRWLockExclusive && pAcquireSRWLockExclusive && pTryAcquireSRWLockExclusive) {
+                impl = &SDL_mutex_impl_srw;
             }
-#endif
         }
 
-        /* Copy instead of using pointer to save one level of indirection */
+        // Copy instead of using pointer to save one level of indirection
         SDL_copyp(&SDL_mutex_impl_active, impl);
     }
     return SDL_mutex_impl_active.Create();
 }
 
-void SDL_DestroyMutex(SDL_mutex *mutex)
+void SDL_DestroyMutex(SDL_Mutex *mutex)
 {
     if (mutex) {
         SDL_mutex_impl_active.Destroy(mutex);
     }
 }
 
-int SDL_LockMutex(SDL_mutex *mutex)
+void SDL_LockMutex(SDL_Mutex *mutex)
 {
-    if (mutex == NULL) {
-        return 0;
+    if (mutex) {
+        SDL_mutex_impl_active.Lock(mutex);
     }
-
-    return SDL_mutex_impl_active.Lock(mutex);
 }
 
-int SDL_TryLockMutex(SDL_mutex *mutex)
+bool SDL_TryLockMutex(SDL_Mutex *mutex)
 {
-    if (mutex == NULL) {
-        return 0;
-    }
+    bool result = true;
 
-    return SDL_mutex_impl_active.TryLock(mutex);
+    if (mutex) {
+        result = SDL_mutex_impl_active.TryLock(mutex);
+    }
+    return result;
 }
 
-int SDL_UnlockMutex(SDL_mutex *mutex)
+void SDL_UnlockMutex(SDL_Mutex *mutex)
 {
-    if (mutex == NULL) {
-        return 0;
+    if (mutex) {
+        SDL_mutex_impl_active.Unlock(mutex);
     }
-
-    return SDL_mutex_impl_active.Unlock(mutex);
 }
 
-#endif /* SDL_THREAD_WINDOWS */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_THREAD_WINDOWS
