@@ -24,6 +24,4 @@
 
 void SDL_BlitCopy(SDL_BlitInfo *info);
 
-#endif /* SDL_blit_copy_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_blit_copy_h_
