@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,31 +18,27 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_DRIVER_ANDROID
+#ifdef SDL_VIDEO_DRIVER_ANDROID
 
 #include "SDL_androidvideo.h"
 #include "SDL_androidclipboard.h"
 #include "../../core/android/SDL_android.h"
 
-int
-Android_SetClipboardText(_THIS, const char *text)
+bool Android_SetClipboardText(SDL_VideoDevice *_this, const char *text)
 {
     return Android_JNI_SetClipboardText(text);
 }
 
-char *
-Android_GetClipboardText(_THIS)
+char *Android_GetClipboardText(SDL_VideoDevice *_this)
 {
     return Android_JNI_GetClipboardText();
 }
 
-SDL_bool Android_HasClipboardText(_THIS)
+bool Android_HasClipboardText(SDL_VideoDevice *_this)
 {
     return Android_JNI_HasClipboardText();
 }
 
-#endif /* SDL_VIDEO_DRIVER_ANDROID */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_ANDROID
