@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2021 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,20 +18,16 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_DRIVER_ANDROID
+#ifdef SDL_VIDEO_DRIVER_ANDROID
 
-#include "SDL_messagebox.h"
 #include "SDL_androidmessagebox.h"
 #include "../../core/android/SDL_android.h"
 
-int
-Android_ShowMessageBox(const SDL_MessageBoxData *messageboxdata, int *buttonid)
+bool Android_ShowMessageBox(const SDL_MessageBoxData *messageboxdata, int *buttonID)
 {
-    return Android_JNI_ShowMessageBox(messageboxdata, buttonid);
+    return Android_JNI_ShowMessageBox(messageboxdata, buttonID);
 }
 
-#endif /* SDL_VIDEO_DRIVER_ANDROID */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_ANDROID
