@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,31 +18,28 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_DRIVER_ANDROID
+#ifdef SDL_VIDEO_DRIVER_ANDROID
 
 #include <android/log.h>
 
-#include "SDL_hints.h"
-#include "SDL_events.h"
-#include "SDL_log.h"
 #include "SDL_androidtouch.h"
 #include "../../events/SDL_mouse_c.h"
 #include "../../events/SDL_touch_c.h"
 #include "../../core/android/SDL_android.h"
 
 #define ACTION_DOWN 0
-#define ACTION_UP 1
+#define ACTION_UP   1
 #define ACTION_MOVE 2
-/* #define ACTION_CANCEL 3 */
-/* #define ACTION_OUTSIDE 4 */
+// #define ACTION_CANCEL 3
+// #define ACTION_OUTSIDE 4
 #define ACTION_POINTER_DOWN 5
-#define ACTION_POINTER_UP 6
+#define ACTION_POINTER_UP   6
 
 void Android_InitTouch(void)
 {
-    /* Add all touch devices */
+    // Add all touch devices
     Android_JNI_InitTouch();
 }
 
@@ -59,32 +56,37 @@ void Android_OnTouch(SDL_Window *window, int touch_device_id_in, int pointer_fin
         return;
     }
 
-    touchDeviceId = (SDL_TouchID)touch_device_id_in;
+    /* Touch device -1 appears when using Android emulator, eg:
+     *  adb shell input mouse tap 100 100
+     *  adb shell input touchscreen tap 100 100
+     */
+    touchDeviceId = (SDL_TouchID)(touch_device_id_in + 2);
+
+    // Finger ID should be greater than 0
+    fingerId = (SDL_FingerID)(pointer_finger_id_in + 1);
+
     if (SDL_AddTouch(touchDeviceId, SDL_TOUCH_DEVICE_DIRECT, "") < 0) {
         SDL_Log("error: can't add touch %s, %d", __FILE__, __LINE__);
     }
 
-    fingerId = (SDL_FingerID)pointer_finger_id_in;
     switch (action) {
-        case ACTION_DOWN:
-        case ACTION_POINTER_DOWN:
-            SDL_SendTouch(touchDeviceId, fingerId, window, SDL_TRUE, x, y, p);
-            break;
+    case ACTION_DOWN:
+    case ACTION_POINTER_DOWN:
+        SDL_SendTouch(0, touchDeviceId, fingerId, window, true, x, y, p);
+        break;
 
-        case ACTION_MOVE:
-            SDL_SendTouchMotion(touchDeviceId, fingerId, window, x, y, p);
-            break;
+    case ACTION_MOVE:
+        SDL_SendTouchMotion(0, touchDeviceId, fingerId, window, x, y, p);
+        break;
 
-        case ACTION_UP:
-        case ACTION_POINTER_UP:
-            SDL_SendTouch(touchDeviceId, fingerId, window, SDL_FALSE, x, y, p);
-            break;
+    case ACTION_UP:
+    case ACTION_POINTER_UP:
+        SDL_SendTouch(0, touchDeviceId, fingerId, window, false, x, y, p);
+        break;
 
-        default:
-            break;
+    default:
+        break;
     }
 }
 
-#endif /* SDL_VIDEO_DRIVER_ANDROID */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_ANDROID
