@@ -18,35 +18,29 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_androidvideo_h_
 #define SDL_androidvideo_h_
 
-#include "SDL_mutex.h"
-#include "SDL_rect.h"
 #include "../SDL_sysvideo.h"
 
-/* Called by the JNI layer when the screen changes size or format */
-extern void Android_SetScreenResolution(int surfaceWidth, int surfaceHeight, int deviceWidth, int deviceHeight, float rate);
+// Called by the JNI layer when the screen changes size or format
+extern void Android_SetScreenResolution(int surfaceWidth, int surfaceHeight, int deviceWidth, int deviceHeight, float density, float rate);
 extern void Android_SetFormat(int format_wanted, int format_got);
 extern void Android_SendResize(SDL_Window *window);
+extern void Android_SetDarkMode(bool enabled);
 
-/* Private display data */
+// Private display data
 
-typedef struct SDL_VideoData
+struct SDL_VideoData
 {
-    SDL_Rect textRect;
     int isPaused;
     int isPausing;
-    int pauseAudio;
-} SDL_VideoData;
+};
 
 extern int Android_SurfaceWidth;
 extern int Android_SurfaceHeight;
-extern SDL_sem *Android_PauseSem, *Android_ResumeSem;
-extern SDL_mutex *Android_ActivityMutex;
+extern float Android_ScreenDensity;
 
-#endif /* SDL_androidvideo_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_androidvideo_h_
