@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_androidwindow_h_
 #define SDL_androidwindow_h_
@@ -26,27 +26,26 @@
 #include "../../core/android/SDL_android.h"
 #include "../SDL_egl_c.h"
 
-extern int Android_CreateWindow(_THIS, SDL_Window *window);
-extern void Android_SetWindowTitle(_THIS, SDL_Window *window);
-extern void Android_SetWindowFullscreen(_THIS, SDL_Window *window, SDL_VideoDisplay *display, SDL_bool fullscreen);
-extern void Android_MinimizeWindow(_THIS, SDL_Window *window);
-extern void Android_SetWindowResizable(_THIS, SDL_Window *window, SDL_bool resizable);
+extern bool Android_CreateWindow(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID create_props);
+extern void Android_SetWindowTitle(SDL_VideoDevice *_this, SDL_Window *window);
+extern SDL_FullscreenResult Android_SetWindowFullscreen(SDL_VideoDevice *_this, SDL_Window *window, SDL_VideoDisplay *display, SDL_FullscreenOp fullscreen);
+extern void Android_MinimizeWindow(SDL_VideoDevice *_this, SDL_Window *window);
+extern void Android_SetWindowResizable(SDL_VideoDevice *_this, SDL_Window *window, bool resizable);
 
-extern void Android_DestroyWindow(_THIS, SDL_Window *window);
-extern SDL_bool Android_GetWindowWMInfo(_THIS, SDL_Window *window, struct SDL_SysWMinfo *info);
+extern void Android_DestroyWindow(SDL_VideoDevice *_this, SDL_Window *window);
 extern SDL_Window *Android_Window;
 
-typedef struct
+struct SDL_WindowData
 {
 #ifdef SDL_VIDEO_OPENGL_EGL
     EGLSurface egl_surface;
-    EGLContext egl_context; /* We use this to preserve the context when losing focus */
+    EGLContext egl_context; // We use this to preserve the context when losing focus
+    int has_swap_interval;  // Save/Restore the swap interval / vsync
+    int swap_interval;
 #endif
-    SDL_bool backup_done;
+    bool backup_done;
     ANativeWindow *native_window;
 
-} SDL_WindowData;
+};
 
-#endif /* SDL_androidwindow_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_androidwindow_h_
