@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -22,7 +22,7 @@
 #ifndef SDL_kmsdrmdyn_h_
 #define SDL_kmsdrmdyn_h_
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #include <xf86drm.h>
 #include <xf86drmMode.h>
@@ -32,22 +32,23 @@
 extern "C" {
 #endif
 
-int SDL_KMSDRM_LoadSymbols(void);
-void SDL_KMSDRM_UnloadSymbols(void);
+extern bool SDL_KMSDRM_LoadSymbols(void);
+extern void SDL_KMSDRM_UnloadSymbols(void);
 
-/* Declare all the function pointers and wrappers... */
+// Declare all the function pointers and wrappers...
 #define SDL_KMSDRM_SYM(rc, fn, params)        \
     typedef rc(*SDL_DYNKMSDRMFN_##fn) params; \
     extern SDL_DYNKMSDRMFN_##fn KMSDRM_##fn;
 #define SDL_KMSDRM_SYM_CONST(type, name)    \
     typedef type SDL_DYNKMSDRMCONST_##name; \
     extern SDL_DYNKMSDRMCONST_##name KMSDRM_##name;
+#define SDL_KMSDRM_SYM_OPT(rc, fn, params)    \
+    typedef rc(*SDL_DYNKMSDRMFN_##fn) params; \
+    extern SDL_DYNKMSDRMFN_##fn KMSDRM_##fn;
 #include "SDL_kmsdrmsym.h"
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* SDL_kmsdrmdyn_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_kmsdrmdyn_h_
