@@ -28,11 +28,9 @@ void N3DS_SwkbInit();
 void N3DS_SwkbPoll();
 void N3DS_SwkbQuit();
 
-SDL_bool N3DS_HasScreenKeyboardSupport(_THIS);
+bool N3DS_HasScreenKeyboardSupport(SDL_VideoDevice *_this);
 
-void N3DS_StartTextInput(_THIS);
-void N3DS_StopTextInput(_THIS);
+bool N3DS_StartTextInput(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID props);
+bool N3DS_StopTextInput(SDL_VideoDevice *_this, SDL_Window *window);
 
-#endif /* SDL_n3dskeyboard_h_ */
-
-/* vi: set sts=4 ts=4 sw=4 expandtab: */
+#endif // SDL_n3dskeyboard_h_
