@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -23,7 +23,7 @@
 #ifdef NULL
 #undef NULL
 #endif
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_VIDEO_DRIVER_NGAGE
 
@@ -31,7 +31,6 @@
 extern "C" {
 #endif
 
-#include "SDL_video.h"
 #include "../SDL_sysvideo.h"
 #include "../SDL_pixels_c.h"
 #include "../../events/SDL_events_c.h"
@@ -47,21 +46,20 @@ extern "C" {
 
 #define NGAGEVID_DRIVER_NAME "ngage"
 
-/* Initialization/Query functions */
-static int NGAGE_VideoInit(_THIS);
-static int NGAGE_SetDisplayMode(_THIS, SDL_VideoDisplay *display, SDL_DisplayMode *mode);
-static void NGAGE_VideoQuit(_THIS);
+// Initialization/Query functions
+static bool NGAGE_VideoInit(SDL_VideoDevice *_this);
+static void NGAGE_VideoQuit(SDL_VideoDevice *_this);
 
-/* NGAGE driver bootstrap functions */
+// NGAGE driver bootstrap functions
 
 static void NGAGE_DeleteDevice(SDL_VideoDevice *device)
 {
-    SDL_VideoData *phdata = (SDL_VideoData *)device->driverdata;
+    SDL_VideoData *phdata = device->internal;
 
     if (phdata) {
-        /* Free Epoc resources */
+        // Free Epoc resources
 
-        /* Disable events for me */
+        // Disable events for me
         if (phdata->NGAGE_WsEventStatus != KRequestPending) {
             phdata->NGAGE_WsSession.EventReadyCancel();
         }
@@ -69,7 +67,7 @@ static void NGAGE_DeleteDevice(SDL_VideoDevice *device)
             phdata->NGAGE_WsSession.RedrawReadyCancel();
         }
 
-        free(phdata->NGAGE_DrawDevice);
+        free(phdata->NGAGE_DrawDevice); // This should NOT be SDL_free()
 
         if (phdata->NGAGE_WsWindow.WsHandle()) {
             phdata->NGAGE_WsWindow.Close();
@@ -104,37 +102,34 @@ static SDL_VideoDevice *NGAGE_CreateDevice(void)
     SDL_VideoDevice *device;
     SDL_VideoData *phdata;
 
-    /* Initialize all variables that we clean on shutdown */
+    // Initialize all variables that we clean on shutdown
     device = (SDL_VideoDevice *)SDL_calloc(1, sizeof(SDL_VideoDevice));
     if (!device) {
-        SDL_OutOfMemory();
-        return 0;
+        return NULL;
     }
 
-    /* Initialize internal N-Gage specific data */
+    // Initialize internal N-Gage specific data
     phdata = (SDL_VideoData *)SDL_calloc(1, sizeof(SDL_VideoData));
     if (!phdata) {
-        SDL_OutOfMemory();
         SDL_free(device);
-        return 0;
+        return NULL;
     }
 
-    /* General video */
+    // General video
     device->VideoInit = NGAGE_VideoInit;
     device->VideoQuit = NGAGE_VideoQuit;
-    device->SetDisplayMode = NGAGE_SetDisplayMode;
     device->PumpEvents = NGAGE_PumpEvents;
     device->CreateWindowFramebuffer = SDL_NGAGE_CreateWindowFramebuffer;
     device->UpdateWindowFramebuffer = SDL_NGAGE_UpdateWindowFramebuffer;
     device->DestroyWindowFramebuffer = SDL_NGAGE_DestroyWindowFramebuffer;
     device->free = NGAGE_DeleteDevice;
 
-    /* "Window" */
+    // "Window"
     device->CreateSDLWindow = NGAGE_CreateWindow;
     device->DestroyWindow = NGAGE_DestroyWindow;
 
-    /* N-Gage specific data */
-    device->driverdata = phdata;
+    // N-Gage specific data
+    device->internal = phdata;
 
     return device;
 }
@@ -142,39 +137,28 @@ static SDL_VideoDevice *NGAGE_CreateDevice(void)
 VideoBootStrap NGAGE_bootstrap = {
     NGAGEVID_DRIVER_NAME, "SDL ngage video driver",
     NGAGE_CreateDevice,
-    NULL /* no ShowMessageBox implementation */
+    NULL // no ShowMessageBox implementation
 };
 
-int NGAGE_VideoInit(_THIS)
+static bool NGAGE_VideoInit(SDL_VideoDevice *_this)
 {
     SDL_DisplayMode mode;
 
-    /* Use 12-bpp desktop mode */
-    mode.format = SDL_PIXELFORMAT_RGB444;
+    // Use 12-bpp desktop mode
+    SDL_zero(mode);
+    mode.format = SDL_PIXELFORMAT_XRGB4444;
     mode.w = 176;
     mode.h = 208;
-    mode.refresh_rate = 0;
-    mode.driverdata = NULL;
-    if (SDL_AddBasicVideoDisplay(&mode) < 0) {
-        return -1;
+    if (SDL_AddBasicVideoDisplay(&mode) == 0) {
+        return false;
     }
 
-    SDL_zero(mode);
-    SDL_AddDisplayMode(&_this->displays[0], &mode);
-
-    /* We're done! */
-    return 0;
+    // We're done!
+    return true;
 }
 
-static int NGAGE_SetDisplayMode(_THIS, SDL_VideoDisplay *display, SDL_DisplayMode *mode)
-{
-    return 0;
-}
-
-void NGAGE_VideoQuit(_THIS)
+static void NGAGE_VideoQuit(SDL_VideoDevice *_this)
 {
 }
 
-#endif /* SDL_VIDEO_DRIVER_NGAGE */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_NGAGE
