@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,27 +19,20 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#ifndef _SDL_ngagewindow_h
-#define _SDL_ngagewindow_h
+#ifndef SDL_ngagewindow_h
+#define SDL_ngagewindow_h
 
 #include "../SDL_sysvideo.h"
-#include "SDL_syswm.h"
 
 #include "SDL_ngagevideo.h"
 
-typedef struct {
-    SDL_Window* sdl_window;
+typedef struct
+{
+    SDL_Window *sdl_window;
 
 } NGAGE_Window;
 
+extern int NGAGE_CreateWindow(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID create_props);
+extern void NGAGE_DestroyWindow(SDL_VideoDevice *_this, SDL_Window *window);
 
-extern int
-NGAGE_CreateWindow(_THIS, SDL_Window* window);
-
-extern void
-NGAGE_DestroyWindow(_THIS, SDL_Window* window);
-
-#endif /* _SDL_ngagewindow */
-
-/* vi: set ts=4 sw=4 expandtab: */
-
+#endif // SDL_ngagewindow
