@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,14 +18,11 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_DRIVER_UIKIT
+#ifdef SDL_VIDEO_DRIVER_UIKIT
 
 #include "../SDL_sysvideo.h"
-#include "SDL_hints.h"
-#include "SDL_system.h"
-#include "SDL_main.h"
 
 #import "SDL_uikitappdelegate.h"
 #import "SDL_uikitmodes.h"
@@ -33,12 +30,12 @@
 
 #include "../../events/SDL_events_c.h"
 
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
 #include <AvailabilityVersions.h>
 
-# ifndef __IPHONE_13_0
-# define __IPHONE_13_0 130000
-# endif
+#ifndef __IPHONE_13_0
+#define __IPHONE_13_0 130000
+#endif
 #endif
 
 #ifdef main
@@ -50,43 +47,40 @@ static int forward_argc;
 static char **forward_argv;
 static int exit_status;
 
-int SDL_UIKitRunApp(int argc, char *argv[], SDL_main_func mainFunction)
+int SDL_RunApp(int argc, char* argv[], SDL_main_func mainFunction, void * reserved)
 {
     int i;
 
-    /* store arguments */
+    // store arguments
+    /* Note that we need to be careful about how we allocate/free memory here.
+     * If the application calls SDL_SetMemoryFunctions(), we can't rely on
+     * SDL_free() to use the same allocator after SDL_main() returns.
+     */
     forward_main = mainFunction;
     forward_argc = argc;
-    forward_argv = (char **)malloc((argc+1) * sizeof(char *));
+    forward_argv = (char **)malloc((argc + 1) * sizeof(char *)); // This should NOT be SDL_malloc()
     for (i = 0; i < argc; i++) {
-        forward_argv[i] = malloc( (strlen(argv[i])+1) * sizeof(char));
+        forward_argv[i] = malloc((strlen(argv[i]) + 1) * sizeof(char)); // This should NOT be SDL_malloc()
         strcpy(forward_argv[i], argv[i]);
     }
     forward_argv[i] = NULL;
 
-    /* Give over control to run loop, SDLUIKitDelegate will handle most things from here */
+    // Give over control to run loop, SDLUIKitDelegate will handle most things from here
     @autoreleasepool {
         UIApplicationMain(argc, argv, nil, [SDLUIKitDelegate getAppDelegateClassName]);
     }
 
-    /* free the memory we used to hold copies of argc and argv */
+    // free the memory we used to hold copies of argc and argv
     for (i = 0; i < forward_argc; i++) {
-        free(forward_argv[i]);
+        free(forward_argv[i]); // This should NOT be SDL_free()
     }
-    free(forward_argv);
+    free(forward_argv); // This should NOT be SDL_free()
 
     return exit_status;
 }
 
-static void SDLCALL
-SDL_IdleTimerDisabledChanged(void *userdata, const char *name, const char *oldValue, const char *hint)
-{
-    BOOL disable = (hint && *hint != '0');
-    [UIApplication sharedApplication].idleTimerDisabled = disable;
-}
-
-#if !TARGET_OS_TV
-/* Load a launch image using the old UILaunchImageFile-era naming rules. */
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
+// Load a launch image using the old UILaunchImageFile-era naming rules.
 static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 {
     UIInterfaceOrientation curorient = [UIApplication sharedApplication].statusBarOrientation;
@@ -94,10 +88,10 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
     UIImage *image = nil;
 
     if (idiom == UIUserInterfaceIdiomPhone && screenh == 568) {
-        /* The image name for the iPhone 5 uses its height as a suffix. */
+        // The image name for the iPhone 5 uses its height as a suffix.
         image = [UIImage imageNamed:[NSString stringWithFormat:@"%@-568h", name]];
     } else if (idiom == UIUserInterfaceIdiomPad) {
-        /* iPad apps can launch in any orientation. */
+        // iPad apps can launch in any orientation.
         if (UIInterfaceOrientationIsLandscape(curorient)) {
             if (curorient == UIInterfaceOrientationLandscapeLeft) {
                 image = [UIImage imageNamed:[NSString stringWithFormat:@"%@-LandscapeLeft", name]];
@@ -125,19 +119,21 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 }
 
 @interface SDLLaunchStoryboardViewController : UIViewController
-@property (nonatomic, strong) UIViewController *storyboardViewController;
+@property(nonatomic, strong) UIViewController *storyboardViewController;
 - (instancetype)initWithStoryboardViewController:(UIViewController *)storyboardViewController;
 @end
 
 @implementation SDLLaunchStoryboardViewController
 
-- (instancetype)initWithStoryboardViewController:(UIViewController *)storyboardViewController {
+- (instancetype)initWithStoryboardViewController:(UIViewController *)storyboardViewController
+{
     self = [super init];
     self.storyboardViewController = storyboardViewController;
     return self;
 }
 
-- (void)viewDidLoad {
+- (void)viewDidLoad
+{
     [super viewDidLoad];
 
     [self addChildViewController:self.storyboardViewController];
@@ -146,15 +142,19 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
     self.storyboardViewController.view.frame = self.view.bounds;
     [self.storyboardViewController didMoveToParentViewController:self];
 
+#ifndef SDL_PLATFORM_VISIONOS
     UIApplication.sharedApplication.statusBarHidden = self.prefersStatusBarHidden;
     UIApplication.sharedApplication.statusBarStyle = self.preferredStatusBarStyle;
+#endif
 }
 
-- (BOOL)prefersStatusBarHidden {
+- (BOOL)prefersStatusBarHidden
+{
     return [[NSBundle.mainBundle objectForInfoDictionaryKey:@"UIStatusBarHidden"] boolValue];
 }
 
-- (UIStatusBarStyle)preferredStatusBarStyle {
+- (UIStatusBarStyle)preferredStatusBarStyle
+{
     NSString *statusBarStyle = [NSBundle.mainBundle objectForInfoDictionaryKey:@"UIStatusBarStyle"];
     if ([statusBarStyle isEqualToString:@"UIStatusBarStyleLightContent"]) {
         return UIStatusBarStyleLightContent;
@@ -170,11 +170,11 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 }
 
 @end
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 
 @interface SDLLaunchScreenController ()
 
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
 - (NSUInteger)supportedInterfaceOrientations;
 #endif
 
@@ -196,7 +196,7 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
     NSString *screenname = nibNameOrNil;
     NSBundle *bundle = nibBundleOrNil;
 
-    /* A launch screen may not exist. Fall back to launch images in that case. */
+    // A launch screen may not exist. Fall back to launch images in that case.
     if (screenname) {
         @try {
             self.view = [bundle loadNibNamed:screenname owner:self options:nil][0];
@@ -213,13 +213,20 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
         NSString *imagename = nil;
         UIImage *image = nil;
 
+#ifdef SDL_PLATFORM_VISIONOS
+        int screenw = SDL_XR_SCREENWIDTH;
+        int screenh = SDL_XR_SCREENHEIGHT;
+#else
         int screenw = (int)([UIScreen mainScreen].bounds.size.width + 0.5);
         int screenh = (int)([UIScreen mainScreen].bounds.size.height + 0.5);
+#endif
 
-#if !TARGET_OS_TV
+
+
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
         UIInterfaceOrientation curorient = [UIApplication sharedApplication].statusBarOrientation;
 
-        /* We always want portrait-oriented size, to match UILaunchImageSize. */
+        // We always want portrait-oriented size, to match UILaunchImageSize.
         if (screenw > screenh) {
             int width = screenw;
             screenw = screenh;
@@ -227,18 +234,18 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
         }
 #endif
 
-        /* Xcode 5 introduced a dictionary of launch images in Info.plist. */
+        // Xcode 5 introduced a dictionary of launch images in Info.plist.
         if (launchimages) {
             for (NSDictionary *dict in launchimages) {
                 NSString *minversion = dict[@"UILaunchImageMinimumOSVersion"];
                 NSString *sizestring = dict[@"UILaunchImageSize"];
 
-                /* Ignore this image if the current version is too low. */
+                // Ignore this image if the current version is too low.
                 if (minversion && !UIKit_IsSystemVersionAtLeast(minversion.doubleValue)) {
                     continue;
                 }
 
-                /* Ignore this image if the size doesn't match. */
+                // Ignore this image if the size doesn't match.
                 if (sizestring) {
                     CGSize size = CGSizeFromString(sizestring);
                     if ((int)(size.width + 0.5) != screenw || (int)(size.height + 0.5) != screenh) {
@@ -246,7 +253,7 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
                     }
                 }
 
-#if !TARGET_OS_TV
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
                 UIInterfaceOrientationMask orientmask = UIInterfaceOrientationMaskPortrait | UIInterfaceOrientationMaskPortraitUpsideDown;
                 NSString *orientstring = dict[@"UILaunchImageOrientation"];
 
@@ -262,7 +269,7 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
                     }
                 }
 
-                /* Ignore this image if the orientation doesn't match. */
+                // Ignore this image if the orientation doesn't match.
                 if ((orientmask & (1 << curorient)) == 0) {
                     continue;
                 }
@@ -275,7 +282,7 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
                 image = [UIImage imageNamed:imagename];
             }
         }
-#if !TARGET_OS_TV
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
         else {
             imagename = [bundle objectForInfoDictionaryKey:@"UILaunchImageFile"];
 
@@ -290,11 +297,16 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 #endif
 
         if (image) {
-            UIImageView *view = [[UIImageView alloc] initWithFrame:[UIScreen mainScreen].bounds];
+#ifdef SDL_PLATFORM_VISIONOS
+            CGRect viewFrame = CGRectMake(0, 0, screenw, screenh);
+#else
+            CGRect viewFrame = [UIScreen mainScreen].bounds;
+#endif
+            UIImageView *view = [[UIImageView alloc] initWithFrame:viewFrame];
             UIImageOrientation imageorient = UIImageOrientationUp;
 
-#if !TARGET_OS_TV
-            /* Bugs observed / workaround tested in iOS 8.3. */
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
+            // Bugs observed / workaround tested in iOS 8.3.
             if (UIInterfaceOrientationIsLandscape(curorient)) {
                 if (image.size.width < image.size.height) {
                     /* On iOS 8, portrait launch images displayed in forced-
@@ -310,7 +322,7 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
             }
 #endif
 
-            /* Create the properly oriented image. */
+            // Create the properly oriented image.
             view.image = [[UIImage alloc] initWithCGImage:image.CGImage scale:image.scale orientation:imageorient];
 
             self.view = view;
@@ -322,13 +334,13 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 
 - (void)loadView
 {
-    /* Do nothing. */
+    // Do nothing.
 }
 
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
 - (BOOL)shouldAutorotate
 {
-    /* If YES, the launch image will be incorrectly rotated in some cases. */
+    // If YES, the launch image will be incorrectly rotated in some cases.
     return NO;
 }
 
@@ -339,15 +351,16 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
      * the ones set here (it will cause an exception in that case.) */
     return UIInterfaceOrientationMaskAll;
 }
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 
 @end
 
-@implementation SDLUIKitDelegate {
+@implementation SDLUIKitDelegate
+{
     UIWindow *launchWindow;
 }
 
-/* convenience method */
+// convenience method
 + (id)sharedAppDelegate
 {
     /* the delegate is set in UIApplicationMain(), which is guaranteed to be
@@ -373,13 +386,15 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 
     launchWindow = nil;
 
-    /* Do a nice animated fade-out (roughly matches the real launch behavior.) */
-    [UIView animateWithDuration:0.2 animations:^{
-        window.alpha = 0.0;
-    } completion:^(BOOL finished) {
-        window.hidden = YES;
-        UIKit_ForceUpdateHomeIndicator(); /* Wait for launch screen to hide so settings are applied to the actual view controller. */
-    }];
+    // Do a nice animated fade-out (roughly matches the real launch behavior.)
+    [UIView animateWithDuration:0.2
+        animations:^{
+          window.alpha = 0.0;
+        }
+        completion:^(BOOL finished) {
+          window.hidden = YES;
+          UIKit_ForceUpdateHomeIndicator(); // Wait for launch screen to hide so settings are applied to the actual view controller.
+        }];
 }
 
 - (void)postFinishLaunch
@@ -388,27 +403,27 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
      * have a chance to load resources while the launch screen is still up. */
     [self performSelector:@selector(hideLaunchScreen) withObject:nil afterDelay:0.0];
 
-    /* run the user's application, passing argc and argv */
-    SDL_iPhoneSetEventPump(SDL_TRUE);
+    // run the user's application, passing argc and argv
+    SDL_SetiOSEventPump(true);
     exit_status = forward_main(forward_argc, forward_argv);
-    SDL_iPhoneSetEventPump(SDL_FALSE);
+    SDL_SetiOSEventPump(false);
 
     if (launchWindow) {
         launchWindow.hidden = YES;
         launchWindow = nil;
     }
 
-    /* exit, passing the return status from the user's application */
+    // exit, passing the return status from the user's application
     /* We don't actually exit to support applications that do setup in their
      * main function and then allow the Cocoa event loop to run. */
-    /* exit(exit_status); */
+    // exit(exit_status);
 }
 
 - (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
 {
     NSBundle *bundle = [NSBundle mainBundle];
 
-#if SDL_IPHONE_LAUNCHSCREEN
+#ifdef SDL_IPHONE_LAUNCHSCREEN
     /* The normal launch screen is displayed until didFinishLaunching returns,
      * but SDL_main is called after that happens and there may be a noticeable
      * delay between the start of SDL_main and when the first real frame is
@@ -418,8 +433,8 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
     UIViewController *vc = nil;
     NSString *screenname = nil;
 
-    /* tvOS only uses a plain launch image. */
-#if !TARGET_OS_TV
+    // tvOS only uses a plain launch image.
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
     screenname = [bundle objectForInfoDictionaryKey:@"UILaunchStoryboardName"];
 
     if (screenname) {
@@ -432,7 +447,7 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
             vc = [[SDLLaunchStoryboardViewController alloc] initWithStoryboardViewController:storyboardVc];
         }
         @catch (NSException *exception) {
-            /* Do nothing (there's more code to execute below). */
+            // Do nothing (there's more code to execute below).
         }
     }
 #endif
@@ -442,7 +457,12 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
     }
 
     if (vc.view) {
-        launchWindow = [[UIWindow alloc] initWithFrame:[UIScreen mainScreen].bounds];
+#ifdef SDL_PLATFORM_VISIONOS
+        CGRect viewFrame = CGRectMake(0, 0, SDL_XR_SCREENWIDTH, SDL_XR_SCREENHEIGHT);
+#else
+        CGRect viewFrame = [UIScreen mainScreen].bounds;
+#endif
+        launchWindow = [[UIWindow alloc] initWithFrame:viewFrame];
 
         /* We don't want the launch window immediately hidden when a real SDL
          * window is shown - we fade it out ourselves when we're ready. */
@@ -456,13 +476,9 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
     }
 #endif
 
-    /* Set working directory to resource path */
+    // Set working directory to resource path
     [[NSFileManager defaultManager] changeCurrentDirectoryPath:[bundle resourcePath]];
 
-    /* register a callback for the idletimer hint */
-    SDL_AddHintCallback(SDL_HINT_IDLE_TIMER_DISABLED,
-                        SDL_IdleTimerDisabledChanged, NULL);
-
     SDL_SetMainReady();
     [self performSelector:@selector(postFinishLaunch) withObject:nil afterDelay:0.0];
 
@@ -475,7 +491,7 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
     if (_this) {
         SDL_Window *window = NULL;
         for (window = _this->windows; window != NULL; window = window->next) {
-            SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+            SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
             if (data != nil) {
                 return data.uiwindow;
             }
@@ -486,26 +502,27 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 
 - (void)setWindow:(UIWindow *)window
 {
-    /* Do nothing. */
+    // Do nothing.
 }
 
-- (void)sendDropFileForURL:(NSURL *)url
+- (void)sendDropFileForURL:(NSURL *)url fromSourceApplication:(NSString *)sourceApplication
 {
     NSURL *fileURL = url.filePathURL;
+    const char *sourceApplicationCString = sourceApplication ? [sourceApplication UTF8String] : NULL;
     if (fileURL != nil) {
-        SDL_SendDropFile(NULL, fileURL.path.UTF8String);
+        SDL_SendDropFile(NULL, sourceApplicationCString, fileURL.path.UTF8String);
     } else {
-        SDL_SendDropFile(NULL, url.absoluteString.UTF8String);
+        SDL_SendDropFile(NULL, sourceApplicationCString, url.absoluteString.UTF8String);
     }
     SDL_SendDropComplete(NULL);
 }
 
-#if TARGET_OS_TV || (defined(__IPHONE_9_0) && __IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_9_0)
+#if defined(SDL_PLATFORM_TVOS) || (defined(__IPHONE_9_0) && __IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_9_0)
 
-- (BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey,id> *)options
+- (BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey, id> *)options
 {
-    /* TODO: Handle options */
-    [self sendDropFileForURL:url];
+    // TODO: Handle options
+    [self sendDropFileForURL:url fromSourceApplication:NULL];
     return YES;
 }
 
@@ -513,7 +530,7 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 
 - (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation
 {
-    [self sendDropFileForURL:url];
+    [self sendDropFileForURL:url fromSourceApplication:sourceApplication];
     return YES;
 }
 
@@ -521,6 +538,4 @@ static UIImage *SDL_LoadLaunchImageNamed(NSString *name, int screenh)
 
 @end
 
-#endif /* SDL_VIDEO_DRIVER_UIKIT */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_UIKIT
