@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -21,19 +21,20 @@
 #ifndef SDL_uikitevents_h_
 #define SDL_uikitevents_h_
 
+#import <UIKit/UIKit.h>
+
 #include "../SDL_sysvideo.h"
 
-extern void UIKit_PumpEvents(_THIS);
+extern void SDL_UpdateLifecycleObserver(void);
+
+extern Uint64 UIKit_GetEventTimestamp(NSTimeInterval nsTimestamp);
+extern void UIKit_PumpEvents(SDL_VideoDevice *_this);
 
 extern void SDL_InitGCKeyboard(void);
-extern SDL_bool SDL_HasGCKeyboard(void);
 extern void SDL_QuitGCKeyboard(void);
 
 extern void SDL_InitGCMouse(void);
-extern SDL_bool SDL_HasGCMouse(void);
-extern SDL_bool SDL_GCMouseRelativeMode(void);
+extern bool SDL_GCMouseRelativeMode(void);
 extern void SDL_QuitGCMouse(void);
 
-#endif /* SDL_uikitevents_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_uikitevents_h_
