@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,12 +18,10 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_VIDEO_DRIVER_UIKIT
 
-#include "SDL_video.h"
-#include "SDL_hints.h"
 #include "../SDL_sysvideo.h"
 #include "../../events/SDL_events_c.h"
 
@@ -35,23 +33,21 @@
 #include "SDL_uikitwindow.h"
 #include "SDL_uikitopengles.h"
 
-#if TARGET_OS_TV
-static void SDLCALL
-SDL_AppleTVControllerUIHintChanged(void *userdata, const char *name, const char *oldValue, const char *hint)
+#ifdef SDL_PLATFORM_TVOS
+static void SDLCALL SDL_AppleTVControllerUIHintChanged(void *userdata, const char *name, const char *oldValue, const char *hint)
 {
     @autoreleasepool {
-        SDL_uikitviewcontroller *viewcontroller = (__bridge SDL_uikitviewcontroller *) userdata;
+        SDL_uikitviewcontroller *viewcontroller = (__bridge SDL_uikitviewcontroller *)userdata;
         viewcontroller.controllerUserInteractionEnabled = hint && (*hint != '0');
     }
 }
 #endif
 
-#if !TARGET_OS_TV
-static void SDLCALL
-SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *oldValue, const char *hint)
+#ifndef SDL_PLATFORM_TVOS
+static void SDLCALL SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *oldValue, const char *hint)
 {
     @autoreleasepool {
-        SDL_uikitviewcontroller *viewcontroller = (__bridge SDL_uikitviewcontroller *) userdata;
+        SDL_uikitviewcontroller *viewcontroller = (__bridge SDL_uikitviewcontroller *)userdata;
         viewcontroller.homeIndicatorHidden = (hint && *hint) ? SDL_atoi(hint) : -1;
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wunguarded-availability-new"
@@ -67,18 +63,19 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 @implementation SDLUITextField : UITextField
 - (BOOL)canPerformAction:(SEL)action withSender:(id)sender
 {
-	if (action == @selector(paste:)) {
-		return NO;
-	}
+    if (action == @selector(paste:)) {
+        return NO;
+    }
 
-	return [super canPerformAction:action withSender:sender];
+    return [super canPerformAction:action withSender:sender];
 }
 @end
 
-@implementation SDL_uikitviewcontroller {
+@implementation SDL_uikitviewcontroller
+{
     CADisplayLink *displayLink;
     int animationInterval;
-    void (*animationCallback)(void*);
+    void (*animationCallback)(void *);
     void *animationCallbackParam;
 
 #ifdef SDL_IPHONE_KEYBOARD
@@ -107,31 +104,17 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
         rotatingOrientation = NO;
 #endif
 
-#if TARGET_OS_TV
+#ifdef SDL_PLATFORM_TVOS
         SDL_AddHintCallback(SDL_HINT_APPLE_TV_CONTROLLER_UI_EVENTS,
                             SDL_AppleTVControllerUIHintChanged,
-                            (__bridge void *) self);
+                            (__bridge void *)self);
 #endif
 
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
         SDL_AddHintCallback(SDL_HINT_IOS_HIDE_HOME_INDICATOR,
                             SDL_HideHomeIndicatorHintChanged,
-                            (__bridge void *) self);
+                            (__bridge void *)self);
 #endif
-
-        /* Enable high refresh rates on iOS
-         * To enable this on phones, you should add the following line to Info.plist:
-         * <key>CADisableMinimumFrameDurationOnPhone</key> <true/>
-         */
-        if (@available(iOS 15.0, tvOS 15.0, *)) {
-            SDL_DisplayMode mode;
-            if (SDL_GetDesktopDisplayMode(0, &mode) == 0 && mode.refresh_rate > 60.0f) {
-                int frame_rate = (int)mode.refresh_rate;
-                displayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(doLoop:)];
-                displayLink.preferredFrameRateRange = CAFrameRateRangeMake((frame_rate * 2) / 3, frame_rate, frame_rate);
-                [displayLink addToRunLoop:NSRunLoop.currentRunLoop forMode:NSDefaultRunLoopMode];
-            }
-        }
     }
     return self;
 }
@@ -142,28 +125,30 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     [self deinitKeyboard];
 #endif
 
-#if TARGET_OS_TV
-    SDL_DelHintCallback(SDL_HINT_APPLE_TV_CONTROLLER_UI_EVENTS,
+#ifdef SDL_PLATFORM_TVOS
+    SDL_RemoveHintCallback(SDL_HINT_APPLE_TV_CONTROLLER_UI_EVENTS,
                         SDL_AppleTVControllerUIHintChanged,
-                        (__bridge void *) self);
+                        (__bridge void *)self);
 #endif
 
-#if !TARGET_OS_TV
-    SDL_DelHintCallback(SDL_HINT_IOS_HIDE_HOME_INDICATOR,
+#ifndef SDL_PLATFORM_TVOS
+    SDL_RemoveHintCallback(SDL_HINT_IOS_HIDE_HOME_INDICATOR,
                         SDL_HideHomeIndicatorHintChanged,
-                        (__bridge void *) self);
+                        (__bridge void *)self);
 #endif
 }
 
+- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection
+{
+    SDL_SetSystemTheme(UIKit_GetSystemTheme());
+}
+
 - (void)setAnimationCallback:(int)interval
-                    callback:(void (*)(void*))callback
-               callbackParam:(void*)callbackParam
+                    callback:(void (*)(void *))callback
+               callbackParam:(void *)callbackParam
 {
     [self stopAnimation];
 
-    if (interval <= 0) {
-        interval = 1;
-    }
     animationInterval = interval;
     animationCallback = callback;
     animationCallbackParam = callbackParam;
@@ -175,18 +160,14 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 
 - (void)startAnimation
 {
-#ifdef __IPHONE_10_3
-    SDL_WindowData *data;
-#endif
-
     displayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(doLoop:)];
 
-#ifdef __IPHONE_10_3
-    data = (__bridge SDL_WindowData *) window->driverdata;
+#ifdef SDL_PLATFORM_VISIONOS
+    displayLink.preferredFramesPerSecond = 90 / animationInterval;      //TODO: Get frame max frame rate on visionOS
+#elif defined(__IPHONE_10_3)
+    SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
 
-    if ([displayLink respondsToSelector:@selector(preferredFramesPerSecond)]
-        && data != nil && data.uiwindow != nil
-        && [data.uiwindow.screen respondsToSelector:@selector(maximumFramesPerSecond)]) {
+    if ([displayLink respondsToSelector:@selector(preferredFramesPerSecond)] && data != nil && data.uiwindow != nil && [data.uiwindow.screen respondsToSelector:@selector(maximumFramesPerSecond)]) {
         displayLink.preferredFramesPerSecond = data.uiwindow.screen.maximumFramesPerSecond / animationInterval;
     } else
 #endif
@@ -205,11 +186,11 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     displayLink = nil;
 }
 
-- (void)doLoop:(CADisplayLink*)sender
+- (void)doLoop:(CADisplayLink *)sender
 {
-    /* Don't run the game loop while a messagebox is up */
-    if (animationCallback && !UIKit_ShowingMessageBox()) {
-        /* See the comment in the function definition. */
+    // Don't run the game loop while a messagebox is up
+    if (!UIKit_ShowingMessageBox()) {
+        // See the comment in the function definition.
 #if defined(SDL_VIDEO_OPENGL_ES) || defined(SDL_VIDEO_OPENGL_ES2)
         UIKit_GL_RestoreCurrentContext();
 #endif
@@ -220,19 +201,19 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 
 - (void)loadView
 {
-    /* Do nothing. */
+    // Do nothing.
 }
 
 - (void)viewDidLayoutSubviews
 {
     const CGSize size = self.view.bounds.size;
-    int w = (int) size.width;
-    int h = (int) size.height;
+    int w = (int)size.width;
+    int h = (int)size.height;
 
-    SDL_SendWindowEvent(window, SDL_WINDOWEVENT_RESIZED, w, h);
+    SDL_SendWindowEvent(window, SDL_EVENT_WINDOW_RESIZED, w, h);
 }
 
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
 - (NSUInteger)supportedInterfaceOrientations
 {
     return UIKit_GetSupportedOrientations(window);
@@ -240,7 +221,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 
 - (BOOL)prefersStatusBarHidden
 {
-    BOOL hidden = (window->flags & (SDL_WINDOW_FULLSCREEN|SDL_WINDOW_BORDERLESS)) != 0;
+    BOOL hidden = (window->flags & (SDL_WINDOW_FULLSCREEN | SDL_WINDOW_BORDERLESS)) != 0;
     return hidden;
 }
 
@@ -263,8 +244,8 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
         }
     }
 
-    /* By default, fullscreen and borderless windows get all screen gestures */
-    if ((window->flags & (SDL_WINDOW_FULLSCREEN|SDL_WINDOW_BORDERLESS)) != 0) {
+    // By default, fullscreen and borderless windows get all screen gestures
+    if ((window->flags & (SDL_WINDOW_FULLSCREEN | SDL_WINDOW_BORDERLESS)) != 0) {
         return UIRectEdgeAll;
     } else {
         return UIRectEdgeNone;
@@ -276,7 +257,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     return SDL_GCMouseRelativeMode() ? YES : NO;
 }
 
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 
 /*
  ---- Keyboard related functionality below this line ----
@@ -287,31 +268,21 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 @synthesize keyboardHeight;
 @synthesize keyboardVisible;
 
-/* Set ourselves up as a UITextFieldDelegate */
+// Set ourselves up as a UITextFieldDelegate
 - (void)initKeyboard
 {
-    NSNotificationCenter *center;
-    obligateForBackspace = @"                                                                "; /* 64 space */
+    obligateForBackspace = @"                                                                "; // 64 space
     textField = [[SDLUITextField alloc] initWithFrame:CGRectZero];
     textField.delegate = self;
-    /* placeholder so there is something to delete! */
+    // placeholder so there is something to delete!
     textField.text = obligateForBackspace;
     committedText = textField.text;
 
-    /* set UITextInputTrait properties, mostly to defaults */
-    textField.autocapitalizationType = UITextAutocapitalizationTypeNone;
-    textField.autocorrectionType = UITextAutocorrectionTypeNo;
-    textField.enablesReturnKeyAutomatically = NO;
-    textField.keyboardAppearance = UIKeyboardAppearanceDefault;
-    textField.keyboardType = UIKeyboardTypeDefault;
-    textField.returnKeyType = UIReturnKeyDefault;
-    textField.secureTextEntry = NO;
-
     textField.hidden = YES;
     keyboardVisible = NO;
 
-    center = [NSNotificationCenter defaultCenter];
-#if !TARGET_OS_TV
+    NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
+#ifndef SDL_PLATFORM_TVOS
     [center addObserver:self
                selector:@selector(keyboardWillShow:)
                    name:UIKeyboardWillShowNotification
@@ -329,7 +300,10 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
                    name:UIKeyboardDidHideNotification
                  object:nil];
 #endif
-    [center addObserver:self selector:@selector(textFieldTextDidChange:) name:UITextFieldTextDidChangeNotification object:nil];
+    [center addObserver:self
+               selector:@selector(textFieldTextDidChange:)
+                   name:UITextFieldTextDidChangeNotification
+                 object:nil];
 }
 
 - (NSArray *)keyCommands
@@ -361,7 +335,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     }
 
     if (scancode != SDL_SCANCODE_UNKNOWN) {
-        SDL_SendKeyboardKeyAutoRelease(scancode);
+        SDL_SendKeyboardKeyAutoRelease(0, scancode);
     }
 }
 
@@ -380,16 +354,18 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 {
     [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
     rotatingOrientation = YES;
-    [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext> context) {}
-                                 completion:^(id<UIViewControllerTransitionCoordinatorContext> context) {
-        self->rotatingOrientation = NO;
-    }];
+    [coordinator
+        animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext> context) {
+        }
+        completion:^(id<UIViewControllerTransitionCoordinatorContext> context) {
+          self->rotatingOrientation = NO;
+        }];
 }
 
 - (void)deinitKeyboard
 {
     NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
     [center removeObserver:self
                       name:UIKeyboardWillShowNotification
                     object:nil];
@@ -403,10 +379,109 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
                       name:UIKeyboardDidHideNotification
                     object:nil];
 #endif
-    [center removeObserver:self name:UITextFieldTextDidChangeNotification object:nil];
+    [center removeObserver:self
+                      name:UITextFieldTextDidChangeNotification
+                    object:nil];
 }
 
-/* reveal onscreen virtual keyboard */
+- (void)setKeyboardProperties:(SDL_PropertiesID) props
+{
+    textField.secureTextEntry = NO;
+
+    switch (SDL_GetTextInputType(props)) {
+    default:
+    case SDL_TEXTINPUT_TYPE_TEXT:
+        textField.keyboardType = UIKeyboardTypeDefault;
+        textField.textContentType = nil;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_NAME:
+        textField.keyboardType = UIKeyboardTypeDefault;
+        textField.textContentType = UITextContentTypeName;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_EMAIL:
+        textField.keyboardType = UIKeyboardTypeEmailAddress;
+        textField.textContentType = UITextContentTypeEmailAddress;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_USERNAME:
+        textField.keyboardType = UIKeyboardTypeDefault;
+        if (@available(iOS 11.0, tvOS 11.0, *)) {
+            textField.textContentType = UITextContentTypeUsername;
+        } else {
+            textField.textContentType = nil;
+        }
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_PASSWORD_HIDDEN:
+        textField.keyboardType = UIKeyboardTypeDefault;
+        if (@available(iOS 11.0, tvOS 11.0, *)) {
+            textField.textContentType = UITextContentTypePassword;
+        } else {
+            textField.textContentType = nil;
+        }
+        textField.secureTextEntry = YES;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_PASSWORD_VISIBLE:
+        textField.keyboardType = UIKeyboardTypeDefault;
+        if (@available(iOS 11.0, tvOS 11.0, *)) {
+            textField.textContentType = UITextContentTypePassword;
+        } else {
+            textField.textContentType = nil;
+        }
+        break;
+    case SDL_TEXTINPUT_TYPE_NUMBER:
+        textField.keyboardType = UIKeyboardTypeNumberPad;
+        textField.textContentType = nil;
+        break;
+    case SDL_TEXTINPUT_TYPE_NUMBER_PASSWORD_HIDDEN:
+        textField.keyboardType = UIKeyboardTypeNumberPad;
+        if (@available(iOS 12.0, tvOS 12.0, *)) {
+            textField.textContentType = UITextContentTypeOneTimeCode;
+        } else {
+            textField.textContentType = nil;
+        }
+        textField.secureTextEntry = YES;
+        break;
+    case SDL_TEXTINPUT_TYPE_NUMBER_PASSWORD_VISIBLE:
+        textField.keyboardType = UIKeyboardTypeNumberPad;
+        if (@available(iOS 12.0, tvOS 12.0, *)) {
+            textField.textContentType = UITextContentTypeOneTimeCode;
+        } else {
+            textField.textContentType = nil;
+        }
+        break;
+    }
+
+    switch (SDL_GetTextInputCapitalization(props)) {
+    default:
+    case SDL_CAPITALIZE_NONE:
+        textField.autocapitalizationType = UITextAutocapitalizationTypeNone;
+        break;
+    case SDL_CAPITALIZE_LETTERS:
+        textField.autocapitalizationType = UITextAutocapitalizationTypeAllCharacters;
+        break;
+    case SDL_CAPITALIZE_WORDS:
+        textField.autocapitalizationType = UITextAutocapitalizationTypeWords;
+        break;
+    case SDL_CAPITALIZE_SENTENCES:
+        textField.autocapitalizationType = UITextAutocapitalizationTypeSentences;
+        break;
+    }
+
+    if (SDL_GetTextInputAutocorrect(props)) {
+        textField.autocorrectionType = UITextAutocorrectionTypeYes;
+        textField.spellCheckingType = UITextSpellCheckingTypeYes;
+    } else {
+        textField.autocorrectionType = UITextAutocorrectionTypeNo;
+        textField.spellCheckingType = UITextSpellCheckingTypeNo;
+    }
+
+    if (SDL_GetTextInputMultiline(props)) {
+        textField.enablesReturnKeyAutomatically = YES;
+    } else {
+        textField.enablesReturnKeyAutomatically = NO;
+    }
+}
+
+// reveal onscreen virtual keyboard
 - (void)showKeyboard
 {
     if (keyboardVisible) {
@@ -420,7 +495,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     }
 }
 
-/* hide onscreen virtual keyboard */
+// hide onscreen virtual keyboard
 - (void)hideKeyboard
 {
     if (!keyboardVisible) {
@@ -437,17 +512,14 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 - (void)keyboardWillShow:(NSNotification *)notification
 {
     BOOL shouldStartTextInput = NO;
-#if !TARGET_OS_TV
-    CGRect kbrect;
-#endif
 
-    if (!SDL_IsTextInputActive() && !hidingKeyboard && !rotatingOrientation) {
+    if (!SDL_TextInputActive(window) && !hidingKeyboard && !rotatingOrientation) {
         shouldStartTextInput = YES;
     }
 
     showingKeyboard = YES;
-#if !TARGET_OS_TV
-    kbrect = [[notification userInfo][UIKeyboardFrameEndUserInfoKey] CGRectValue];
+#ifndef SDL_PLATFORM_TVOS
+    CGRect kbrect = [[notification userInfo][UIKeyboardFrameEndUserInfoKey] CGRectValue];
 
     /* The keyboard rect is in the coordinate space of the screen/window, but we
      * want its height in the coordinate space of the view. */
@@ -457,7 +529,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 #endif
 
     if (shouldStartTextInput) {
-        SDL_StartTextInput();
+        SDL_StartTextInput(window);
     }
 }
 
@@ -470,7 +542,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 {
     BOOL shouldStopTextInput = NO;
 
-    if (SDL_IsTextInputActive() && !showingKeyboard && !rotatingOrientation) {
+    if (SDL_TextInputActive(window) && !showingKeyboard && !rotatingOrientation) {
         shouldStopTextInput = YES;
     }
 
@@ -478,7 +550,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     [self setKeyboardHeight:0];
 
     if (shouldStopTextInput) {
-        SDL_StopTextInput();
+        SDL_StopTextInput(window);
     }
 }
 
@@ -493,7 +565,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
         NSUInteger compareLength = SDL_min(textField.text.length, committedText.length);
         NSUInteger matchLength;
 
-        /* Backspace over characters that are no longer in the string */
+        // Backspace over characters that are no longer in the string
         for (matchLength = 0; matchLength < compareLength; ++matchLength) {
             if ([committedText characterAtIndex:matchLength] != [textField.text characterAtIndex:matchLength]) {
                 break;
@@ -502,9 +574,9 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
         if (matchLength < committedText.length) {
             size_t deleteLength = SDL_utf8strlen([[committedText substringFromIndex:matchLength] UTF8String]);
             while (deleteLength > 0) {
-                /* Send distinct down and up events for each backspace action */
-                SDL_SendVirtualKeyboardKey(SDL_PRESSED, SDL_SCANCODE_BACKSPACE);
-                SDL_SendVirtualKeyboardKey(SDL_RELEASED, SDL_SCANCODE_BACKSPACE);
+                // Send distinct down and up events for each backspace action
+                SDL_SendKeyboardKey(0, SDL_GLOBAL_KEYBOARD_ID, 0, SDL_SCANCODE_BACKSPACE, true);
+                SDL_SendKeyboardKey(0, SDL_GLOBAL_KEYBOARD_ID, 0, SDL_SCANCODE_BACKSPACE, false);
                 --deleteLength;
             }
         }
@@ -516,7 +588,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
                  * convert them to key presses */
                 NSUInteger i;
                 for (i = 0; i < pendingText.length; i++) {
-                    SDL_SendKeyboardUnicodeKey([pendingText characterAtIndex:i]);
+                    SDL_SendKeyboardUnicodeKey(0, [pendingText characterAtIndex:i]);
                 }
             }
             SDL_SendKeyboardText([pendingText UTF8String]);
@@ -527,15 +599,19 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 
 - (void)updateKeyboard
 {
-    SDL_WindowData *data = (__bridge SDL_WindowData *)window->driverdata;
+    SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *) window->internal;
 
     CGAffineTransform t = self.view.transform;
     CGPoint offset = CGPointMake(0.0, 0.0);
+#ifdef SDL_PLATFORM_VISIONOS
+    CGRect frame = UIKit_ComputeViewFrame(window);
+#else
     CGRect frame = UIKit_ComputeViewFrame(window, data.uiwindow.screen);
+#endif
 
-    if (self.keyboardHeight) {
-        int rectbottom = self.textInputRect.y + self.textInputRect.h;
-        int keybottom = self.view.bounds.size.height - self.keyboardHeight;
+    if (self.keyboardHeight && self.textInputRect.h) {
+        int rectbottom = (int)(self.textInputRect.y + self.textInputRect.h);
+        int keybottom = (int)(self.view.bounds.size.height - self.keyboardHeight);
         if (keybottom < rectbottom) {
             offset.y = keybottom - rectbottom;
         }
@@ -547,7 +623,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     t.ty = 0.0;
     offset = CGPointApplyAffineTransform(offset, t);
 
-    /* Apply the updated offset to the view's frame. */
+    // Apply the updated offset to the view's frame.
     frame.origin.x += offset.x;
     frame.origin.y += offset.y;
 
@@ -561,7 +637,7 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     [self updateKeyboard];
 }
 
-/* UITextFieldDelegate method.  Invoked when user types something. */
+// UITextFieldDelegate method.  Invoked when user types something.
 - (BOOL)textField:(UITextField *)_textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
 {
     if (textField.markedTextRange == nil) {
@@ -573,13 +649,13 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
     return YES;
 }
 
-/* Terminates the editing session */
-- (BOOL)textFieldShouldReturn:(UITextField*)_textField
+// Terminates the editing session
+- (BOOL)textFieldShouldReturn:(UITextField *)_textField
 {
-    SDL_SendKeyboardKeyAutoRelease(SDL_SCANCODE_RETURN);
+    SDL_SendKeyboardKeyAutoRelease(0, SDL_SCANCODE_RETURN);
     if (keyboardVisible &&
-        SDL_GetHintBoolean(SDL_HINT_RETURN_KEY_HIDES_IME, SDL_FALSE)) {
-         SDL_StopTextInput();
+        SDL_GetHintBoolean(SDL_HINT_RETURN_KEY_HIDES_IME, false)) {
+        SDL_StopTextInput(window);
     }
     return YES;
 }
@@ -588,36 +664,36 @@ SDL_HideHomeIndicatorHintChanged(void *userdata, const char *name, const char *o
 
 @end
 
-/* iPhone keyboard addition functions */
+// iPhone keyboard addition functions
 #ifdef SDL_IPHONE_KEYBOARD
 
-static SDL_uikitviewcontroller *GetWindowViewController(SDL_Window * window)
+static SDL_uikitviewcontroller *GetWindowViewController(SDL_Window *window)
 {
-    SDL_WindowData *data;
-    if (!window || !window->driverdata) {
+    if (!window || !window->internal) {
         SDL_SetError("Invalid window");
         return nil;
     }
 
-    data = (__bridge SDL_WindowData *)window->driverdata;
+    SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
 
     return data.viewcontroller;
 }
 
-SDL_bool UIKit_HasScreenKeyboardSupport(_THIS)
+bool UIKit_HasScreenKeyboardSupport(SDL_VideoDevice *_this)
 {
-    return SDL_TRUE;
+    return true;
 }
 
-void UIKit_ShowScreenKeyboard(_THIS, SDL_Window *window)
+void UIKit_ShowScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID props)
 {
     @autoreleasepool {
         SDL_uikitviewcontroller *vc = GetWindowViewController(window);
+        [vc setKeyboardProperties:props];
         [vc showKeyboard];
     }
 }
 
-void UIKit_HideScreenKeyboard(_THIS, SDL_Window *window)
+void UIKit_HideScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window)
 {
     @autoreleasepool {
         SDL_uikitviewcontroller *vc = GetWindowViewController(window);
@@ -625,39 +701,32 @@ void UIKit_HideScreenKeyboard(_THIS, SDL_Window *window)
     }
 }
 
-SDL_bool UIKit_IsScreenKeyboardShown(_THIS, SDL_Window *window)
+bool UIKit_IsScreenKeyboardShown(SDL_VideoDevice *_this, SDL_Window *window)
 {
     @autoreleasepool {
         SDL_uikitviewcontroller *vc = GetWindowViewController(window);
         if (vc != nil) {
             return vc.keyboardVisible;
         }
-        return SDL_FALSE;
+        return false;
     }
 }
 
-void UIKit_SetTextInputRect(_THIS, const SDL_Rect *rect)
+bool UIKit_UpdateTextInputArea(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    if (!rect) {
-        SDL_InvalidParamError("rect");
-        return;
-    }
-
     @autoreleasepool {
-        SDL_uikitviewcontroller *vc = GetWindowViewController(SDL_GetFocusWindow());
+        SDL_uikitviewcontroller *vc = GetWindowViewController(window);
         if (vc != nil) {
-            vc.textInputRect = *rect;
+            vc.textInputRect = window->text_input_rect;
 
             if (vc.keyboardVisible) {
                 [vc updateKeyboard];
             }
         }
     }
+    return true;
 }
 
+#endif // SDL_IPHONE_KEYBOARD
 
-#endif /* SDL_IPHONE_KEYBOARD */
-
-#endif /* SDL_VIDEO_DRIVER_UIKIT */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_UIKIT
