@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,15 +18,10 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_DRIVER_UIKIT
+#ifdef SDL_VIDEO_DRIVER_UIKIT
 
-#include "SDL_hints.h"
-#include "SDL_mouse.h"
-#include "SDL_system.h"
-#include "SDL_syswm.h"
-#include "SDL_video.h"
 #include "../SDL_sysvideo.h"
 #include "../SDL_pixels_c.h"
 #include "../../events/SDL_events_c.h"
@@ -35,14 +30,13 @@
 #include "SDL_uikitevents.h"
 #include "SDL_uikitmodes.h"
 #include "SDL_uikitwindow.h"
-#import "SDL_uikitappdelegate.h"
-
-#import "SDL_uikitview.h"
-#import "SDL_uikitopenglview.h"
+#include "SDL_uikitappdelegate.h"
+#include "SDL_uikitview.h"
+#include "SDL_uikitopenglview.h"
 
 #include <Foundation/Foundation.h>
 
-@implementation SDL_WindowData
+@implementation SDL_UIKitWindowData
 
 @synthesize uiwindow;
 @synthesize viewcontroller;
@@ -69,7 +63,8 @@
 
 - (void)layoutSubviews
 {
-    /* Workaround to fix window orientation issues in iOS 8. */
+#ifndef SDL_PLATFORM_VISIONOS
+    // Workaround to fix window orientation issues in iOS 8.
     /* As of July 1 2019, I haven't been able to reproduce any orientation
      * issues with this disabled on iOS 12. The issue this is meant to fix might
      * only happen on iOS 8, or it might have been fixed another way with other
@@ -78,65 +73,60 @@
     if (!UIKit_IsSystemVersionAtLeast(9.0)) {
         self.frame = self.screen.bounds;
     }
+#endif
     [super layoutSubviews];
 }
 
 @end
 
-
-static int
-SetupWindowData(_THIS, SDL_Window *window, UIWindow *uiwindow, SDL_bool created)
+static bool SetupWindowData(SDL_VideoDevice *_this, SDL_Window *window, UIWindow *uiwindow, bool created)
 {
-    SDL_VideoDisplay *display = SDL_GetDisplayForWindow(window);
-    SDL_DisplayData *displaydata = (__bridge SDL_DisplayData *) display->driverdata;
+    SDL_VideoDisplay *display = SDL_GetVideoDisplayForWindow(window);
+    SDL_UIKitDisplayData *displaydata = (__bridge SDL_UIKitDisplayData *)display->internal;
     SDL_uikitview *view;
 
+#ifdef SDL_PLATFORM_VISIONOS
+    CGRect frame = UIKit_ComputeViewFrame(window);
+#else
     CGRect frame = UIKit_ComputeViewFrame(window, displaydata.uiscreen);
-    int width  = (int) frame.size.width;
-    int height = (int) frame.size.height;
+#endif
 
-    SDL_WindowData *data = [[SDL_WindowData alloc] init];
+    int width = (int)frame.size.width;
+    int height = (int)frame.size.height;
+
+    SDL_UIKitWindowData *data = [[SDL_UIKitWindowData alloc] init];
     if (!data) {
         return SDL_OutOfMemory();
     }
 
-    window->driverdata = (void *) CFBridgingRetain(data);
+    window->internal = (SDL_WindowData *)CFBridgingRetain(data);
 
     data.uiwindow = uiwindow;
 
-    /* only one window on iOS, always shown */
-    window->flags &= ~SDL_WINDOW_HIDDEN;
-
+#ifndef SDL_PLATFORM_VISIONOS
     if (displaydata.uiscreen != [UIScreen mainScreen]) {
-        window->flags &= ~SDL_WINDOW_RESIZABLE;  /* window is NEVER resizable */
-        window->flags &= ~SDL_WINDOW_INPUT_FOCUS;  /* never has input focus */
-        window->flags |= SDL_WINDOW_BORDERLESS;  /* never has a status bar. */
+        window->flags &= ~SDL_WINDOW_RESIZABLE;   // window is NEVER resizable
+        window->flags &= ~SDL_WINDOW_INPUT_FOCUS; // never has input focus
+        window->flags |= SDL_WINDOW_BORDERLESS;   // never has a status bar.
     }
+#endif
 
-#if !TARGET_OS_TV
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
     if (displaydata.uiscreen == [UIScreen mainScreen]) {
-        /* SDL_CreateWindow sets the window w&h to the display's bounds if the
-         * fullscreen flag is set. But the display bounds orientation might not
-         * match what we want, and GetSupportedOrientations call below uses the
-         * window w&h. They're overridden below anyway, so we'll just set them
-         * to the requested size for the purposes of determining orientation. */
-        window->w = window->windowed.w;
-        window->h = window->windowed.h;
-
         NSUInteger orients = UIKit_GetSupportedOrientations(window);
         BOOL supportsLandscape = (orients & UIInterfaceOrientationMaskLandscape) != 0;
-        BOOL supportsPortrait = (orients & (UIInterfaceOrientationMaskPortrait|UIInterfaceOrientationMaskPortraitUpsideDown)) != 0;
+        BOOL supportsPortrait = (orients & (UIInterfaceOrientationMaskPortrait | UIInterfaceOrientationMaskPortraitUpsideDown)) != 0;
 
-        /* Make sure the width/height are oriented correctly */
+        // Make sure the width/height are oriented correctly
         if ((width > height && !supportsLandscape) || (height > width && !supportsPortrait)) {
             int temp = width;
             width = height;
             height = temp;
         }
     }
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 
-#if 0 /* Don't set the x/y position, it's already placed on a display */
+#if 0 // Don't set the x/y position, it's already placed on a display
     window->x = 0;
     window->y = 0;
 #endif
@@ -152,23 +142,26 @@ SetupWindowData(_THIS, SDL_Window *window, UIWindow *uiwindow, SDL_bool created)
     view = [[SDL_uikitview alloc] initWithFrame:frame];
 
     /* Sets this view as the controller's view, and adds the view to the window
-     * heirarchy. */
+     * hierarchy. */
     [view setSDLWindow:window];
 
-    return 0;
+    SDL_PropertiesID props = SDL_GetWindowProperties(window);
+    SDL_SetPointerProperty(props, SDL_PROP_WINDOW_UIKIT_WINDOW_POINTER, (__bridge void *)data.uiwindow);
+    SDL_SetNumberProperty(props, SDL_PROP_WINDOW_UIKIT_METAL_VIEW_TAG_NUMBER, SDL_METALVIEW_TAG);
+
+    return true;
 }
 
-int
-UIKit_CreateWindow(_THIS, SDL_Window *window)
+bool UIKit_CreateWindow(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID create_props)
 {
     @autoreleasepool {
-        SDL_VideoDisplay *display = SDL_GetDisplayForWindow(window);
-        SDL_DisplayData *data = (__bridge SDL_DisplayData *) display->driverdata;
+        SDL_VideoDisplay *display = SDL_GetVideoDisplayForWindow(window);
+        SDL_UIKitDisplayData *data = (__bridge SDL_UIKitDisplayData *)display->internal;
         SDL_Window *other;
 
-        /* We currently only handle a single window per display on iOS */
+        // We currently only handle a single window per display on iOS
         for (other = _this->windows; other; other = other->next) {
-            if (other != window && SDL_GetDisplayForWindow(other) == display) {
+            if (other != window && SDL_GetVideoDisplayForWindow(other) == display) {
                 return SDL_SetError("Only one window allowed per display.");
             }
         }
@@ -176,111 +169,109 @@ UIKit_CreateWindow(_THIS, SDL_Window *window)
         /* If monitor has a resolution of 0x0 (hasn't been explicitly set by the
          * user, so it's in standby), try to force the display to a resolution
          * that most closely matches the desired window size. */
-#if !TARGET_OS_TV
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
         const CGSize origsize = data.uiscreen.currentMode.size;
         if ((origsize.width == 0.0f) && (origsize.height == 0.0f)) {
-            if (display->num_display_modes == 0) {
-                _this->GetDisplayModes(_this, display);
+            SDL_DisplayMode bestmode;
+            bool include_high_density_modes = false;
+            if (window->flags & SDL_WINDOW_HIGH_PIXEL_DENSITY) {
+                include_high_density_modes = true;
             }
-
-            int i;
-            const SDL_DisplayMode *bestmode = NULL;
-            for (i = display->num_display_modes; i >= 0; i--) {
-                const SDL_DisplayMode *mode = &display->display_modes[i];
-                if ((mode->w >= window->w) && (mode->h >= window->h)) {
-                    bestmode = mode;
-                }
-            }
-
-            if (bestmode) {
-                SDL_DisplayModeData *modedata = (__bridge SDL_DisplayModeData *)bestmode->driverdata;
+            if (SDL_GetClosestFullscreenDisplayMode(display->id, window->w, window->h, 0.0f, include_high_density_modes, &bestmode)) {
+                SDL_UIKitDisplayModeData *modedata = (__bridge SDL_UIKitDisplayModeData *)bestmode.internal;
                 [data.uiscreen setCurrentMode:modedata.uiscreenmode];
 
                 /* desktop_mode doesn't change here (the higher level will
                  * use it to set all the screens back to their defaults
                  * upon window destruction, SDL_Quit(), etc. */
-                display->current_mode = *bestmode;
+                SDL_SetCurrentDisplayMode(display, &bestmode);
             }
         }
 
         if (data.uiscreen == [UIScreen mainScreen]) {
-            if (window->flags & (SDL_WINDOW_FULLSCREEN|SDL_WINDOW_BORDERLESS)) {
+            if (window->flags & (SDL_WINDOW_FULLSCREEN | SDL_WINDOW_BORDERLESS)) {
                 [UIApplication sharedApplication].statusBarHidden = YES;
             } else {
                 [UIApplication sharedApplication].statusBarHidden = NO;
             }
         }
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 
-        /* ignore the size user requested, and make a fullscreen window */
-        /* !!! FIXME: can we have a smaller view? */
+        // ignore the size user requested, and make a fullscreen window
+        // !!! FIXME: can we have a smaller view?
+#ifdef SDL_PLATFORM_VISIONOS
+        UIWindow *uiwindow = [[SDL_uikitwindow alloc] initWithFrame:CGRectMake(window->x, window->y, window->w, window->h)];
+#else
         UIWindow *uiwindow = [[SDL_uikitwindow alloc] initWithFrame:data.uiscreen.bounds];
+#endif
 
-        /* put the window on an external display if appropriate. */
+        // put the window on an external display if appropriate.
+#ifndef SDL_PLATFORM_VISIONOS
         if (data.uiscreen != [UIScreen mainScreen]) {
             [uiwindow setScreen:data.uiscreen];
         }
+#endif
 
-        if (SetupWindowData(_this, window, uiwindow, SDL_TRUE) < 0) {
-            return -1;
+        if (!SetupWindowData(_this, window, uiwindow, true)) {
+            return false;
         }
     }
 
-    return 1;
+    return true;
 }
 
-void
-UIKit_SetWindowTitle(_THIS, SDL_Window * window)
+void UIKit_SetWindowTitle(SDL_VideoDevice *_this, SDL_Window *window)
 {
     @autoreleasepool {
-        SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+        SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
         data.viewcontroller.title = @(window->title);
     }
 }
 
-void
-UIKit_ShowWindow(_THIS, SDL_Window * window)
+void UIKit_ShowWindow(SDL_VideoDevice *_this, SDL_Window *window)
 {
     @autoreleasepool {
-        SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+        SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
         [data.uiwindow makeKeyAndVisible];
 
-        /* Make this window the current mouse focus for touch input */
-        SDL_VideoDisplay *display = SDL_GetDisplayForWindow(window);
-        SDL_DisplayData *displaydata = (__bridge SDL_DisplayData *) display->driverdata;
-        if (displaydata.uiscreen == [UIScreen mainScreen]) {
+        // Make this window the current mouse focus for touch input
+        SDL_VideoDisplay *display = SDL_GetVideoDisplayForWindow(window);
+        SDL_UIKitDisplayData *displaydata = (__bridge SDL_UIKitDisplayData *)display->internal;
+#ifndef SDL_PLATFORM_VISIONOS
+        if (displaydata.uiscreen == [UIScreen mainScreen])
+#endif
+        {
             SDL_SetMouseFocus(window);
             SDL_SetKeyboardFocus(window);
         }
     }
 }
 
-void
-UIKit_HideWindow(_THIS, SDL_Window * window)
+void UIKit_HideWindow(SDL_VideoDevice *_this, SDL_Window *window)
 {
     @autoreleasepool {
-        SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+        SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
         data.uiwindow.hidden = YES;
     }
 }
 
-void
-UIKit_RaiseWindow(_THIS, SDL_Window * window)
+void UIKit_RaiseWindow(SDL_VideoDevice *_this, SDL_Window *window)
 {
+#if defined(SDL_VIDEO_OPENGL_ES) || defined(SDL_VIDEO_OPENGL_ES2)
     /* We don't currently offer a concept of "raising" the SDL window, since
      * we only allow one per display, in the iOS fashion.
      * However, we use this entry point to rebind the context to the view
      * during OnWindowRestored processing. */
     _this->GL_MakeCurrent(_this, _this->current_glwin, _this->current_glctx);
+#endif
 }
 
-static void
-UIKit_UpdateWindowBorder(_THIS, SDL_Window * window)
+static void UIKit_UpdateWindowBorder(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+    SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
     SDL_uikitviewcontroller *viewcontroller = data.viewcontroller;
 
-#if !TARGET_OS_TV
+#if !defined(SDL_PLATFORM_TVOS) && !defined(SDL_PLATFORM_VISIONOS)
     if (data.uiwindow.screen == [UIScreen mainScreen]) {
         if (window->flags & (SDL_WINDOW_FULLSCREEN | SDL_WINDOW_BORDERLESS)) {
             [UIApplication sharedApplication].statusBarHidden = YES;
@@ -291,12 +282,12 @@ UIKit_UpdateWindowBorder(_THIS, SDL_Window * window)
         [viewcontroller setNeedsStatusBarAppearanceUpdate];
     }
 
-    /* Update the view's frame to account for the status bar change. */
+    // Update the view's frame to account for the status bar change.
     viewcontroller.view.frame = UIKit_ComputeViewFrame(window, data.uiwindow.screen);
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 
 #ifdef SDL_IPHONE_KEYBOARD
-    /* Make sure the view is offset correctly when the keyboard is visible. */
+    // Make sure the view is offset correctly when the keyboard is visible.
     [viewcontroller updateKeyboard];
 #endif
 
@@ -304,50 +295,47 @@ UIKit_UpdateWindowBorder(_THIS, SDL_Window * window)
     [viewcontroller.view layoutIfNeeded];
 }
 
-void
-UIKit_SetWindowBordered(_THIS, SDL_Window * window, SDL_bool bordered)
+void UIKit_SetWindowBordered(SDL_VideoDevice *_this, SDL_Window *window, bool bordered)
 {
     @autoreleasepool {
+        if (bordered) {
+            window->flags &= ~SDL_WINDOW_BORDERLESS;
+        } else {
+            window->flags |= SDL_WINDOW_BORDERLESS;
+        }
         UIKit_UpdateWindowBorder(_this, window);
     }
 }
 
-void
-UIKit_SetWindowFullscreen(_THIS, SDL_Window * window, SDL_VideoDisplay * display, SDL_bool fullscreen)
+SDL_FullscreenResult UIKit_SetWindowFullscreen(SDL_VideoDevice *_this, SDL_Window *window, SDL_VideoDisplay *display, SDL_FullscreenOp fullscreen)
 {
     @autoreleasepool {
+        SDL_SendWindowEvent(window, fullscreen ? SDL_EVENT_WINDOW_ENTER_FULLSCREEN : SDL_EVENT_WINDOW_LEAVE_FULLSCREEN, 0, 0);
         UIKit_UpdateWindowBorder(_this, window);
     }
+    return SDL_FULLSCREEN_SUCCEEDED;
 }
 
-void
-UIKit_SetWindowMouseGrab(_THIS, SDL_Window * window, SDL_bool grabbed)
+void UIKit_UpdatePointerLock(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    /* There really isn't a concept of window grab or cursor confinement on iOS */
-}
-
-void
-UIKit_UpdatePointerLock(_THIS, SDL_Window * window)
-{
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
 #if defined(__IPHONE_14_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_14_0
     @autoreleasepool {
-        SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+        SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
         SDL_uikitviewcontroller *viewcontroller = data.viewcontroller;
         if (@available(iOS 14.0, *)) {
             [viewcontroller setNeedsUpdateOfPrefersPointerLocked];
         }
     }
-#endif /* defined(__IPHONE_14_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_14_0 */
-#endif /* !TARGET_OS_TV */
+#endif // defined(__IPHONE_14_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_14_0
+#endif // !SDL_PLATFORM_TVOS
 }
 
-void
-UIKit_DestroyWindow(_THIS, SDL_Window * window)
+void UIKit_DestroyWindow(SDL_VideoDevice *_this, SDL_Window *window)
 {
     @autoreleasepool {
-        if (window->driverdata != NULL) {
-            SDL_WindowData *data = (SDL_WindowData *) CFBridgingRelease(window->driverdata);
+        if (window->internal != NULL) {
+            SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
             NSArray *views = nil;
 
             [data.viewcontroller stopAnimation];
@@ -366,79 +354,48 @@ UIKit_DestroyWindow(_THIS, SDL_Window * window)
              * SDL window. */
             data.uiwindow.rootViewController = nil;
             data.uiwindow.hidden = YES;
+
+            CFRelease(window->internal);
+            window->internal = NULL;
         }
     }
-    window->driverdata = NULL;
 }
 
-void
-UIKit_GetWindowSizeInPixels(_THIS, SDL_Window * window, int *w, int *h)
-{ @autoreleasepool
-{
-    SDL_WindowData *windata = (__bridge SDL_WindowData *) window->driverdata;
-    UIView *view = windata.viewcontroller.view;
-    CGSize size = view.bounds.size;
-    CGFloat scale = 1.0;
-
-    if (window->flags & SDL_WINDOW_ALLOW_HIGHDPI) {
-        scale = windata.uiwindow.screen.nativeScale;
-    }
-
-    /* Integer truncation of fractional values matches SDL_uikitmetalview and
-     * SDL_uikitopenglview. */
-    *w = size.width * scale;
-    *h = size.height * scale;
-}}
-
-SDL_bool
-UIKit_GetWindowWMInfo(_THIS, SDL_Window * window, SDL_SysWMinfo * info)
+void UIKit_GetWindowSizeInPixels(SDL_VideoDevice *_this, SDL_Window *window, int *w, int *h)
 {
     @autoreleasepool {
-        SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+        SDL_UIKitWindowData *windata = (__bridge SDL_UIKitWindowData *)window->internal;
+        UIView *view = windata.viewcontroller.view;
+        CGSize size = view.bounds.size;
+        CGFloat scale = 1.0;
 
-        if (info->version.major <= SDL_MAJOR_VERSION) {
-            int versionnum = SDL_VERSIONNUM(info->version.major, info->version.minor, info->version.patch);
 
-            info->subsystem = SDL_SYSWM_UIKIT;
-            info->info.uikit.window = data.uiwindow;
-
-            /* These struct members were added in SDL 2.0.4. */
-            if (versionnum >= SDL_VERSIONNUM(2,0,4)) {
-#if SDL_VIDEO_OPENGL_ES || SDL_VIDEO_OPENGL_ES2
-                if ([data.viewcontroller.view isKindOfClass:[SDL_uikitopenglview class]]) {
-                    SDL_uikitopenglview *glview = (SDL_uikitopenglview *)data.viewcontroller.view;
-                    info->info.uikit.framebuffer = glview.drawableFramebuffer;
-                    info->info.uikit.colorbuffer = glview.drawableRenderbuffer;
-                    info->info.uikit.resolveFramebuffer = glview.msaaResolveFramebuffer;
-                } else {
+        if (window->flags & SDL_WINDOW_HIGH_PIXEL_DENSITY) {
+#ifndef SDL_PLATFORM_VISIONOS
+            scale = windata.uiwindow.screen.nativeScale;
 #else
-                {
+            scale = 2.0;
 #endif
-                    info->info.uikit.framebuffer = 0;
-                    info->info.uikit.colorbuffer = 0;
-                    info->info.uikit.resolveFramebuffer = 0;
-                }
-            }
-
-            return SDL_TRUE;
-        } else {
-            SDL_SetError("Application not compiled with SDL %d",
-                         SDL_MAJOR_VERSION);
-            return SDL_FALSE;
         }
+
+
+        /* Integer truncation of fractional values matches SDL_uikitmetalview and
+         * SDL_uikitopenglview. */
+        *w = (int)(size.width * scale);
+        *h = (int)(size.height * scale);
     }
 }
 
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
 NSUInteger
-UIKit_GetSupportedOrientations(SDL_Window * window)
+UIKit_GetSupportedOrientations(SDL_Window *window)
 {
     const char *hint = SDL_GetHint(SDL_HINT_ORIENTATIONS);
     NSUInteger validOrientations = UIInterfaceOrientationMaskAll;
     NSUInteger orientationMask = 0;
 
     @autoreleasepool {
-        SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+        SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
         UIApplication *app = [UIApplication sharedApplication];
 
         /* Get all possible valid orientations. If the app delegate doesn't tell
@@ -467,20 +424,20 @@ UIKit_GetSupportedOrientations(SDL_Window * window)
         }
 
         if (orientationMask == 0 && (window->flags & SDL_WINDOW_RESIZABLE)) {
-            /* any orientation is okay. */
+            // any orientation is okay.
             orientationMask = UIInterfaceOrientationMaskAll;
         }
 
         if (orientationMask == 0) {
-            if (window->w >= window->h) {
+            if (window->floating.w >= window->floating.h) {
                 orientationMask |= UIInterfaceOrientationMaskLandscape;
             }
-            if (window->h >= window->w) {
+            if (window->floating.h >= window->floating.w) {
                 orientationMask |= (UIInterfaceOrientationMaskPortrait | UIInterfaceOrientationMaskPortraitUpsideDown);
             }
         }
 
-        /* Don't allow upside-down orientation on phones, so answering calls is in the natural orientation */
+        // Don't allow upside-down orientation on phones, so answering calls is in the natural orientation
         if ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPhone) {
             orientationMask &= ~UIInterfaceOrientationMaskPortraitUpsideDown;
         }
@@ -495,25 +452,22 @@ UIKit_GetSupportedOrientations(SDL_Window * window)
 
     return orientationMask;
 }
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 
-int
-SDL_iPhoneSetAnimationCallback(SDL_Window * window, int interval, void (*callback)(void*), void *callbackParam)
+bool SDL_SetiOSAnimationCallback(SDL_Window *window, int interval, SDL_iOSAnimationCallback callback, void *callbackParam)
 {
-    if (!window || !window->driverdata) {
+    if (!window || !window->internal) {
         return SDL_SetError("Invalid window");
     }
 
     @autoreleasepool {
-        SDL_WindowData *data = (__bridge SDL_WindowData *)window->driverdata;
+        SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
         [data.viewcontroller setAnimationCallback:interval
                                          callback:callback
                                     callbackParam:callbackParam];
     }
 
-    return 0;
+    return true;
 }
 
-#endif /* SDL_VIDEO_DRIVER_UIKIT */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_UIKIT
