@@ -19,20 +19,16 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_VIDEO_DRIVER_VITA
 
-/* SDL internals */
+// SDL internals
 #include "../SDL_sysvideo.h"
-#include "SDL_version.h"
-#include "SDL_syswm.h"
-#include "SDL_loadso.h"
-#include "SDL_events.h"
 #include "../../events/SDL_mouse_c.h"
 #include "../../events/SDL_keyboard_c.h"
 
-/* VITA declarations */
+// VITA declarations
 #include <psp2/kernel/processmgr.h>
 #include "SDL_vitavideo.h"
 #include "SDL_vitatouch.h"
@@ -41,51 +37,44 @@
 #include "SDL_vitaframebuffer.h"
 #include "SDL_vitamessagebox.h"
 
-#if defined(SDL_VIDEO_VITA_PIB)
+#ifdef SDL_VIDEO_VITA_PIB
 #include "SDL_vitagles_c.h"
 #elif defined(SDL_VIDEO_VITA_PVR)
 #include "SDL_vitagles_pvr_c.h"
-#if defined(SDL_VIDEO_VITA_PVR_OGL)
+#ifdef SDL_VIDEO_VITA_PVR_OGL
 #include "SDL_vitagl_pvr_c.h"
 #endif
-  #define VITA_GLES_GetProcAddress SDL_EGL_GetProcAddress
-  #define VITA_GLES_UnloadLibrary SDL_EGL_UnloadLibrary
-  #define VITA_GLES_SetSwapInterval SDL_EGL_SetSwapInterval
-  #define VITA_GLES_GetSwapInterval SDL_EGL_GetSwapInterval
-  #define VITA_GLES_DeleteContext SDL_EGL_DeleteContext
+#define VITA_GLES_GetProcAddress  SDL_EGL_GetProcAddressInternal
+#define VITA_GLES_UnloadLibrary   SDL_EGL_UnloadLibrary
+#define VITA_GLES_SetSwapInterval SDL_EGL_SetSwapInterval
+#define VITA_GLES_GetSwapInterval SDL_EGL_GetSwapInterval
+#define VITA_GLES_DestroyContext   SDL_EGL_DestroyContext
 #endif
 
 SDL_Window *Vita_Window;
 
 static void VITA_Destroy(SDL_VideoDevice *device)
 {
-    /*    SDL_VideoData *phdata = (SDL_VideoData *) device->driverdata; */
-
-    SDL_free(device->driverdata);
+    SDL_free(device->internal);
     SDL_free(device);
-    //    if (device->driverdata != NULL) {
-    //        device->driverdata = NULL;
-    //    }
 }
 
-static SDL_VideoDevice *VITA_Create()
+static SDL_VideoDevice *VITA_Create(void)
 {
     SDL_VideoDevice *device;
     SDL_VideoData *phdata;
 #ifdef SDL_VIDEO_VITA_PIB
     SDL_GLDriverData *gldata;
 #endif
-    /* Initialize SDL_VideoDevice structure */
+    // Initialize SDL_VideoDevice structure
     device = (SDL_VideoDevice *)SDL_calloc(1, sizeof(SDL_VideoDevice));
     if (!device) {
-        SDL_OutOfMemory();
         return NULL;
     }
 
-    /* Initialize internal VITA specific data */
+    // Initialize internal VITA specific data
     phdata = (SDL_VideoData *)SDL_calloc(1, sizeof(SDL_VideoData));
     if (!phdata) {
-        SDL_OutOfMemory();
         SDL_free(device);
         return NULL;
     }
@@ -93,33 +82,28 @@ static SDL_VideoDevice *VITA_Create()
 
     gldata = (SDL_GLDriverData *)SDL_calloc(1, sizeof(SDL_GLDriverData));
     if (!gldata) {
-        SDL_OutOfMemory();
         SDL_free(device);
         SDL_free(phdata);
         return NULL;
     }
     device->gl_data = gldata;
-    phdata->egl_initialized = SDL_TRUE;
+    phdata->egl_initialized = true;
 #endif
-    phdata->ime_active = SDL_FALSE;
+    phdata->ime_active = false;
 
-    device->driverdata = phdata;
+    device->internal = phdata;
 
-    /* Setup amount of available displays and current display */
+    // Setup amount of available displays and current display
     device->num_displays = 0;
 
-    /* Set device free function */
+    // Set device free function
     device->free = VITA_Destroy;
 
-    /* Setup all functions which we can handle */
+    // Setup all functions which we can handle
     device->VideoInit = VITA_VideoInit;
     device->VideoQuit = VITA_VideoQuit;
-    device->GetDisplayModes = VITA_GetDisplayModes;
-    device->SetDisplayMode = VITA_SetDisplayMode;
     device->CreateSDLWindow = VITA_CreateWindow;
-    device->CreateSDLWindowFrom = VITA_CreateWindowFrom;
     device->SetWindowTitle = VITA_SetWindowTitle;
-    device->SetWindowIcon = VITA_SetWindowIcon;
     device->SetWindowPosition = VITA_SetWindowPosition;
     device->SetWindowSize = VITA_SetWindowSize;
     device->ShowWindow = VITA_ShowWindow;
@@ -131,7 +115,6 @@ static SDL_VideoDevice *VITA_Create()
     device->SetWindowMouseGrab = VITA_SetWindowGrab;
     device->SetWindowKeyboardGrab = VITA_SetWindowGrab;
     device->DestroyWindow = VITA_DestroyWindow;
-    device->GetWindowWMInfo = VITA_GetWindowWMInfo;
 
     /*
         // Disabled, causes issues on high-framerate updates. SDL still emulates this.
@@ -141,8 +124,8 @@ static SDL_VideoDevice *VITA_Create()
     */
 
 #if defined(SDL_VIDEO_VITA_PIB) || defined(SDL_VIDEO_VITA_PVR)
-#if defined(SDL_VIDEO_VITA_PVR_OGL)
-    if (SDL_getenv("VITA_PVR_OGL") != NULL) {
+#ifdef SDL_VIDEO_VITA_PVR_OGL
+    if (SDL_GetHintBoolean(SDL_HINT_VITA_PVR_OPENGL, false)) {
         device->GL_LoadLibrary = VITA_GL_LoadLibrary;
         device->GL_CreateContext = VITA_GL_CreateContext;
         device->GL_GetProcAddress = VITA_GL_GetProcAddress;
@@ -151,7 +134,7 @@ static SDL_VideoDevice *VITA_Create()
         device->GL_LoadLibrary = VITA_GLES_LoadLibrary;
         device->GL_CreateContext = VITA_GLES_CreateContext;
         device->GL_GetProcAddress = VITA_GLES_GetProcAddress;
-#if defined(SDL_VIDEO_VITA_PVR_OGL)
+#ifdef SDL_VIDEO_VITA_PVR_OGL
     }
 #endif
 
@@ -160,7 +143,7 @@ static SDL_VideoDevice *VITA_Create()
     device->GL_SetSwapInterval = VITA_GLES_SetSwapInterval;
     device->GL_GetSwapInterval = VITA_GLES_GetSwapInterval;
     device->GL_SwapWindow = VITA_GLES_SwapWindow;
-    device->GL_DeleteContext = VITA_GLES_DeleteContext;
+    device->GL_DestroyContext = VITA_GLES_DestroyContext;
 #endif
 
     device->HasScreenKeyboardSupport = VITA_HasScreenKeyboardSupport;
@@ -174,98 +157,84 @@ static SDL_VideoDevice *VITA_Create()
 }
 
 VideoBootStrap VITA_bootstrap = {
-    "VITA",
+    "vita",
     "VITA Video Driver",
     VITA_Create,
     VITA_ShowMessageBox
 };
 
 /*****************************************************************************/
-/* SDL Video and Display initialization/handling functions                   */
+// SDL Video and Display initialization/handling functions
 /*****************************************************************************/
-int VITA_VideoInit(_THIS)
+bool VITA_VideoInit(SDL_VideoDevice *_this)
 {
-    SDL_VideoDisplay display;
-    SDL_DisplayMode current_mode;
-#if defined(SDL_VIDEO_VITA_PVR)
-    char *res = SDL_getenv("VITA_RESOLUTION");
+    SDL_DisplayMode mode;
+#ifdef SDL_VIDEO_VITA_PVR
+    const char *res = SDL_GetHint(SDL_HINT_VITA_RESOLUTION);
 #endif
-    SDL_zero(current_mode);
+    SDL_zero(mode);
 
-#if defined(SDL_VIDEO_VITA_PVR)
+#ifdef SDL_VIDEO_VITA_PVR
     if (res) {
-        /* 1088i for PSTV (Or Sharpscale) */
-        if (!SDL_strncmp(res, "1080", 4)) {
-            current_mode.w = 1920;
-            current_mode.h = 1088;
+        // 1088i for PSTV (Or Sharpscale)
+        if (SDL_strncmp(res, "1080", 4) == 0) {
+            mode.w = 1920;
+            mode.h = 1088;
         }
-        /* 725p for PSTV (Or Sharpscale) */
-        else if (!SDL_strncmp(res, "720", 3)) {
-            current_mode.w = 1280;
-            current_mode.h = 725;
+        // 725p for PSTV (Or Sharpscale)
+        else if (SDL_strncmp(res, "720", 3) == 0) {
+            mode.w = 1280;
+            mode.h = 725;
         }
     }
-    /* 544p */
+    // 544p
     else {
 #endif
-        current_mode.w = 960;
-        current_mode.h = 544;
-#if defined(SDL_VIDEO_VITA_PVR)
+        mode.w = 960;
+        mode.h = 544;
+#ifdef SDL_VIDEO_VITA_PVR
     }
 #endif
 
-    current_mode.refresh_rate = 60;
-    /* 32 bpp for default */
-    current_mode.format = SDL_PIXELFORMAT_ABGR8888;
+    mode.refresh_rate = 60.0f;
 
-    current_mode.driverdata = NULL;
+    // 32 bpp for default
+    mode.format = SDL_PIXELFORMAT_ABGR8888;
 
-    SDL_zero(display);
-    display.desktop_mode = current_mode;
-    display.current_mode = current_mode;
-    display.driverdata = NULL;
+    if (SDL_AddBasicVideoDisplay(&mode) == 0) {
+        return false;
+    }
 
-    SDL_AddVideoDisplay(&display, SDL_FALSE);
     VITA_InitTouch();
     VITA_InitKeyboard();
     VITA_InitMouse();
 
-    return 1;
+    return true;
 }
 
-void VITA_VideoQuit(_THIS)
+void VITA_VideoQuit(SDL_VideoDevice *_this)
 {
     VITA_QuitTouch();
 }
 
-void VITA_GetDisplayModes(_THIS, SDL_VideoDisplay *display)
-{
-    SDL_AddDisplayMode(display, &display->current_mode);
-}
-
-int VITA_SetDisplayMode(_THIS, SDL_VideoDisplay *display, SDL_DisplayMode *mode)
-{
-    return 0;
-}
-
-int VITA_CreateWindow(_THIS, SDL_Window *window)
+bool VITA_CreateWindow(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID create_props)
 {
     SDL_WindowData *wdata;
-#if defined(SDL_VIDEO_VITA_PVR)
+#ifdef SDL_VIDEO_VITA_PVR
     Psp2NativeWindow win;
     int temp_major = 2;
     int temp_minor = 1;
     int temp_profile = 0;
 #endif
 
-    /* Allocate window internal data */
+    // Allocate window internal data
     wdata = (SDL_WindowData *)SDL_calloc(1, sizeof(SDL_WindowData));
     if (!wdata) {
-        return SDL_OutOfMemory();
+        return false;
     }
 
-    /* Setup driver data for this window */
-    window->driverdata = wdata;
+    // Setup driver data for this window
+    window->internal = wdata;
 
     // Vita can only have one window
     if (Vita_Window) {
@@ -274,26 +243,27 @@ int VITA_CreateWindow(_THIS, SDL_Window *window)
 
     Vita_Window = window;
 
-#if defined(SDL_VIDEO_VITA_PVR)
+#ifdef SDL_VIDEO_VITA_PVR
     win.type = PSP2_DRAWABLE_TYPE_WINDOW;
     win.numFlipBuffers = 2;
     win.flipChainThrdAffinity = 0x20000;
 
-    /* 1088i for PSTV (Or Sharpscale) */
+    // 1088i for PSTV (Or Sharpscale)
     if (window->w == 1920) {
         win.windowSize = PSP2_WINDOW_1920X1088;
     }
-    /* 725p for PSTV (Or Sharpscale) */
+    // 725p for PSTV (Or Sharpscale)
     else if (window->w == 1280) {
         win.windowSize = PSP2_WINDOW_1280X725;
     }
-    /* 544p */
+    // 544p
     else {
         win.windowSize = PSP2_WINDOW_960X544;
     }
     if (window->flags & SDL_WINDOW_OPENGL) {
-        if (SDL_getenv("VITA_PVR_OGL") != NULL) {
-            /* Set version to 2.1 and PROFILE to ES */
+        bool use_opengl = SDL_GetHintBoolean(SDL_HINT_VITA_PVR_OPENGL, false);
+        if (use_opengl) {
+            // Set version to 2.1 and PROFILE to ES
             temp_major = _this->gl_config.major_version;
             temp_minor = _this->gl_config.minor_version;
             temp_profile = _this->gl_config.profile_mask;
@@ -302,12 +272,12 @@ int VITA_CreateWindow(_THIS, SDL_Window *window)
             _this->gl_config.minor_version = 1;
             _this->gl_config.profile_mask = SDL_GL_CONTEXT_PROFILE_ES;
         }
-        wdata->egl_surface = SDL_EGL_CreateSurface(_this, &win);
+        wdata->egl_surface = SDL_EGL_CreateSurface(_this, window, &win);
         if (wdata->egl_surface == EGL_NO_SURFACE) {
             return SDL_SetError("Could not create GLES window surface");
         }
-        if (SDL_getenv("VITA_PVR_OGL") != NULL) {
-            /* Revert */
+        if (use_opengl) {
+            // Revert
             _this->gl_config.major_version = temp_major;
             _this->gl_config.minor_version = temp_minor;
             _this->gl_config.profile_mask = temp_profile;
@@ -318,87 +288,63 @@ int VITA_CreateWindow(_THIS, SDL_Window *window)
     // fix input, we need to find a better way
     SDL_SetKeyboardFocus(window);
 
-    /* Window has been successfully created */
-    return 0;
+    // Window has been successfully created
+    return true;
 }
 
-int VITA_CreateWindowFrom(_THIS, SDL_Window *window, const void *data)
+void VITA_SetWindowTitle(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    return -1;
+}
+bool VITA_SetWindowPosition(SDL_VideoDevice *_this, SDL_Window *window)
+{
+    return SDL_Unsupported();
+}
+void VITA_SetWindowSize(SDL_VideoDevice *_this, SDL_Window *window)
+{
+}
+void VITA_ShowWindow(SDL_VideoDevice *_this, SDL_Window *window)
+{
+}
+void VITA_HideWindow(SDL_VideoDevice *_this, SDL_Window *window)
+{
+}
+void VITA_RaiseWindow(SDL_VideoDevice *_this, SDL_Window *window)
+{
+}
+void VITA_MaximizeWindow(SDL_VideoDevice *_this, SDL_Window *window)
+{
+}
+void VITA_MinimizeWindow(SDL_VideoDevice *_this, SDL_Window *window)
+{
+}
+void VITA_RestoreWindow(SDL_VideoDevice *_this, SDL_Window *window)
+{
+}
+bool VITA_SetWindowGrab(SDL_VideoDevice *_this, SDL_Window *window, bool grabbed)
+{
+    return true;
 }
 
-void VITA_SetWindowTitle(_THIS, SDL_Window *window)
+void VITA_DestroyWindow(SDL_VideoDevice *_this, SDL_Window *window)
 {
-}
-void VITA_SetWindowIcon(_THIS, SDL_Window *window, SDL_Surface *icon)
-{
-}
-void VITA_SetWindowPosition(_THIS, SDL_Window *window)
-{
-}
-void VITA_SetWindowSize(_THIS, SDL_Window *window)
-{
-}
-void VITA_ShowWindow(_THIS, SDL_Window *window)
-{
-}
-void VITA_HideWindow(_THIS, SDL_Window *window)
-{
-}
-void VITA_RaiseWindow(_THIS, SDL_Window *window)
-{
-}
-void VITA_MaximizeWindow(_THIS, SDL_Window *window)
-{
-}
-void VITA_MinimizeWindow(_THIS, SDL_Window *window)
-{
-}
-void VITA_RestoreWindow(_THIS, SDL_Window *window)
-{
-}
-void VITA_SetWindowGrab(_THIS, SDL_Window *window, SDL_bool grabbed)
-{
-}
-
-void VITA_DestroyWindow(_THIS, SDL_Window *window)
-{
-    //    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
     SDL_WindowData *data;
 
-    data = window->driverdata;
+    data = window->internal;
     if (data) {
         // TODO: should we destroy egl context? No one sane should recreate ogl window as non-ogl
         SDL_free(data);
     }
 
-    window->driverdata = NULL;
+    window->internal = NULL;
     Vita_Window = NULL;
 }
 
-/*****************************************************************************/
-/* SDL Window Manager function                                               */
-/*****************************************************************************/
-SDL_bool VITA_GetWindowWMInfo(_THIS, SDL_Window * window, struct SDL_SysWMinfo *info)
+bool VITA_HasScreenKeyboardSupport(SDL_VideoDevice *_this)
 {
-    if (info->version.major <= SDL_MAJOR_VERSION) {
-        return SDL_TRUE;
-    } else {
-        SDL_SetError("application not compiled with SDL %d\n",
-                     SDL_MAJOR_VERSION);
-        return SDL_FALSE;
-    }
-
-    /* Failed to get window manager information */
-    return SDL_FALSE;
+    return true;
 }
 
-SDL_bool VITA_HasScreenKeyboardSupport(_THIS)
-{
-    return SDL_TRUE;
-}
-
-#if !defined(SCE_IME_LANGUAGE_ENGLISH_US)
+#ifndef SCE_IME_LANGUAGE_ENGLISH_US
 #define SCE_IME_LANGUAGE_ENGLISH_US SCE_IME_LANGUAGE_ENGLISH
 #endif
 
@@ -427,7 +373,7 @@ static void utf16_to_utf8(const uint16_t *src, uint8_t *dst)
     *dst = '\0';
 }
 
-#if defined(SDL_VIDEO_VITA_PVR)
+#ifdef SDL_VIDEO_VITA_PVR
 SceWChar16 libime_out[SCE_IME_MAX_PREEDIT_LENGTH + SCE_IME_MAX_TEXT_LENGTH + 1];
 char libime_initval[8] = { 1 };
 SceImeCaret caret_rev;
@@ -435,19 +381,17 @@ SceImeCaret caret_rev;
 void VITA_ImeEventHandler(void *arg, const SceImeEventData *e)
 {
     SDL_VideoData *videodata = (SDL_VideoData *)arg;
-    SDL_Scancode scancode;
     uint8_t utf8_buffer[SCE_IME_MAX_TEXT_LENGTH];
     switch (e->id) {
     case SCE_IME_EVENT_UPDATE_TEXT:
         if (e->param.text.caretIndex == 0) {
-            SDL_SendKeyboardKeyAutoRelease(SDL_SCANCODE_BACKSPACE);
+            SDL_SendKeyboardKeyAutoRelease(0, SDL_SCANCODE_BACKSPACE);
             sceImeSetText((SceWChar16 *)libime_initval, 4);
         } else {
-            scancode = SDL_GetScancodeFromKey(*(SceWChar16 *)&libime_out[1]);
-            if (scancode == SDL_SCANCODE_SPACE) {
-                SDL_SendKeyboardKeyAutoRelease(SDL_SCANCODE_SPACE);
+            utf16_to_utf8((SceWChar16 *)&libime_out[1], utf8_buffer);
+            if (utf8_buffer[0] == ' ') {
+                SDL_SendKeyboardKeyAutoRelease(0, SDL_SCANCODE_SPACE);
             } else {
-                utf16_to_utf8((SceWChar16 *)&libime_out[1], utf8_buffer);
                 SDL_SendKeyboardText((const char *)utf8_buffer);
             }
             SDL_memset(&caret_rev, 0, sizeof(SceImeCaret));
@@ -458,21 +402,22 @@ void VITA_ImeEventHandler(void *arg, const SceImeEventData *e)
         }
         break;
     case SCE_IME_EVENT_PRESS_ENTER:
-        SDL_SendKeyboardKeyAutoRelease(SDL_SCANCODE_RETURN);
+        SDL_SendKeyboardKeyAutoRelease(0, SDL_SCANCODE_RETURN);
+        break;
     case SCE_IME_EVENT_PRESS_CLOSE:
         sceImeClose();
-        videodata->ime_active = SDL_FALSE;
+        videodata->ime_active = false;
         break;
     }
 }
 #endif
 
-void VITA_ShowScreenKeyboard(_THIS, SDL_Window *window)
+void VITA_ShowScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID props)
 {
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *videodata = _this->internal;
     SceInt32 res;
 
-#if defined(SDL_VIDEO_VITA_PVR)
+#ifdef SDL_VIDEO_VITA_PVR
 
     SceUInt32 libime_work[SCE_IME_WORK_BUFFER_SIZE / sizeof(SceInt32)];
     SceImeParam param;
@@ -483,8 +428,46 @@ void VITA_ShowScreenKeyboard(_THIS, SDL_Window *window)
 
     param.supportedLanguages = SCE_IME_LANGUAGE_ENGLISH_US;
     param.languagesForced = SCE_FALSE;
-    param.type = SCE_IME_TYPE_DEFAULT;
-    param.option = SCE_IME_OPTION_NO_ASSISTANCE;
+    switch (SDL_GetTextInputType(props)) {
+    default:
+    case SDL_TEXTINPUT_TYPE_TEXT:
+        param.type = SCE_IME_TYPE_DEFAULT;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_NAME:
+        param.type = SCE_IME_TYPE_DEFAULT;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_EMAIL:
+        param.type = SCE_IME_TYPE_MAIL;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_USERNAME:
+        param.type = SCE_IME_TYPE_DEFAULT;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_PASSWORD_HIDDEN:
+        param.type = SCE_IME_TYPE_DEFAULT;
+        break;
+    case SDL_TEXTINPUT_TYPE_TEXT_PASSWORD_VISIBLE:
+        param.type = SCE_IME_TYPE_DEFAULT;
+        break;
+    case SDL_TEXTINPUT_TYPE_NUMBER:
+        param.type = SCE_IME_TYPE_NUMBER;
+        break;
+    case SDL_TEXTINPUT_TYPE_NUMBER_PASSWORD_HIDDEN:
+        param.type = SCE_IME_TYPE_NUMBER;
+        break;
+    case SDL_TEXTINPUT_TYPE_NUMBER_PASSWORD_VISIBLE:
+        param.type = SCE_IME_TYPE_NUMBER;
+        break;
+    }
+    param.option = 0;
+    if (SDL_GetTextInputCapitalization(props) != SDL_CAPITALIZE_SENTENCES) {
+        param.option |= SCE_IME_OPTION_NO_AUTO_CAPITALIZATION;
+    }
+    if (!SDL_GetTextInputAutocorrect(props)) {
+        param.option |= SCE_IME_OPTION_NO_ASSISTANCE;
+    }
+    if (SDL_GetTextInputMultiline(props)) {
+        param.option |= SCE_IME_OPTION_MULTILINE;
+    }
     param.inputTextBuffer = libime_out;
     param.maxTextLength = SCE_IME_MAX_TEXT_LENGTH;
     param.handler = VITA_ImeEventHandler;
@@ -525,13 +508,13 @@ void VITA_ShowScreenKeyboard(_THIS, SDL_Window *window)
 
 #endif
 
-    videodata->ime_active = SDL_TRUE;
+    videodata->ime_active = true;
 }
 
-void VITA_HideScreenKeyboard(_THIS, SDL_Window *window)
+void VITA_HideScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window)
 {
-#if !defined(SDL_VIDEO_VITA_PVR)
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
+#ifndef SDL_VIDEO_VITA_PVR
+    SDL_VideoData *videodata = _this->internal;
 
     SceCommonDialogStatus dialogStatus = sceImeDialogGetStatus();
 
@@ -545,14 +528,14 @@ void VITA_HideScreenKeyboard(_THIS, SDL_Window *window)
         break;
     }
 
-    videodata->ime_active = SDL_FALSE;
+    videodata->ime_active = false;
 #endif
 }
 
-SDL_bool VITA_IsScreenKeyboardShown(_THIS, SDL_Window *window)
+bool VITA_IsScreenKeyboardShown(SDL_VideoDevice *_this, SDL_Window *window)
 {
-#if defined(SDL_VIDEO_VITA_PVR)
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
+#ifdef SDL_VIDEO_VITA_PVR
+    SDL_VideoData *videodata = _this->internal;
     return videodata->ime_active;
 #else
     SceCommonDialogStatus dialogStatus = sceImeDialogGetStatus();
@@ -560,10 +543,10 @@ SDL_bool VITA_IsScreenKeyboardShown(_THIS, SDL_Window *window)
 #endif
 }
 
-void VITA_PumpEvents(_THIS)
+void VITA_PumpEvents(SDL_VideoDevice *_this)
 {
-#if !defined(SDL_VIDEO_VITA_PVR)
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
+#ifndef SDL_VIDEO_VITA_PVR
+    SDL_VideoData *videodata = _this->internal;
 #endif
 
     if (_this->suspend_screensaver) {
@@ -575,8 +558,8 @@ void VITA_PumpEvents(_THIS)
     VITA_PollKeyboard();
     VITA_PollMouse();
 
-#if !defined(SDL_VIDEO_VITA_PVR)
-    if (videodata->ime_active == SDL_TRUE) {
+#ifndef SDL_VIDEO_VITA_PVR
+    if (videodata->ime_active == true) {
         // update IME status. Terminate, if finished
         SceCommonDialogStatus dialogStatus = sceImeDialogGetStatus();
         if (dialogStatus == SCE_COMMON_DIALOG_STATUS_FINISHED) {
@@ -594,17 +577,15 @@ void VITA_PumpEvents(_THIS)
 
             // Send enter key only on enter
             if (result.button == SCE_IME_DIALOG_BUTTON_ENTER) {
-                SDL_SendKeyboardKeyAutoRelease(SDL_SCANCODE_RETURN);
+                SDL_SendKeyboardKeyAutoRelease(0, SDL_SCANCODE_RETURN);
             }
 
             sceImeDialogTerm();
 
-            videodata->ime_active = SDL_FALSE;
+            videodata->ime_active = false;
         }
     }
 #endif
 }
 
-#endif /* SDL_VIDEO_DRIVER_VITA */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_VITA
