@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_VIDEO_DRIVER_VIVANTE
 
@@ -26,26 +26,24 @@
 
 #ifdef VIVANTE_PLATFORM_GENERIC
 
-int VIVANTE_SetupPlatform(_THIS)
+bool VIVANTE_SetupPlatform(SDL_VideoDevice *_this)
 {
-    return 0;
+    return true;
 }
 
-char *VIVANTE_GetDisplayName(_THIS)
+char *VIVANTE_GetDisplayName(SDL_VideoDevice *_this)
 {
     return NULL;
 }
 
-void VIVANTE_UpdateDisplayScale(_THIS)
+void VIVANTE_UpdateDisplayScale(SDL_VideoDevice *_this)
 {
 }
 
-void VIVANTE_CleanupPlatform(_THIS)
+void VIVANTE_CleanupPlatform(SDL_VideoDevice *_this)
 {
 }
 
-#endif /* VIVANTE_PLATFORM_GENERIC */
+#endif // VIVANTE_PLATFORM_GENERIC
 
-#endif /* SDL_VIDEO_DRIVER_VIVANTE */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_VIVANTE
