@@ -18,35 +18,38 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_windowsmodes_h_
 #define SDL_windowsmodes_h_
 
-typedef struct
+typedef enum
+{
+    DisplayUnchanged,
+    DisplayAdded,
+    DisplayRemoved,
+
+} WIN_DisplayState;
+
+struct SDL_DisplayData
 {
     WCHAR DeviceName[32];
     HMONITOR MonitorHandle;
-    SDL_bool IsValid;
+    WIN_DisplayState state;
     SDL_Rect bounds;
-} SDL_DisplayData;
+};
 
-typedef struct
+struct SDL_DisplayModeData
 {
     DEVMODE DeviceMode;
-} SDL_DisplayModeData;
+};
 
-extern int WIN_InitModes(_THIS);
-extern int WIN_GetDisplayBounds(_THIS, SDL_VideoDisplay *display, SDL_Rect *rect);
-extern int WIN_GetDisplayUsableBounds(_THIS, SDL_VideoDisplay *display, SDL_Rect *rect);
-extern void WIN_ScreenPointFromSDL(int *x, int *y, int *dpiOut);
-extern void WIN_ScreenPointToSDL(int *x, int *y);
-extern int WIN_GetDisplayDPI(_THIS, SDL_VideoDisplay *display, float *ddpi, float *hdpi, float *vdpi);
-extern void WIN_GetDisplayModes(_THIS, SDL_VideoDisplay *display);
-extern int WIN_SetDisplayMode(_THIS, SDL_VideoDisplay *display, SDL_DisplayMode *mode);
-extern void WIN_RefreshDisplays(_THIS);
-extern void WIN_QuitModes(_THIS);
+extern bool WIN_InitModes(SDL_VideoDevice *_this);
+extern bool WIN_GetDisplayBounds(SDL_VideoDevice *_this, SDL_VideoDisplay *display, SDL_Rect *rect);
+extern bool WIN_GetDisplayUsableBounds(SDL_VideoDevice *_this, SDL_VideoDisplay *display, SDL_Rect *rect);
+extern bool WIN_GetDisplayModes(SDL_VideoDevice *_this, SDL_VideoDisplay *display);
+extern bool WIN_SetDisplayMode(SDL_VideoDevice *_this, SDL_VideoDisplay *display, SDL_DisplayMode *mode);
+extern void WIN_RefreshDisplays(SDL_VideoDevice *_this);
+extern void WIN_QuitModes(SDL_VideoDevice *_this);
 
-#endif /* SDL_windowsmodes_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_windowsmodes_h_
