@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2021 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,16 +18,17 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_windowsmouse_h_
 #define SDL_windowsmouse_h_
 
+extern DWORD SDL_last_warp_time;
 extern HCURSOR SDL_cursor;
 
-extern void WIN_InitMouse(_THIS);
-extern void WIN_QuitMouse(_THIS);
+extern void WIN_InitMouse(SDL_VideoDevice *_this);
+extern void WIN_QuitMouse(SDL_VideoDevice *_this);
+extern void WIN_SetCursorPos(int x, int y);
+extern void WIN_UpdateMouseSystemScale(void);
 
-#endif /* SDL_windowsmouse_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_windowsmouse_h_
