@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifndef SDL_windowsvideo_h_
 #define SDL_windowsvideo_h_
@@ -27,7 +27,13 @@
 
 #include "../SDL_sysvideo.h"
 
-#if defined(_MSC_VER) && (_MSC_VER >= 1500) && !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
+#ifdef HAVE_DXGI_H
+#define CINTERFACE
+#define COBJMACROS
+#include <dxgi.h>
+#endif
+
+#if defined(_MSC_VER) && (_MSC_VER >= 1500) && !defined(SDL_PLATFORM_XBOXONE) && !defined(SDL_PLATFORM_XBOXSERIES)
 #include <msctf.h>
 #else
 #include "SDL_msctf.h"
@@ -41,9 +47,11 @@
 
 #include "SDL_windowsclipboard.h"
 #include "SDL_windowsevents.h"
+#include "SDL_windowsgameinput.h"
 #include "SDL_windowsopengl.h"
 
-#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
+#if !defined(SDL_PLATFORM_XBOXONE) && !defined(SDL_PLATFORM_XBOXSERIES)
+#include "SDL_windowsshape.h"
 #include "SDL_windowskeyboard.h"
 #include "SDL_windowsmodes.h"
 #include "SDL_windowsmouse.h"
@@ -51,11 +59,13 @@
 #endif
 
 #include "SDL_windowswindow.h"
-#include "SDL_events.h"
-#include "SDL_loadso.h"
+
+#ifndef USER_DEFAULT_SCREEN_DPI
+#define USER_DEFAULT_SCREEN_DPI 96
+#endif
 
 #if WINVER < 0x0601
-/* Touch input definitions */
+// Touch input definitions
 #define TWF_FINETOUCH 1
 #define TWF_WANTPALM  2
 
@@ -79,8 +89,8 @@ typedef struct _TOUCHINPUT
     DWORD cyContact;
 } TOUCHINPUT, *PTOUCHINPUT;
 
-/* More-robust display information in Vista... */
-/* This is a huge amount of data to be stuffing into three API calls. :( */
+// More-robust display information in Vista...
+// This is a huge amount of data to be stuffing into three API calls. :(
 typedef struct DISPLAYCONFIG_PATH_SOURCE_INFO
 {
     LUID adapterId;
@@ -275,7 +285,7 @@ typedef struct DISPLAYCONFIG_TARGET_DEVICE_NAME
 
 #define QDC_ONLY_ACTIVE_PATHS 0x00000002
 
-#endif /* WINVER < 0x0601 */
+#endif // WINVER < 0x0601
 
 #ifndef HAVE_SHELLSCALINGAPI_H
 
@@ -314,17 +324,17 @@ DECLARE_HANDLE(DPI_AWARENESS_CONTEXT);
 #define DPI_AWARENESS_CONTEXT_SYSTEM_AWARE      ((DPI_AWARENESS_CONTEXT)-2)
 #define DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE ((DPI_AWARENESS_CONTEXT)-3)
 
-#endif /* _DPI_AWARENESS_CONTEXTS_ */
+#endif // _DPI_AWARENESS_CONTEXTS_
 
-/* Windows 10 Creators Update */
+// Windows 10 Creators Update
 #if NTDDI_VERSION < 0x0A000003
 #define DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2 ((DPI_AWARENESS_CONTEXT)-4)
-#endif /* NTDDI_VERSION < 0x0A000003 */
+#endif // NTDDI_VERSION < 0x0A000003
 
-/* Windows 10 version 1809 */
+// Windows 10 version 1809
 #if NTDDI_VERSION < 0x0A000006
 #define DPI_AWARENESS_CONTEXT_UNAWARE_GDISCALED ((DPI_AWARENESS_CONTEXT)-5)
-#endif /* NTDDI_VERSION < 0x0A000006 */
+#endif // NTDDI_VERSION < 0x0A000006
 
 typedef BOOL (*PFNSHFullScreen)(HWND, DWORD);
 typedef void (*PFCoordTransform)(SDL_Window *, POINT *);
@@ -337,7 +347,7 @@ typedef struct
 } TSFSink;
 
 #ifndef SDL_DISABLE_WINDOWS_IME
-/* Definition from Win98DDK version of IMM.H */
+// Definition from Win98DDK version of IMM.H
 typedef struct tagINPUTCONTEXT2
 {
     HWND hWnd;
@@ -362,20 +372,25 @@ typedef struct tagINPUTCONTEXT2
     DWORD fdwInit;
     DWORD dwReserve[3];
 } INPUTCONTEXT2, *PINPUTCONTEXT2, NEAR *NPINPUTCONTEXT2, FAR *LPINPUTCONTEXT2;
-#endif /* !SDL_DISABLE_WINDOWS_IME */
+#endif
 
-/* Private display data */
+// Private display data
 
-typedef struct SDL_VideoData
+struct SDL_VideoData
 {
     int render;
 
+    bool coinitialized;
+#if !(defined(SDL_PLATFORM_XBOXONE) || defined(SDL_PLATFORM_XBOXSERIES))
+    bool oleinitialized;
+#endif // !(defined(SDL_PLATFORM_XBOXONE) || defined(SDL_PLATFORM_XBOXSERIES))
+
     DWORD clipboard_count;
 
-#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__) /* Xbox doesn't support user32/shcore*/
-    /* Touch input functions */
-    void *userDLL;
-    /* *INDENT-OFF* */ /* clang-format off */
+#if !defined(SDL_PLATFORM_XBOXONE) && !defined(SDL_PLATFORM_XBOXSERIES) // Xbox doesn't support user32/shcore
+    // Touch input functions
+    SDL_SharedObject *userDLL;
+    /* *INDENT-OFF* */ // clang-format off
     BOOL (WINAPI *CloseTouchInputHandle)( HTOUCHINPUT );
     BOOL (WINAPI *GetTouchInputInfo)( HTOUCHINPUT, UINT, PTOUCHINPUT, int );
     BOOL (WINAPI *RegisterTouchWindow)( HWND, ULONG );
@@ -389,87 +404,101 @@ typedef struct SDL_VideoData
     UINT (WINAPI *GetDpiForWindow)( HWND );
     BOOL (WINAPI *AreDpiAwarenessContextsEqual)(DPI_AWARENESS_CONTEXT, DPI_AWARENESS_CONTEXT);
     BOOL (WINAPI *IsValidDpiAwarenessContext)(DPI_AWARENESS_CONTEXT);
-    /* *INDENT-ON* */ /* clang-format on */
+    // DisplayConfig functions
+    LONG (WINAPI *GetDisplayConfigBufferSizes)( UINT32, UINT32*, UINT32* );
+    LONG (WINAPI *QueryDisplayConfig)( UINT32, UINT32*, DISPLAYCONFIG_PATH_INFO*, UINT32*, DISPLAYCONFIG_MODE_INFO*, DISPLAYCONFIG_TOPOLOGY_ID*);
+    LONG (WINAPI *DisplayConfigGetDeviceInfo)( DISPLAYCONFIG_DEVICE_INFO_HEADER*);
+    /* *INDENT-ON* */ // clang-format on
 
-    void *shcoreDLL;
-    /* *INDENT-OFF* */ /* clang-format off */
+    SDL_SharedObject *shcoreDLL;
+    /* *INDENT-OFF* */ // clang-format off
     HRESULT (WINAPI *GetDpiForMonitor)( HMONITOR         hmonitor,
                                         MONITOR_DPI_TYPE dpiType,
                                         UINT             *dpiX,
                                         UINT             *dpiY );
     HRESULT (WINAPI *SetProcessDpiAwareness)(PROCESS_DPI_AWARENESS dpiAwareness);
-    /* *INDENT-ON* */ /* clang-format on */
-#endif                /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
+    /* *INDENT-ON* */ // clang-format on
+#endif                // !defined(SDL_PLATFORM_XBOXONE) && !defined(SDL_PLATFORM_XBOXSERIES)
 
-    SDL_bool dpi_scaling_enabled;
-    SDL_bool cleared;
+#ifdef HAVE_DXGI_H
+    SDL_SharedObject *dxgiDLL;
+    IDXGIFactory *pDXGIFactory;
+#endif
+
+    bool cleared;
+
+    BYTE *rawinput;
+    UINT rawinput_offset;
+    UINT rawinput_size;
+    UINT rawinput_count;
+    Uint64 last_rawinput_poll;
+    SDL_Point last_raw_mouse_position;
+    bool raw_mouse_enabled;
+    bool raw_keyboard_enabled;
+    bool pending_E1_key_sequence;
+    Uint32 raw_input_enabled;
+
+    WIN_GameInputData *gameinput_context;
 
 #ifndef SDL_DISABLE_WINDOWS_IME
-    SDL_bool ime_com_initialized;
-    struct ITfThreadMgr *ime_threadmgr;
-    SDL_bool ime_initialized;
-    SDL_bool ime_enabled;
-    SDL_bool ime_available;
+    bool ime_initialized;
+    bool ime_enabled;
+    bool ime_available;
+    bool ime_internal_composition;
+    bool ime_internal_candidates;
     HWND ime_hwnd_main;
     HWND ime_hwnd_current;
-    SDL_bool ime_suppress_endcomposition_event;
+    bool ime_needs_clear_composition;
     HIMC ime_himc;
 
     WCHAR *ime_composition;
     int ime_composition_length;
     WCHAR ime_readingstring[16];
     int ime_cursor;
+    int ime_selected_start;
+    int ime_selected_length;
 
-    SDL_bool ime_candlist;
-    WCHAR *ime_candidates;
-    DWORD ime_candcount;
+    bool ime_candidates_open;
+    bool ime_update_candidates;
+    char *ime_candidates[MAX_CANDLIST];
+    int ime_candcount;
     DWORD ime_candref;
     DWORD ime_candsel;
-    UINT ime_candpgsize;
     int ime_candlistindexbase;
-    SDL_bool ime_candvertical;
+    bool ime_horizontal_candidates;
+#endif
 
-    SDL_bool ime_dirty;
-    SDL_Rect ime_rect;
-    SDL_Rect ime_candlistrect;
-    int ime_winwidth;
-    int ime_winheight;
+#if !defined(SDL_PLATFORM_XBOXONE) && !defined(SDL_PLATFORM_XBOXSERIES)
+    COMPOSITIONFORM ime_composition_area;
+    CANDIDATEFORM ime_candidate_area;
+#endif // !defined(SDL_PLATFORM_XBOXONE) && !defined(SDL_PLATFORM_XBOXSERIES)
 
+#ifndef SDL_DISABLE_WINDOWS_IME
     HKL ime_hkl;
-    void *ime_himm32;
-    /* *INDENT-OFF* */ /* clang-format off */
+    SDL_SharedObject *ime_himm32;
+    /* *INDENT-OFF* */ // clang-format off
     UINT (WINAPI *GetReadingString)(HIMC himc, UINT uReadingBufLen, LPWSTR lpwReadingBuf, PINT pnErrorIndex, BOOL *pfIsVertical, PUINT puMaxReadingLen);
     BOOL (WINAPI *ShowReadingWindow)(HIMC himc, BOOL bShow);
     LPINPUTCONTEXT2 (WINAPI *ImmLockIMC)(HIMC himc);
     BOOL (WINAPI *ImmUnlockIMC)(HIMC himc);
     LPVOID (WINAPI *ImmLockIMCC)(HIMCC himcc);
     BOOL (WINAPI *ImmUnlockIMCC)(HIMCC himcc);
-    /* *INDENT-ON* */ /* clang-format on */
+    /* *INDENT-ON* */ // clang-format on
 
-    SDL_bool ime_uiless;
-    struct ITfThreadMgrEx *ime_threadmgrex;
-    DWORD ime_uielemsinkcookie;
-    DWORD ime_alpnsinkcookie;
-    DWORD ime_openmodesinkcookie;
-    DWORD ime_convmodesinkcookie;
-    TSFSink *ime_uielemsink;
-    TSFSink *ime_ippasink;
-    LONG ime_uicontext;
-#endif /* !SDL_DISABLE_WINDOWS_IME */
+#endif // !SDL_DISABLE_WINDOWS_IME
 
     BYTE pre_hook_key_state[256];
     UINT _SDL_WAKEUP;
-} SDL_VideoData;
+};
 
-extern SDL_bool g_WindowsEnableMessageLoop;
-extern SDL_bool g_WindowsEnableMenuMnemonics;
-extern SDL_bool g_WindowFrameUsableWhileCursorHidden;
+extern bool g_WindowsEnableMessageLoop;
+extern bool g_WindowsEnableMenuMnemonics;
+extern bool g_WindowFrameUsableWhileCursorHidden;
 
 typedef struct IDirect3D9 IDirect3D9;
-extern SDL_bool D3D_LoadDLL(void **pD3DDLL, IDirect3D9 **pDirect3D9Interface);
+extern bool D3D_LoadDLL(void **pD3DDLL, IDirect3D9 **pDirect3D9Interface);
 
-extern SDL_bool WIN_IsPerMonitorV2DPIAware(_THIS);
+extern SDL_SystemTheme WIN_GetSystemTheme(void);
+extern bool WIN_IsPerMonitorV2DPIAware(SDL_VideoDevice *_this);
 
-#endif /* SDL_windowsvideo_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_windowsvideo_h_
