@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -22,15 +22,13 @@
 #ifndef SDL_x11framebuffer_h_
 #define SDL_x11framebuffer_h_
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-extern int X11_CreateWindowFramebuffer(_THIS, SDL_Window *window,
-                                       Uint32 *format,
-                                       void **pixels, int *pitch);
-extern int X11_UpdateWindowFramebuffer(_THIS, SDL_Window *window,
-                                       const SDL_Rect *rects, int numrects);
-extern void X11_DestroyWindowFramebuffer(_THIS, SDL_Window *window);
+extern bool X11_CreateWindowFramebuffer(SDL_VideoDevice *_this, SDL_Window *window,
+                                        SDL_PixelFormat *format,
+                                        void **pixels, int *pitch);
+extern bool X11_UpdateWindowFramebuffer(SDL_VideoDevice *_this, SDL_Window *window,
+                                        const SDL_Rect *rects, int numrects);
+extern void X11_DestroyWindowFramebuffer(SDL_VideoDevice *_this, SDL_Window *window);
 
-#endif /* SDL_x11framebuffer_h_ */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_x11framebuffer_h_
