@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,12 +18,10 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_DRIVER_X11
+#ifdef SDL_VIDEO_DRIVER_X11
 
-#include "SDL_hints.h"
-#include "SDL_misc.h"
 #include "SDL_x11video.h"
 
 #include "../../events/SDL_keyboard_c.h"
@@ -46,7 +44,7 @@ static SDL_ScancodeTable scancode_set[] = {
     SDL_SCANCODE_TABLE_XVNC,
 };
 
-static SDL_bool X11_ScancodeIsRemappable(SDL_Scancode scancode)
+static bool X11_ScancodeIsRemappable(SDL_Scancode scancode)
 {
     /*
      * XKB remappings can assign different keysyms for these scancodes, but
@@ -65,16 +63,16 @@ static SDL_bool X11_ScancodeIsRemappable(SDL_Scancode scancode)
     case SDL_SCANCODE_RALT:
     case SDL_SCANCODE_LGUI:
     case SDL_SCANCODE_RGUI:
-        return SDL_FALSE;
+        return false;
     default:
-        return SDL_TRUE;
+        return true;
     }
 }
 
-/* This function only correctly maps letters and numbers for keyboards in US QWERTY layout */
-static SDL_Scancode X11_KeyCodeToSDLScancode(_THIS, KeyCode keycode)
+// This function only correctly maps letters and numbers for keyboards in US QWERTY layout
+static SDL_Scancode X11_KeyCodeToSDLScancode(SDL_VideoDevice *_this, KeyCode keycode)
 {
-    const KeySym keysym = X11_KeyCodeToSym(_this, keycode, 0);
+    const KeySym keysym = X11_KeyCodeToSym(_this, keycode, 0, 0);
 
     if (keysym == NoSymbol) {
         return SDL_SCANCODE_UNKNOWN;
@@ -83,24 +81,15 @@ static SDL_Scancode X11_KeyCodeToSDLScancode(_THIS, KeyCode keycode)
     return SDL_GetScancodeFromKeySym(keysym, keycode);
 }
 
-static Uint32 X11_KeyCodeToUcs4(_THIS, KeyCode keycode, unsigned char group)
+KeySym X11_KeyCodeToSym(SDL_VideoDevice *_this, KeyCode keycode, unsigned char group, unsigned int mod_mask)
 {
-    KeySym keysym = X11_KeyCodeToSym(_this, keycode, group);
-
-    if (keysym == NoSymbol) {
-        return 0;
-    }
-
-    return SDL_KeySymToUcs4(keysym);
-}
-
-KeySym
-X11_KeyCodeToSym(_THIS, KeyCode keycode, unsigned char group)
-{
-    SDL_VideoData *data = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *data = _this->internal;
     KeySym keysym;
+    unsigned int mods_ret[16];
 
-#if SDL_VIDEO_DRIVER_X11_HAS_XKBKEYCODETOKEYSYM
+    SDL_zero(mods_ret);
+
+#ifdef SDL_VIDEO_DRIVER_X11_HAS_XKBLOOKUPKEYSYM
     if (data->xkb) {
         int num_groups = XkbKeyNumGroups(data->xkb, keycode);
         unsigned char info = XkbKeyGroupInfo(data->xkb, keycode);
@@ -120,20 +109,23 @@ X11_KeyCodeToSym(_THIS, KeyCode keycode, unsigned char group)
                 group %= num_groups;
             }
         }
-        keysym = X11_XkbKeycodeToKeysym(data->display, keycode, group, 0);
-    } else {
+
+        if (X11_XkbLookupKeySym(data->display, keycode, XkbBuildCoreState(mod_mask, group), mods_ret, &keysym) == NoSymbol) {
+            keysym = NoSymbol;
+        }
+    } else
+#endif
+    {
+        // TODO: Handle groups and modifiers on the legacy path.
         keysym = X11_XKeycodeToKeysym(data->display, keycode, 0);
     }
-#else
-    keysym = X11_XKeycodeToKeysym(data->display, keycode, 0);
-#endif
 
     return keysym;
 }
 
-int X11_InitKeyboard(_THIS)
+bool X11_InitKeyboard(SDL_VideoDevice *_this)
 {
-    SDL_VideoData *data = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *data = _this->internal;
     int i = 0;
     int j = 0;
     int min_keycode, max_keycode;
@@ -155,7 +147,7 @@ int X11_InitKeyboard(_THIS)
     int distance;
     Bool xkb_repeat = 0;
 
-#if SDL_VIDEO_DRIVER_X11_HAS_XKBKEYCODETOKEYSYM
+#ifdef SDL_VIDEO_DRIVER_X11_HAS_XKBLOOKUPKEYSYM
     {
         int xkb_major = XkbMajorVersion;
         int xkb_minor = XkbMinorVersion;
@@ -164,12 +156,12 @@ int X11_InitKeyboard(_THIS)
             data->xkb = X11_XkbGetMap(data->display, XkbAllClientInfoMask, XkbUseCoreKbd);
         }
 
-        /* This will remove KeyRelease events for held keys */
+        // This will remove KeyRelease events for held keys
         X11_XkbSetDetectableAutoRepeat(data->display, True, &xkb_repeat);
     }
 #endif
 
-    /* Open a connection to the X input manager */
+    // Open a connection to the X input manager
 #ifdef X_HAVE_UTF8_STRING
     if (SDL_X11_HAVE_UTF8) {
         /* Set the locale, and call XSetLocaleModifiers before XOpenIM so that
@@ -178,7 +170,7 @@ int X11_InitKeyboard(_THIS)
         char *prev_xmods = X11_XSetLocaleModifiers(NULL);
         const char *new_xmods = "";
         const char *env_xmods = SDL_getenv("XMODIFIERS");
-        SDL_bool has_dbus_ime_support = SDL_FALSE;
+        bool has_dbus_ime_support = false;
 
         if (prev_locale) {
             prev_locale = SDL_strdup(prev_locale);
@@ -193,10 +185,10 @@ int X11_InitKeyboard(_THIS)
            @im=none if XMODIFIERS contains @im=ibus. IBus can still be used via
            the DBus implementation, which also has support for pre-editing. */
         if (env_xmods && SDL_strstr(env_xmods, "@im=ibus") != NULL) {
-            has_dbus_ime_support = SDL_TRUE;
+            has_dbus_ime_support = true;
         }
         if (env_xmods && SDL_strstr(env_xmods, "@im=fcitx") != NULL) {
-            has_dbus_ime_support = SDL_TRUE;
+            has_dbus_ime_support = true;
         }
         if (has_dbus_ime_support || !xkb_repeat) {
             new_xmods = "@im=none";
@@ -205,7 +197,7 @@ int X11_InitKeyboard(_THIS)
         (void)setlocale(LC_ALL, "");
         X11_XSetLocaleModifiers(new_xmods);
 
-        data->im = X11_XOpenIM(data->display, NULL, data->classname, data->classname);
+        data->im = X11_XOpenIM(data->display, NULL, NULL, NULL);
 
         /* Reset the locale + X locale modifiers back to how they were,
            locale first because the X locale modifiers depend on it. */
@@ -221,7 +213,7 @@ int X11_InitKeyboard(_THIS)
         }
     }
 #endif
-    /* Try to determine which scancodes are being used based on fingerprint */
+    // Try to determine which scancodes are being used based on fingerprint
     best_distance = SDL_arraysize(fingerprint) + 1;
     best_index = -1;
     X11_XDisplayKeycodes(data->display, &min_keycode, &max_keycode);
@@ -246,7 +238,7 @@ int X11_InitKeyboard(_THIS)
         }
     }
     if (best_index < 0 || best_distance > 2) {
-        /* This is likely to be SDL_SCANCODE_TABLE_XFREE86_2 with remapped keys, double check a rarely remapped value */
+        // This is likely to be SDL_SCANCODE_TABLE_XFREE86_2 with remapped keys, double check a rarely remapped value
         int fingerprint_value = X11_XKeysymToKeycode(data->display, 0x1008FF5B /* XF86Documents */) - min_keycode;
         if (fingerprint_value == 235) {
             for (i = 0; i < SDL_arraysize(scancode_set); ++i) {
@@ -259,14 +251,13 @@ int X11_InitKeyboard(_THIS)
         }
     }
     if (best_index >= 0 && best_distance <= 2) {
-        SDL_Keycode default_keymap[SDL_NUM_SCANCODES];
         int table_size;
         const SDL_Scancode *table = SDL_GetScancodeTable(scancode_set[best_index], &table_size);
 
 #ifdef DEBUG_KEYBOARD
         SDL_Log("Using scancode set %d, min_keycode = %d, max_keycode = %d, table_size = %d\n", best_index, min_keycode, max_keycode, table_size);
 #endif
-        /* This should never happen, but just in case... */
+        // This should never happen, but just in case...
         if (table_size > (SDL_arraysize(data->key_layout) - min_keycode)) {
             table_size = (SDL_arraysize(data->key_layout) - min_keycode);
         }
@@ -276,8 +267,6 @@ int X11_InitKeyboard(_THIS)
            However, there are a number of extended scancodes that have no standard location, so use
            the X11 mapping for all non-character keys.
          */
-        SDL_GetDefaultKeymap(default_keymap);
-
         for (i = min_keycode; i <= max_keycode; ++i) {
             SDL_Scancode scancode = X11_KeyCodeToSDLScancode(_this, i);
 #ifdef DEBUG_KEYBOARD
@@ -291,8 +280,8 @@ int X11_InitKeyboard(_THIS)
             if (scancode == data->key_layout[i]) {
                 continue;
             }
-            if (default_keymap[scancode] >= SDLK_SCANCODE_MASK && X11_ScancodeIsRemappable(scancode)) {
-                /* Not a character key and the scancode is safe to remap */
+            if ((SDL_GetKeymapKeycode(NULL, scancode, SDL_KMOD_NONE) & SDLK_SCANCODE_MASK) && X11_ScancodeIsRemappable(scancode)) {
+                // Not a character key and the scancode is safe to remap
 #ifdef DEBUG_KEYBOARD
                 SDL_Log("Changing scancode, was %d (%s), now %d (%s)\n", data->key_layout[i], SDL_GetScancodeName(data->key_layout[i]), scancode, SDL_GetScancodeName(scancode));
 #endif
@@ -304,7 +293,7 @@ int X11_InitKeyboard(_THIS)
         SDL_Log("Keyboard layout unknown, please report the following to the SDL forums/mailing list (https://discourse.libsdl.org/):\n");
 #endif
 
-        /* Determine key_layout - only works on US QWERTY layout */
+        // Determine key_layout - only works on US QWERTY layout
         for (i = min_keycode; i <= max_keycode; ++i) {
             SDL_Scancode scancode = X11_KeyCodeToSDLScancode(_this, i);
 #ifdef DEBUG_SCANCODES
@@ -324,7 +313,7 @@ int X11_InitKeyboard(_THIS)
         }
     }
 
-    X11_UpdateKeymap(_this, SDL_FALSE);
+    X11_UpdateKeymap(_this, false);
 
     SDL_SetScancodeName(SDL_SCANCODE_APPLICATION, "Menu");
 
@@ -334,76 +323,78 @@ int X11_InitKeyboard(_THIS)
 
     X11_ReconcileKeyboardState(_this);
 
-    return 0;
+    return true;
 }
 
-void X11_UpdateKeymap(_THIS, SDL_bool send_event)
+void X11_UpdateKeymap(SDL_VideoDevice *_this, bool send_event)
 {
-    SDL_VideoData *data = (SDL_VideoData *)_this->driverdata;
+    struct Keymod_masks
+    {
+        SDL_Keymod sdl_mask;
+        unsigned int xkb_mask;
+    } const keymod_masks[] = {
+        { SDL_KMOD_NONE, 0 },
+        { SDL_KMOD_SHIFT, ShiftMask },
+        { SDL_KMOD_CAPS, LockMask },
+        { SDL_KMOD_SHIFT | SDL_KMOD_CAPS, ShiftMask | LockMask },
+        { SDL_KMOD_MODE, Mod5Mask },
+        { SDL_KMOD_MODE | SDL_KMOD_SHIFT, Mod5Mask | ShiftMask },
+        { SDL_KMOD_MODE | SDL_KMOD_CAPS, Mod5Mask | LockMask },
+        { SDL_KMOD_MODE | SDL_KMOD_SHIFT | SDL_KMOD_CAPS, Mod5Mask | ShiftMask | LockMask }
+    };
+
+    SDL_VideoData *data = _this->internal;
     int i;
     SDL_Scancode scancode;
-    SDL_Keycode keymap[SDL_NUM_SCANCODES];
-    unsigned char group = 0;
+    SDL_Keymap *keymap;
 
-    SDL_GetDefaultKeymap(keymap);
+    keymap = SDL_CreateKeymap();
 
-#if SDL_VIDEO_DRIVER_X11_HAS_XKBKEYCODETOKEYSYM
+#ifdef SDL_VIDEO_DRIVER_X11_HAS_XKBLOOKUPKEYSYM
     if (data->xkb) {
         XkbStateRec state;
         X11_XkbGetUpdatedMap(data->display, XkbAllClientInfoMask, data->xkb);
 
         if (X11_XkbGetState(data->display, XkbUseCoreKbd, &state) == Success) {
-            group = state.group;
+            data->xkb_group = state.group;
         }
     }
 #endif
 
-    for (i = 0; i < SDL_arraysize(data->key_layout); i++) {
-        Uint32 key;
+    for (int m = 0; m < SDL_arraysize(keymod_masks); ++m) {
+        for (i = 0; i < SDL_arraysize(data->key_layout); i++) {
+            SDL_Keycode keycode;
 
-        /* Make sure this is a valid scancode */
-        scancode = data->key_layout[i];
-        if (scancode == SDL_SCANCODE_UNKNOWN) {
-            continue;
-        }
-
-        /* See if there is a UCS keycode for this scancode */
-        key = X11_KeyCodeToUcs4(_this, (KeyCode)i, group);
-        if (key) {
-            keymap[scancode] = key;
-        } else {
-            SDL_Scancode keyScancode = X11_KeyCodeToSDLScancode(_this, (KeyCode)i);
-
-            switch (keyScancode) {
-            case SDL_SCANCODE_RETURN:
-                keymap[scancode] = SDLK_RETURN;
-                break;
-            case SDL_SCANCODE_ESCAPE:
-                keymap[scancode] = SDLK_ESCAPE;
-                break;
-            case SDL_SCANCODE_BACKSPACE:
-                keymap[scancode] = SDLK_BACKSPACE;
-                break;
-            case SDL_SCANCODE_TAB:
-                keymap[scancode] = SDLK_TAB;
-                break;
-            case SDL_SCANCODE_DELETE:
-                keymap[scancode] = SDLK_DELETE;
-                break;
-            default:
-                keymap[scancode] = SDL_SCANCODE_TO_KEYCODE(keyScancode);
-                break;
+            // Make sure this is a valid scancode
+            scancode = data->key_layout[i];
+            if (scancode == SDL_SCANCODE_UNKNOWN) {
+                continue;
             }
+
+            KeySym keysym = X11_KeyCodeToSym(_this, i, data->xkb_group, keymod_masks[m].xkb_mask);
+
+            // Note: The default SDL scancode table sets this to right alt instead of AltGr/Mode, so handle it separately.
+            if (keysym != XK_ISO_Level3_Shift) {
+                keycode = SDL_KeySymToUcs4(keysym);
+            } else {
+                keycode = SDLK_MODE;
+            }
+
+            if (!keycode) {
+                SDL_Scancode keyScancode = SDL_GetScancodeFromKeySym(keysym, (KeyCode)i);
+                keycode = SDL_GetKeymapKeycode(NULL, keyScancode, keymod_masks[m].sdl_mask);
+            }
+            SDL_SetKeymapEntry(keymap, scancode, keymod_masks[m].sdl_mask, keycode);
         }
     }
-    SDL_SetKeymap(0, keymap, SDL_NUM_SCANCODES, send_event);
+    SDL_SetKeymap(keymap, send_event);
 }
 
-void X11_QuitKeyboard(_THIS)
+void X11_QuitKeyboard(SDL_VideoDevice *_this)
 {
-    SDL_VideoData *data = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *data = _this->internal;
 
-#if SDL_VIDEO_DRIVER_X11_HAS_XKBKEYCODETOKEYSYM
+#ifdef SDL_VIDEO_DRIVER_X11_HAS_XKBLOOKUPKEYSYM
     if (data->xkb) {
         X11_XkbFreeKeyboard(data->xkb, 0, True);
         data->xkb = NULL;
@@ -415,92 +406,106 @@ void X11_QuitKeyboard(_THIS)
 #endif
 }
 
-static void X11_ResetXIM(_THIS)
+static void X11_ResetXIM(SDL_VideoDevice *_this, SDL_Window *window)
 {
 #ifdef X_HAVE_UTF8_STRING
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
-    int i;
+    SDL_WindowData *data = window->internal;
 
-    if (videodata && videodata->windowlist) {
-        for (i = 0; i < videodata->numwindows; ++i) {
-            SDL_WindowData *data = videodata->windowlist[i];
-            if (data && data->ic) {
-                /* Clear any partially entered dead keys */
-                char *contents = X11_Xutf8ResetIC(data->ic);
-                if (contents) {
-                    X11_XFree(contents);
-                }
-            }
+    if (data && data->ic) {
+        // Clear any partially entered dead keys
+        char *contents = X11_Xutf8ResetIC(data->ic);
+        if (contents) {
+            X11_XFree(contents);
         }
     }
 #endif
 }
 
-void X11_StartTextInput(_THIS)
+bool X11_StartTextInput(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID props)
 {
-    X11_ResetXIM(_this);
+    X11_ResetXIM(_this, window);
+
+    return X11_UpdateTextInputArea(_this, window);
 }
 
-void X11_StopTextInput(_THIS)
+bool X11_StopTextInput(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    X11_ResetXIM(_this);
+    X11_ResetXIM(_this, window);
 #ifdef SDL_USE_IME
     SDL_IME_Reset();
 #endif
+    return true;
 }
 
-void X11_SetTextInputRect(_THIS, const SDL_Rect *rect)
+bool X11_UpdateTextInputArea(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    if (rect == NULL) {
-        SDL_InvalidParamError("rect");
-        return;
-    }
-
 #ifdef SDL_USE_IME
-    SDL_IME_UpdateTextRect(rect);
+    SDL_IME_UpdateTextInputArea(window);
 #endif
+    return true;
 }
 
-SDL_bool X11_HasScreenKeyboardSupport(_THIS)
+bool X11_HasScreenKeyboardSupport(SDL_VideoDevice *_this)
 {
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *videodata = _this->internal;
     return videodata->is_steam_deck;
 }
 
-void X11_ShowScreenKeyboard(_THIS, SDL_Window *window)
+void X11_ShowScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window, SDL_PropertiesID props)
 {
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *videodata = _this->internal;
 
     if (videodata->is_steam_deck) {
         /* For more documentation of the URL parameters, see:
          * https://partner.steamgames.com/doc/api/ISteamUtils#ShowFloatingGamepadTextInput
          */
+        const int k_EFloatingGamepadTextInputModeModeSingleLine = 0;    // Enter dismisses the keyboard
+        const int k_EFloatingGamepadTextInputModeModeMultipleLines = 1; // User needs to explicitly dismiss the keyboard
+        const int k_EFloatingGamepadTextInputModeModeEmail = 2;         // Keyboard is displayed in a special mode that makes it easier to enter emails
+        const int k_EFloatingGamepadTextInputModeModeNumeric = 3;       // Numeric keypad is shown
         char deeplink[128];
+        int mode;
+
+        switch (SDL_GetTextInputType(props)) {
+        case SDL_TEXTINPUT_TYPE_TEXT_EMAIL:
+            mode = k_EFloatingGamepadTextInputModeModeEmail;
+            break;
+        case SDL_TEXTINPUT_TYPE_NUMBER:
+        case SDL_TEXTINPUT_TYPE_NUMBER_PASSWORD_HIDDEN:
+        case SDL_TEXTINPUT_TYPE_NUMBER_PASSWORD_VISIBLE:
+            mode = k_EFloatingGamepadTextInputModeModeNumeric;
+            break;
+        default:
+            if (SDL_GetTextInputMultiline(props)) {
+                mode = k_EFloatingGamepadTextInputModeModeMultipleLines;
+            } else {
+                mode = k_EFloatingGamepadTextInputModeModeSingleLine;
+            }
+            break;
+        }
         (void)SDL_snprintf(deeplink, sizeof(deeplink),
                            "steam://open/keyboard?XPosition=0&YPosition=0&Width=0&Height=0&Mode=%d",
-                           SDL_GetHintBoolean(SDL_HINT_RETURN_KEY_HIDES_IME, SDL_FALSE) ? 0 : 1);
+                           mode);
         SDL_OpenURL(deeplink);
-        videodata->steam_keyboard_open = SDL_TRUE;
+        videodata->steam_keyboard_open = true;
     }
 }
 
-void X11_HideScreenKeyboard(_THIS, SDL_Window *window)
+void X11_HideScreenKeyboard(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *videodata = _this->internal;
 
     if (videodata->is_steam_deck) {
         SDL_OpenURL("steam://close/keyboard");
-        videodata->steam_keyboard_open = SDL_FALSE;
+        videodata->steam_keyboard_open = false;
     }
 }
 
-SDL_bool X11_IsScreenKeyboardShown(_THIS, SDL_Window *window)
+bool X11_IsScreenKeyboardShown(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    SDL_VideoData *videodata = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *videodata = _this->internal;
 
     return videodata->steam_keyboard_open;
 }
 
-#endif /* SDL_VIDEO_DRIVER_X11 */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_X11
