@@ -19,30 +19,26 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_VIDEO_DRIVER_X11
 
 #include "SDL_x11video.h"
-#include "SDL_hints.h"
 
-/* GLX implementation of SDL OpenGL support */
+// GLX implementation of SDL OpenGL support
 
 #ifdef SDL_VIDEO_OPENGL_GLX
-#include "SDL_loadso.h"
 #include "SDL_x11opengles.h"
 
-#if defined(__IRIX__) || defined(__NetBSD__) || defined(__OpenBSD__)
+#if defined(SDL_PLATFORM_IRIX) || defined(SDL_PLATFORM_NETBSD) || defined(SDL_PLATFORM_OPENBSD)
 /*
  * IRIX doesn't have a GL library versioning system.
  * NetBSD and OpenBSD have different GL library versions depending on how
  * the library was installed.
  */
-#define DEFAULT_OPENGL  "libGL.so"
-#elif defined(__MACOSX__)
-#define DEFAULT_OPENGL  "/opt/X11/lib/libGL.1.dylib"
-#elif defined(__QNXNTO__)
-#define DEFAULT_OPENGL  "libGL.so.3"
+#define DEFAULT_OPENGL "libGL.so"
+#elif defined(SDL_PLATFORM_MACOS)
+#define DEFAULT_OPENGL "/opt/X11/lib/libGL.1.dylib"
 #else
 #define DEFAULT_OPENGL "libGL.so.1"
 #endif
@@ -79,7 +75,7 @@
 #define GLX_CONTEXT_DEBUG_BIT_ARB              0x0001
 #define GLX_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB 0x0002
 
-/* Typedef for the GL 3.0 context creation function */
+// Typedef for the GL 3.0 context creation function
 typedef GLXContext (*PFNGLXCREATECONTEXTATTRIBSARBPROC)(Display *dpy,
                                                         GLXFBConfig config,
                                                         GLXContext
@@ -163,20 +159,20 @@ typedef GLXContext (*PFNGLXCREATECONTEXTATTRIBSARBPROC)(Display *dpy,
 #define GL_UnloadObject SDL_UnloadObject
 #endif
 
-static void X11_GL_InitExtensions(_THIS);
+static void X11_GL_InitExtensions(SDL_VideoDevice *_this);
 
-int X11_GL_LoadLibrary(_THIS, const char *path)
+bool X11_GL_LoadLibrary(SDL_VideoDevice *_this, const char *path)
 {
     Display *display;
-    void *handle;
+    SDL_SharedObject *handle;
 
     if (_this->gl_data) {
         return SDL_SetError("OpenGL context already created");
     }
 
-    /* Load the OpenGL library */
+    // Load the OpenGL library
     if (path == NULL) {
-        path = SDL_getenv("SDL_OPENGL_LIBRARY");
+        path = SDL_GetHint(SDL_HINT_OPENGL_LIBRARY);
     }
     if (path == NULL) {
         path = DEFAULT_OPENGL;
@@ -186,27 +182,27 @@ int X11_GL_LoadLibrary(_THIS, const char *path)
 #if defined(OPENGL_REQUIRES_DLOPEN) && defined(HAVE_DLOPEN)
         SDL_SetError("Failed loading %s: %s", path, dlerror());
 #endif
-        return -1;
+        return false;
     }
     SDL_strlcpy(_this->gl_config.driver_path, path,
                 SDL_arraysize(_this->gl_config.driver_path));
 
-    /* Allocate OpenGL memory */
+    // Allocate OpenGL memory
     _this->gl_data =
         (struct SDL_GLDriverData *)SDL_calloc(1,
                                               sizeof(struct
                                                      SDL_GLDriverData));
     if (!_this->gl_data) {
-        return SDL_OutOfMemory();
+        return false;
     }
 
-    /* Load function pointers */
+    // Load function pointers
     handle = _this->gl_config.dll_handle;
     _this->gl_data->glXQueryExtension =
         (Bool(*)(Display *, int *, int *))
             GL_LoadFunction(handle, "glXQueryExtension");
     _this->gl_data->glXGetProcAddress =
-        (void *(*)(const GLubyte *))
+        (__GLXextFuncPtr (*)(const GLubyte *))
             GL_LoadFunction(handle, "glXGetProcAddressARB");
     _this->gl_data->glXChooseVisual =
         (XVisualInfo * (*)(Display *, int, int *))
@@ -236,14 +232,14 @@ int X11_GL_LoadLibrary(_THIS, const char *path)
         return SDL_SetError("Could not retrieve OpenGL functions");
     }
 
-    display = ((SDL_VideoData *)_this->driverdata)->display;
+    display = _this->internal->display;
     if (!_this->gl_data->glXQueryExtension(display, &_this->gl_data->errorBase, &_this->gl_data->eventBase)) {
         return SDL_SetError("GLX is not supported");
     }
 
     _this->gl_data->swap_interval_tear_behavior = SDL_SWAPINTERVALTEAR_UNTESTED;
 
-    /* Initialize extensions */
+    // Initialize extensions
     /* See lengthy comment about the inc/dec in
        ../windows/SDL_windowsopengl.c. */
     ++_this->gl_config.driver_loaded;
@@ -254,8 +250,8 @@ int X11_GL_LoadLibrary(_THIS, const char *path)
      * GLX_EXT_create_context_es2_profile extension, switch over to X11_GLES functions
      */
     if (((_this->gl_config.profile_mask == SDL_GL_CONTEXT_PROFILE_ES) ||
-         SDL_GetHintBoolean(SDL_HINT_VIDEO_X11_FORCE_EGL, SDL_FALSE)) &&
-        X11_GL_UseEGL(_this) ) {
+         SDL_GetHintBoolean(SDL_HINT_VIDEO_FORCE_EGL, false)) &&
+        X11_GL_UseEGL(_this)) {
 #ifdef SDL_VIDEO_OPENGL_EGL
         X11_GL_UnloadLibrary(_this);
         _this->GL_LoadLibrary = X11_GLES_LoadLibrary;
@@ -266,17 +262,17 @@ int X11_GL_LoadLibrary(_THIS, const char *path)
         _this->GL_SetSwapInterval = X11_GLES_SetSwapInterval;
         _this->GL_GetSwapInterval = X11_GLES_GetSwapInterval;
         _this->GL_SwapWindow = X11_GLES_SwapWindow;
-        _this->GL_DeleteContext = X11_GLES_DeleteContext;
+        _this->GL_DestroyContext = X11_GLES_DestroyContext;
         return X11_GLES_LoadLibrary(_this, NULL);
 #else
         return SDL_SetError("SDL not configured with EGL support");
 #endif
     }
 
-    return 0;
+    return true;
 }
 
-void *X11_GL_GetProcAddress(_THIS, const char *proc)
+SDL_FunctionPointer X11_GL_GetProcAddress(SDL_VideoDevice *_this, const char *proc)
 {
     if (_this->gl_data->glXGetProcAddress) {
         return _this->gl_data->glXGetProcAddress((const GLubyte *)proc);
@@ -284,7 +280,7 @@ void *X11_GL_GetProcAddress(_THIS, const char *proc)
     return GL_LoadFunction(_this->gl_config.dll_handle, proc);
 }
 
-void X11_GL_UnloadLibrary(_THIS)
+void X11_GL_UnloadLibrary(SDL_VideoDevice *_this)
 {
     /* Don't actually unload the library, since it may have registered
      * X11 shutdown hooks, per the notes at:
@@ -295,24 +291,24 @@ void X11_GL_UnloadLibrary(_THIS)
     _this->gl_config.dll_handle = NULL;
 #endif
 
-    /* Free OpenGL memory */
+    // Free OpenGL memory
     SDL_free(_this->gl_data);
     _this->gl_data = NULL;
 }
 
-static SDL_bool HasExtension(const char *extension, const char *extensions)
+static bool HasExtension(const char *extension, const char *extensions)
 {
     const char *start;
     const char *where, *terminator;
 
     if (!extensions) {
-        return SDL_FALSE;
+        return false;
     }
 
-    /* Extension names should not have spaces. */
+    // Extension names should not have spaces.
     where = SDL_strchr(extension, ' ');
     if (where || *extension == '\0') {
-        return SDL_FALSE;
+        return false;
     }
 
     /* It takes a bit of care to be fool-proof about parsing the
@@ -330,18 +326,18 @@ static SDL_bool HasExtension(const char *extension, const char *extensions)
         terminator = where + SDL_strlen(extension);
         if (where == start || *(where - 1) == ' ') {
             if (*terminator == ' ' || *terminator == '\0') {
-                return SDL_TRUE;
+                return true;
             }
         }
 
         start = terminator;
     }
-    return SDL_FALSE;
+    return false;
 }
 
-static void X11_GL_InitExtensions(_THIS)
+static void X11_GL_InitExtensions(SDL_VideoDevice *_this)
 {
-    Display *display = ((SDL_VideoData *)_this->driverdata)->display;
+    Display *display = _this->internal->display;
     const int screen = DefaultScreen(display);
     XVisualInfo *vinfo = NULL;
     Window w = 0;
@@ -351,7 +347,7 @@ static void X11_GL_InitExtensions(_THIS)
     const char *(*glXQueryExtensionsStringFunc)(Display *, int);
     const char *extensions;
 
-    vinfo = X11_GL_GetVisual(_this, display, screen);
+    vinfo = X11_GL_GetVisual(_this, display, screen, false);
     if (vinfo) {
         GLXContext (*glXGetCurrentContextFunc)(void) =
             (GLXContext(*)(void))
@@ -394,18 +390,18 @@ static void X11_GL_InitExtensions(_THIS)
         extensions = NULL;
     }
 
-    /* Check for GLX_EXT_swap_control(_tear) */
-    _this->gl_data->HAS_GLX_EXT_swap_control_tear = SDL_FALSE;
+    // Check for GLX_EXT_swap_control(_tear)
+    _this->gl_data->HAS_GLX_EXT_swap_control_tear = false;
     if (HasExtension("GLX_EXT_swap_control", extensions)) {
         _this->gl_data->glXSwapIntervalEXT =
             (void (*)(Display *, GLXDrawable, int))
                 X11_GL_GetProcAddress(_this, "glXSwapIntervalEXT");
         if (HasExtension("GLX_EXT_swap_control_tear", extensions)) {
-            _this->gl_data->HAS_GLX_EXT_swap_control_tear = SDL_TRUE;
+            _this->gl_data->HAS_GLX_EXT_swap_control_tear = true;
         }
     }
 
-    /* Check for GLX_MESA_swap_control */
+    // Check for GLX_MESA_swap_control
     if (HasExtension("GLX_MESA_swap_control", extensions)) {
         _this->gl_data->glXSwapIntervalMESA =
             (int (*)(int))X11_GL_GetProcAddress(_this, "glXSwapIntervalMESA");
@@ -414,13 +410,13 @@ static void X11_GL_InitExtensions(_THIS)
                                                  "glXGetSwapIntervalMESA");
     }
 
-    /* Check for GLX_SGI_swap_control */
+    // Check for GLX_SGI_swap_control
     if (HasExtension("GLX_SGI_swap_control", extensions)) {
         _this->gl_data->glXSwapIntervalSGI =
             (int (*)(int))X11_GL_GetProcAddress(_this, "glXSwapIntervalSGI");
     }
 
-    /* Check for GLX_ARB_create_context */
+    // Check for GLX_ARB_create_context
     if (HasExtension("GLX_ARB_create_context", extensions)) {
         _this->gl_data->glXCreateContextAttribsARB =
             (GLXContext(*)(Display *, GLXFBConfig, GLXContext, Bool, const int *))
@@ -433,20 +429,20 @@ static void X11_GL_InitExtensions(_THIS)
                 X11_GL_GetProcAddress(_this, "glXGetVisualFromFBConfig");
     }
 
-    /* Check for GLX_EXT_visual_rating */
+    // Check for GLX_EXT_visual_rating
     if (HasExtension("GLX_EXT_visual_rating", extensions)) {
-        _this->gl_data->HAS_GLX_EXT_visual_rating = SDL_TRUE;
+        _this->gl_data->HAS_GLX_EXT_visual_rating = true;
     }
 
-    /* Check for GLX_EXT_visual_info */
+    // Check for GLX_EXT_visual_info
     if (HasExtension("GLX_EXT_visual_info", extensions)) {
-        _this->gl_data->HAS_GLX_EXT_visual_info = SDL_TRUE;
+        _this->gl_data->HAS_GLX_EXT_visual_info = true;
     }
 
-    /* Check for GLX_EXT_create_context_es2_profile */
+    // Check for GLX_EXT_create_context_es2_profile
     if (HasExtension("GLX_EXT_create_context_es2_profile", extensions)) {
-        /* this wants to call glGetString(), so it needs a context. */
-        /* !!! FIXME: it would be nice not to make a context here though! */
+        // this wants to call glGetString(), so it needs a context.
+        // !!! FIXME: it would be nice not to make a context here though!
         if (context) {
             SDL_GL_DeduceMaxSupportedESProfile(
                 &_this->gl_data->es_profile_max_supported_version.major,
@@ -454,19 +450,19 @@ static void X11_GL_InitExtensions(_THIS)
         }
     }
 
-    /* Check for GLX_ARB_context_flush_control */
+    // Check for GLX_ARB_context_flush_control
     if (HasExtension("GLX_ARB_context_flush_control", extensions)) {
-        _this->gl_data->HAS_GLX_ARB_context_flush_control = SDL_TRUE;
+        _this->gl_data->HAS_GLX_ARB_context_flush_control = true;
     }
 
-    /* Check for GLX_ARB_create_context_robustness */
+    // Check for GLX_ARB_create_context_robustness
     if (HasExtension("GLX_ARB_create_context_robustness", extensions)) {
-        _this->gl_data->HAS_GLX_ARB_create_context_robustness = SDL_TRUE;
+        _this->gl_data->HAS_GLX_ARB_create_context_robustness = true;
     }
 
-    /* Check for GLX_ARB_create_context_no_error */
+    // Check for GLX_ARB_create_context_no_error
     if (HasExtension("GLX_ARB_create_context_no_error", extensions)) {
-        _this->gl_data->HAS_GLX_ARB_create_context_no_error = SDL_TRUE;
+        _this->gl_data->HAS_GLX_ARB_create_context_no_error = true;
     }
 
     if (context) {
@@ -490,16 +486,16 @@ static void X11_GL_InitExtensions(_THIS)
  *  In case of failure, if that pointer is not NULL, set that pointer to None
  *  and try again.
  */
-static int X11_GL_GetAttributes(_THIS, Display *display, int screen, int *attribs, int size, Bool for_FBConfig, int **_pvistypeattr)
+static int X11_GL_GetAttributes(SDL_VideoDevice *_this, Display *display, int screen, int *attribs, int size, Bool for_FBConfig, int **_pvistypeattr, bool transparent)
 {
     int i = 0;
     const int MAX_ATTRIBUTES = 64;
     int *pvistypeattr = NULL;
 
-    /* assert buffer is large enough to hold all SDL attributes. */
+    // assert buffer is large enough to hold all SDL attributes.
     SDL_assert(size >= MAX_ATTRIBUTES);
 
-    /* Setup our GLX attributes according to the gl_config. */
+    // Setup our GLX attributes according to the gl_config.
     if (for_FBConfig) {
         attribs[i++] = GLX_RENDER_TYPE;
         if (_this->gl_config.floatbuffers) {
@@ -581,7 +577,7 @@ static int X11_GL_GetAttributes(_THIS, Display *display, int screen, int *attrib
 
     if (_this->gl_config.framebuffer_srgb_capable) {
         attribs[i++] = GLX_FRAMEBUFFER_SRGB_CAPABLE_ARB;
-        attribs[i++] = True; /* always needed, for_FBConfig or not! */
+        attribs[i++] = True; // always needed, for_FBConfig or not!
     }
 
     if (_this->gl_config.accelerated >= 0 &&
@@ -590,13 +586,15 @@ static int X11_GL_GetAttributes(_THIS, Display *display, int screen, int *attrib
         attribs[i++] = _this->gl_config.accelerated ? GLX_NONE_EXT : GLX_SLOW_VISUAL_EXT;
     }
 
-    /* If we're supposed to use DirectColor visuals, and we've got the
-       EXT_visual_info extension, then add GLX_X_VISUAL_TYPE_EXT. */
-    if (X11_UseDirectColorVisuals() &&
-        _this->gl_data->HAS_GLX_EXT_visual_info) {
-        pvistypeattr = &attribs[i];
-        attribs[i++] = GLX_X_VISUAL_TYPE_EXT;
-        attribs[i++] = GLX_DIRECT_COLOR_EXT;
+    // Un-wanted when we request a transparent buffer
+    if (!transparent) {
+        /* If we're supposed to use DirectColor visuals, and we've got the
+           EXT_visual_info extension, then add GLX_X_VISUAL_TYPE_EXT. */
+        if (X11_UseDirectColorVisuals() && _this->gl_data->HAS_GLX_EXT_visual_info) {
+            pvistypeattr = &attribs[i];
+            attribs[i++] = GLX_X_VISUAL_TYPE_EXT;
+            attribs[i++] = GLX_DIRECT_COLOR_EXT;
+        }
     }
 
     attribs[i++] = None;
@@ -610,15 +608,41 @@ static int X11_GL_GetAttributes(_THIS, Display *display, int screen, int *attrib
     return i;
 }
 
-XVisualInfo *X11_GL_GetVisual(_THIS, Display *display, int screen)
+//get the first transparent Visual
+static XVisualInfo* X11_GL_GetTransparentVisualInfo(Display *display, int screen)
 {
-    /* 64 seems nice. */
+    XVisualInfo* visualinfo = NULL;
+    XVisualInfo vi_in;
+    int out_count = 0;
+
+    vi_in.screen = screen;
+    visualinfo = X11_XGetVisualInfo(display, VisualScreenMask, &vi_in, &out_count);
+    if (visualinfo != NULL) {
+        int i = 0;
+        for (i = 0; i < out_count; i++) {
+            XVisualInfo* v = &visualinfo[i];
+            Uint32 format = X11_GetPixelFormatFromVisualInfo(display, v);
+            if (SDL_ISPIXELFORMAT_ALPHA(format)) {
+                vi_in.screen = screen;
+                vi_in.visualid = v->visualid;
+                X11_XFree(visualinfo);
+                visualinfo = X11_XGetVisualInfo(display, VisualScreenMask | VisualIDMask, &vi_in, &out_count);
+                break;
+            }
+        }
+    }
+    return visualinfo;
+}
+
+XVisualInfo *X11_GL_GetVisual(SDL_VideoDevice *_this, Display *display, int screen, bool transparent)
+{
+    // 64 seems nice.
     int attribs[64];
     XVisualInfo *vinfo = NULL;
     int *pvistypeattr = NULL;
 
     if (!_this->gl_data) {
-        /* The OpenGL library wasn't loaded, SDL_GetError() should have info */
+        // The OpenGL library wasn't loaded, SDL_GetError() should have info
         return NULL;
     }
 
@@ -627,13 +651,30 @@ XVisualInfo *X11_GL_GetVisual(_THIS, Display *display, int screen)
         GLXFBConfig *framebuffer_config = NULL;
         int fbcount = 0;
 
-        X11_GL_GetAttributes(_this, display, screen, attribs, 64, SDL_TRUE, &pvistypeattr);
+        X11_GL_GetAttributes(_this, display, screen, attribs, 64, true, &pvistypeattr, transparent);
         framebuffer_config = _this->gl_data->glXChooseFBConfig(display, screen, attribs, &fbcount);
         if (!framebuffer_config && (pvistypeattr != NULL)) {
             *pvistypeattr = None;
             framebuffer_config = _this->gl_data->glXChooseFBConfig(display, screen, attribs, &fbcount);
         }
 
+        if (transparent) {
+            // Return the first transparent Visual
+            int i;
+            for (i = 0; i < fbcount; i++) {
+                Uint32 format;
+                vinfo = _this->gl_data->glXGetVisualFromFBConfig(display, framebuffer_config[i]);
+                format = X11_GetPixelFormatFromVisualInfo(display, vinfo);
+                if (SDL_ISPIXELFORMAT_ALPHA(format)) { // found!
+                    X11_XFree(framebuffer_config);
+                    framebuffer_config = NULL;
+                    break;
+                }
+                X11_XFree(vinfo);
+                vinfo = NULL;
+            }
+        }
+
         if (framebuffer_config) {
             vinfo = _this->gl_data->glXGetVisualFromFBConfig(display, framebuffer_config[0]);
         }
@@ -642,7 +683,7 @@ XVisualInfo *X11_GL_GetVisual(_THIS, Display *display, int screen)
     }
 
     if (!vinfo) {
-        X11_GL_GetAttributes(_this, display, screen, attribs, 64, SDL_FALSE, &pvistypeattr);
+        X11_GL_GetAttributes(_this, display, screen, attribs, 64, false, &pvistypeattr, transparent);
         vinfo = _this->gl_data->glXChooseVisual(display, screen, attribs);
 
         if (!vinfo && (pvistypeattr != NULL)) {
@@ -651,6 +692,18 @@ XVisualInfo *X11_GL_GetVisual(_THIS, Display *display, int screen)
         }
     }
 
+    if (transparent && vinfo) {
+        Uint32 format = X11_GetPixelFormatFromVisualInfo(display, vinfo);
+        if (!SDL_ISPIXELFORMAT_ALPHA(format)) {
+            // not transparent!
+            XVisualInfo* visualinfo = X11_GL_GetTransparentVisualInfo(display, screen);
+            if (visualinfo != NULL) {
+                X11_XFree(vinfo);
+                vinfo = visualinfo;
+            }
+        }
+    }
+
     if (!vinfo) {
         SDL_SetError("Couldn't find matching GLX visual");
     }
@@ -678,33 +731,33 @@ static int X11_GL_ErrorHandler(Display *d, XErrorEvent *e)
         SDL_SetError("Could not %s: %i (Base %i)", errorHandlerOperation, errorCode, errorBase);
     }
 
-    return (0);
+    return 0;
 }
 
-SDL_bool X11_GL_UseEGL(_THIS)
+bool X11_GL_UseEGL(SDL_VideoDevice *_this)
 {
     SDL_assert(_this->gl_data != NULL);
-    if (SDL_GetHintBoolean(SDL_HINT_VIDEO_X11_FORCE_EGL, SDL_FALSE))
-    {
-        /* use of EGL has been requested, even for desktop GL */
-        return SDL_TRUE;
+    if (SDL_GetHintBoolean(SDL_HINT_VIDEO_FORCE_EGL, false)) {
+        // use of EGL has been requested, even for desktop GL
+        return true;
     }
 
     SDL_assert(_this->gl_config.profile_mask == SDL_GL_CONTEXT_PROFILE_ES);
-    return (SDL_GetHintBoolean(SDL_HINT_OPENGL_ES_DRIVER, SDL_FALSE) || _this->gl_config.major_version == 1 /* No GLX extension for OpenGL ES 1.x profiles. */
+    return (SDL_GetHintBoolean(SDL_HINT_OPENGL_ES_DRIVER, false) || _this->gl_config.major_version == 1 // No GLX extension for OpenGL ES 1.x profiles.
             || _this->gl_config.major_version > _this->gl_data->es_profile_max_supported_version.major || (_this->gl_config.major_version == _this->gl_data->es_profile_max_supported_version.major && _this->gl_config.minor_version > _this->gl_data->es_profile_max_supported_version.minor));
 }
 
-SDL_GLContext X11_GL_CreateContext(_THIS, SDL_Window *window)
+SDL_GLContext X11_GL_CreateContext(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    SDL_WindowData *data = (SDL_WindowData *)window->driverdata;
+    SDL_WindowData *data = window->internal;
     Display *display = data->videodata->display;
-    int screen =
-        ((SDL_DisplayData *)SDL_GetDisplayForWindow(window)->driverdata)->screen;
+    int screen = SDL_GetDisplayDriverDataForWindow(window)->screen;
     XWindowAttributes xattr;
     XVisualInfo v, *vinfo;
     int n;
-    GLXContext context = NULL, share_context;
+    SDL_GLContext context = NULL;
+    GLXContext share_context;
+    const int transparent = (window->flags & SDL_WINDOW_TRANSPARENT) ? true : false;
 
     if (_this->gl_config.share_with_current_context) {
         share_context = (GLXContext)SDL_GL_GetCurrentContext();
@@ -712,7 +765,7 @@ SDL_GLContext X11_GL_CreateContext(_THIS, SDL_Window *window)
         share_context = NULL;
     }
 
-    /* We do this to create a clean separation between X and GLX errors. */
+    // We do this to create a clean separation between X and GLX errors.
     X11_XSync(display, False);
     errorHandlerOperation = "create GL context";
     errorBase = _this->gl_data->errorBase;
@@ -725,12 +778,12 @@ SDL_GLContext X11_GL_CreateContext(_THIS, SDL_Window *window)
     if (vinfo) {
         if (_this->gl_config.major_version < 3 &&
             _this->gl_config.profile_mask == 0 &&
-            _this->gl_config.flags == 0) {
-            /* Create legacy context */
+            _this->gl_config.flags == 0 && !transparent) {
+            // Create legacy context
             context =
-                _this->gl_data->glXCreateContext(display, vinfo, share_context, True);
+                (SDL_GLContext)_this->gl_data->glXCreateContext(display, vinfo, share_context, True);
         } else {
-            /* max 14 attributes plus terminator */
+            // max 14 attributes plus terminator
             int attribs[15] = {
                 GLX_CONTEXT_MAJOR_VERSION_ARB,
                 _this->gl_config.major_version,
@@ -740,33 +793,33 @@ SDL_GLContext X11_GL_CreateContext(_THIS, SDL_Window *window)
             };
             int iattr = 4;
 
-            /* SDL profile bits match GLX profile bits */
+            // SDL profile bits match GLX profile bits
             if (_this->gl_config.profile_mask != 0) {
                 attribs[iattr++] = GLX_CONTEXT_PROFILE_MASK_ARB;
                 attribs[iattr++] = _this->gl_config.profile_mask;
             }
 
-            /* SDL flags match GLX flags */
+            // SDL flags match GLX flags
             if (_this->gl_config.flags != 0) {
                 attribs[iattr++] = GLX_CONTEXT_FLAGS_ARB;
                 attribs[iattr++] = _this->gl_config.flags;
             }
 
-            /* only set if glx extension is available and not the default setting */
+            // only set if glx extension is available and not the default setting
             if ((_this->gl_data->HAS_GLX_ARB_context_flush_control) && (_this->gl_config.release_behavior == 0)) {
                 attribs[iattr++] = GLX_CONTEXT_RELEASE_BEHAVIOR_ARB;
                 attribs[iattr++] =
                     _this->gl_config.release_behavior ? GLX_CONTEXT_RELEASE_BEHAVIOR_FLUSH_ARB : GLX_CONTEXT_RELEASE_BEHAVIOR_NONE_ARB;
             }
 
-            /* only set if glx extension is available and not the default setting */
+            // only set if glx extension is available and not the default setting
             if ((_this->gl_data->HAS_GLX_ARB_create_context_robustness) && (_this->gl_config.reset_notification != 0)) {
                 attribs[iattr++] = GLX_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB;
                 attribs[iattr++] =
                     _this->gl_config.reset_notification ? GLX_LOSE_CONTEXT_ON_RESET_ARB : GLX_NO_RESET_NOTIFICATION_ARB;
             }
 
-            /* only set if glx extension is available and not the default setting */
+            // only set if glx extension is available and not the default setting
             if ((_this->gl_data->HAS_GLX_ARB_create_context_no_error) && (_this->gl_config.no_error != 0)) {
                 attribs[iattr++] = GLX_CONTEXT_OPENGL_NO_ERROR_ARB;
                 attribs[iattr++] = _this->gl_config.no_error;
@@ -774,18 +827,18 @@ SDL_GLContext X11_GL_CreateContext(_THIS, SDL_Window *window)
 
             attribs[iattr++] = 0;
 
-            /* Get a pointer to the context creation function for GL 3.0 */
+            // Get a pointer to the context creation function for GL 3.0
             if (!_this->gl_data->glXCreateContextAttribsARB) {
                 SDL_SetError("OpenGL 3.0 and later are not supported by this system");
             } else {
                 int glxAttribs[64];
 
-                /* Create a GL 3.x context */
+                // Create a GL 3.x context
                 GLXFBConfig *framebuffer_config = NULL;
                 int fbcount = 0;
                 int *pvistypeattr = NULL;
 
-                X11_GL_GetAttributes(_this, display, screen, glxAttribs, 64, SDL_TRUE, &pvistypeattr);
+                X11_GL_GetAttributes(_this, display, screen, glxAttribs, 64, true, &pvistypeattr, transparent);
 
                 if (_this->gl_data->glXChooseFBConfig) {
                     framebuffer_config = _this->gl_data->glXChooseFBConfig(display,
@@ -799,8 +852,28 @@ SDL_GLContext X11_GL_CreateContext(_THIS, SDL_Window *window)
                                                                                &fbcount);
                     }
 
+                    if (transparent && (framebuffer_config != NULL)) {
+                        int i;
+                        for (i = 0; i < fbcount; i++) {
+                            XVisualInfo* vinfo_temp = _this->gl_data->glXGetVisualFromFBConfig(display, framebuffer_config[i]);
+                            if ( vinfo_temp != NULL) {
+                                Uint32 format = X11_GetPixelFormatFromVisualInfo(display, vinfo_temp);
+                                if (SDL_ISPIXELFORMAT_ALPHA(format)) {
+                                    // found!
+                                    context = (SDL_GLContext)_this->gl_data->glXCreateContextAttribsARB(display,
+                                                                                                        framebuffer_config[i],
+                                                                                                        share_context, True, attribs);
+                                    X11_XFree(framebuffer_config);
+                                    framebuffer_config = NULL;
+                                    X11_XFree(vinfo_temp);
+                                    break;
+                                }
+                                X11_XFree(vinfo_temp);
+                            }
+                        }
+                    }
                     if (framebuffer_config) {
-                        context = _this->gl_data->glXCreateContextAttribsARB(display,
+                        context = (SDL_GLContext)_this->gl_data->glXCreateContextAttribsARB(display,
                                                                              framebuffer_config[0],
                                                                              share_context, True, attribs);
                         X11_XFree(framebuffer_config);
@@ -820,19 +893,19 @@ SDL_GLContext X11_GL_CreateContext(_THIS, SDL_Window *window)
         return NULL;
     }
 
-    if (X11_GL_MakeCurrent(_this, window, context) < 0) {
-        X11_GL_DeleteContext(_this, context);
+    if (!X11_GL_MakeCurrent(_this, window, context)) {
+        X11_GL_DestroyContext(_this, context);
         return NULL;
     }
 
     return context;
 }
 
-int X11_GL_MakeCurrent(_THIS, SDL_Window *window, SDL_GLContext context)
+bool X11_GL_MakeCurrent(SDL_VideoDevice *_this, SDL_Window *window, SDL_GLContext context)
 {
-    Display *display = ((SDL_VideoData *)_this->driverdata)->display;
+    Display *display = _this->internal->display;
     Window drawable =
-        (context ? ((SDL_WindowData *)window->driverdata)->xwindow : None);
+        (context ? window->internal->xwindow : None);
     GLXContext glx_context = (GLXContext)context;
     int rc;
 
@@ -840,7 +913,7 @@ int X11_GL_MakeCurrent(_THIS, SDL_Window *window, SDL_GLContext context)
         return SDL_SetError("OpenGL not initialized");
     }
 
-    /* We do this to create a clean separation between X and GLX errors. */
+    // We do this to create a clean separation between X and GLX errors.
     X11_XSync(display, False);
     errorHandlerOperation = "make GL context current";
     errorBase = _this->gl_data->errorBase;
@@ -849,13 +922,13 @@ int X11_GL_MakeCurrent(_THIS, SDL_Window *window, SDL_GLContext context)
     rc = _this->gl_data->glXMakeCurrent(display, drawable, glx_context);
     X11_XSetErrorHandler(handler);
 
-    if (errorCode != Success) { /* uhoh, an X error was thrown! */
-        return -1;              /* the error handler called SDL_SetError() already. */
-    } else if (!rc) {           /* glXMakeCurrent() failed without throwing an X error */
+    if (errorCode != Success) { // uhoh, an X error was thrown!
+        return false;              // the error handler called SDL_SetError() already.
+    } else if (!rc) {           // glXMakeCurrent() failed without throwing an X error
         return SDL_SetError("Unable to make GL context current");
     }
 
-    return 0;
+    return true;
 }
 
 /*
@@ -867,17 +940,15 @@ int X11_GL_MakeCurrent(_THIS, SDL_Window *window, SDL_GLContext context)
 */
 
 static int swapinterval = 0;
-int X11_GL_SetSwapInterval(_THIS, int interval)
+bool X11_GL_SetSwapInterval(SDL_VideoDevice *_this, int interval)
 {
-    int status = -1;
+    bool result = false;
 
     if ((interval < 0) && (!_this->gl_data->HAS_GLX_EXT_swap_control_tear)) {
-        SDL_SetError("Negative swap interval unsupported in this GL");
+        return SDL_SetError("Negative swap interval unsupported in this GL");
     } else if (_this->gl_data->glXSwapIntervalEXT) {
-        Display *display = ((SDL_VideoData *)_this->driverdata)->display;
-        const SDL_WindowData *windowdata = (SDL_WindowData *)
-                                               SDL_GL_GetCurrentWindow()
-                                                   ->driverdata;
+        Display *display = _this->internal->display;
+        const SDL_WindowData *windowdata = SDL_GL_GetCurrentWindow()->internal;
 
         Window drawable = windowdata->xwindow;
 
@@ -889,29 +960,32 @@ int X11_GL_SetSwapInterval(_THIS, int interval)
          * it has the wrong value cached. To work around it, we just run a no-op
          * update to the current value.
          */
-        int currentInterval = X11_GL_GetSwapInterval(_this);
+        int currentInterval = 0;
+        X11_GL_GetSwapInterval(_this, &currentInterval);
         _this->gl_data->glXSwapIntervalEXT(display, drawable, currentInterval);
         _this->gl_data->glXSwapIntervalEXT(display, drawable, interval);
-        status = 0;
+        result = true;
         swapinterval = interval;
     } else if (_this->gl_data->glXSwapIntervalMESA) {
-        status = _this->gl_data->glXSwapIntervalMESA(interval);
-        if (status != 0) {
-            SDL_SetError("glXSwapIntervalMESA failed");
-        } else {
+        const int rc = _this->gl_data->glXSwapIntervalMESA(interval);
+        if (rc == 0) {
             swapinterval = interval;
+            result = true;
+        } else {
+            result = SDL_SetError("glXSwapIntervalMESA failed");
         }
     } else if (_this->gl_data->glXSwapIntervalSGI) {
-        status = _this->gl_data->glXSwapIntervalSGI(interval);
-        if (status != 0) {
-            SDL_SetError("glXSwapIntervalSGI failed");
-        } else {
+        const int rc = _this->gl_data->glXSwapIntervalSGI(interval);
+        if (rc == 0) {
             swapinterval = interval;
+            result = true;
+        } else {
+            result = SDL_SetError("glXSwapIntervalSGI failed");
         }
     } else {
-        SDL_Unsupported();
+        return SDL_Unsupported();
     }
-    return status;
+    return result;
 }
 
 static SDL_GLSwapIntervalTearBehavior CheckSwapIntervalTearBehavior(SDL_VideoDevice *_this, Window drawable, unsigned int current_val, unsigned int current_allow_late)
@@ -923,7 +997,7 @@ static SDL_GLSwapIntervalTearBehavior CheckSwapIntervalTearBehavior(SDL_VideoDev
         if (!_this->gl_data->HAS_GLX_EXT_swap_control_tear) {
             _this->gl_data->swap_interval_tear_behavior = SDL_SWAPINTERVALTEAR_UNKNOWN;
         } else {
-            Display *display = ((SDL_VideoData *)_this->driverdata)->display;
+            Display *display = _this->internal->display;
             unsigned int allow_late_swap_tearing = 22;
             int original_val = (int) current_val;
 
@@ -937,22 +1011,22 @@ static SDL_GLSwapIntervalTearBehavior CheckSwapIntervalTearBehavior(SDL_VideoDev
              */
             _this->gl_data->glXSwapIntervalEXT(display, drawable, current_val);
 
-            /* set it to no swap interval and see how it affects GLX_LATE_SWAPS_TEAR_EXT... */
+            // set it to no swap interval and see how it affects GLX_LATE_SWAPS_TEAR_EXT...
             _this->gl_data->glXSwapIntervalEXT(display, drawable, 0);
             _this->gl_data->glXQueryDrawable(display, drawable, GLX_LATE_SWAPS_TEAR_EXT, &allow_late_swap_tearing);
 
-            if (allow_late_swap_tearing == 0) { /* GLX_LATE_SWAPS_TEAR_EXT says whether late swapping is currently in use */
+            if (allow_late_swap_tearing == 0) { // GLX_LATE_SWAPS_TEAR_EXT says whether late swapping is currently in use
                 _this->gl_data->swap_interval_tear_behavior = SDL_SWAPINTERVALTEAR_NVIDIA;
                 if (current_allow_late) {
                     original_val = -original_val;
                 }
-            } else if (allow_late_swap_tearing == 1) {  /* GLX_LATE_SWAPS_TEAR_EXT says whether the Drawable can use late swapping at all */
+            } else if (allow_late_swap_tearing == 1) {  // GLX_LATE_SWAPS_TEAR_EXT says whether the Drawable can use late swapping at all
                 _this->gl_data->swap_interval_tear_behavior = SDL_SWAPINTERVALTEAR_MESA;
-            } else {  /* unexpected outcome! */
+            } else {  // unexpected outcome!
                 _this->gl_data->swap_interval_tear_behavior = SDL_SWAPINTERVALTEAR_UNKNOWN;
             }
 
-            /* set us back to what it was originally... */
+            // set us back to what it was originally...
             _this->gl_data->glXSwapIntervalEXT(display, drawable, original_val);
         }
     }
@@ -961,70 +1035,76 @@ static SDL_GLSwapIntervalTearBehavior CheckSwapIntervalTearBehavior(SDL_VideoDev
 }
 
 
-int X11_GL_GetSwapInterval(_THIS)
+bool X11_GL_GetSwapInterval(SDL_VideoDevice *_this, int *interval)
 {
     if (_this->gl_data->glXSwapIntervalEXT) {
-        Display *display = ((SDL_VideoData *)_this->driverdata)->display;
-        const SDL_WindowData *windowdata = (SDL_WindowData *)
-                                               SDL_GL_GetCurrentWindow()
-                                                   ->driverdata;
+        Display *display = _this->internal->display;
+        const SDL_WindowData *windowdata = SDL_GL_GetCurrentWindow()->internal;
         Window drawable = windowdata->xwindow;
         unsigned int allow_late_swap_tearing = 0;
-        unsigned int interval = 0;
+        unsigned int val = 0;
 
         if (_this->gl_data->HAS_GLX_EXT_swap_control_tear) {
-            allow_late_swap_tearing = 22;  /* set this to nonsense. */
+            allow_late_swap_tearing = 22;  // set this to nonsense.
             _this->gl_data->glXQueryDrawable(display, drawable,
                                              GLX_LATE_SWAPS_TEAR_EXT,
                                              &allow_late_swap_tearing);
         }
 
         _this->gl_data->glXQueryDrawable(display, drawable,
-                                         GLX_SWAP_INTERVAL_EXT, &interval);
+                                         GLX_SWAP_INTERVAL_EXT, &val);
 
-        switch (CheckSwapIntervalTearBehavior(_this, drawable, interval, allow_late_swap_tearing)) {
+        *interval = (int)val;
+
+        switch (CheckSwapIntervalTearBehavior(_this, drawable, val, allow_late_swap_tearing)) {
             case SDL_SWAPINTERVALTEAR_MESA:
-                return (int)interval;  /* unsigned int cast to signed that generates negative value if necessary. */
+                *interval = (int)val;  // unsigned int cast to signed that generates negative value if necessary.
+                break;
 
             case SDL_SWAPINTERVALTEAR_NVIDIA:
             default:
-                if ((allow_late_swap_tearing) && (interval > 0)) {
-                    return -((int)interval);
+                if ((allow_late_swap_tearing) && (val > 0)) {
+                    *interval = -((int)val);
                 }
-                return (int)interval;
+                break;
         }
 
-        return (int)interval;  /* shouldn't hit this, but just in case. */
+        return true;
     } else if (_this->gl_data->glXGetSwapIntervalMESA) {
-        return _this->gl_data->glXGetSwapIntervalMESA();
+        int val = _this->gl_data->glXGetSwapIntervalMESA();
+        if (val == GLX_BAD_CONTEXT) {
+            return SDL_SetError("GLX_BAD_CONTEXT");
+        }
+        *interval = val;
+        return true;
     } else {
-        return swapinterval;
+        *interval = swapinterval;
+        return true;
     }
 }
 
-int X11_GL_SwapWindow(_THIS, SDL_Window *window)
+bool X11_GL_SwapWindow(SDL_VideoDevice *_this, SDL_Window *window)
 {
-    SDL_WindowData *data = (SDL_WindowData *)window->driverdata;
+    SDL_WindowData *data = window->internal;
     Display *display = data->videodata->display;
 
     _this->gl_data->glXSwapBuffers(display, data->xwindow);
-    return 0;
+    return true;
 }
 
-void X11_GL_DeleteContext(_THIS, SDL_GLContext context)
+bool X11_GL_DestroyContext(SDL_VideoDevice *_this, SDL_GLContext context)
 {
-    Display *display = ((SDL_VideoData *)_this->driverdata)->display;
+    Display *display = _this->internal->display;
     GLXContext glx_context = (GLXContext)context;
 
     if (!_this->gl_data) {
-        return;
+        return true;
     }
     _this->gl_data->glXDestroyContext(display, glx_context);
     X11_XSync(display, False);
+    return true;
 }
 
-#endif /* SDL_VIDEO_OPENGL_GLX */
+#endif // SDL_VIDEO_OPENGL_GLX
 
-#endif /* SDL_VIDEO_DRIVER_X11 */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_X11
