@@ -18,90 +18,44 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_VIDEO_DRIVER_X11
 
-#include <unistd.h> /* For getpid() and readlink() */
+#include <unistd.h> // For getpid() and readlink()
 
-#include "SDL_video.h"
-#include "SDL_mouse.h"
-#include "SDL_timer.h"
-#include "SDL_hints.h"
-#include "../SDL_sysvideo.h"
+#include "../../core/linux/SDL_system_theme.h"
+#include "../../events/SDL_keyboard_c.h"
+#include "../../events/SDL_mouse_c.h"
 #include "../SDL_pixels_c.h"
+#include "../SDL_sysvideo.h"
 
-#include "SDL_x11video.h"
 #include "SDL_x11framebuffer.h"
-#include "SDL_x11shape.h"
+#include "SDL_x11pen.h"
 #include "SDL_x11touch.h"
-#include "SDL_x11xinput2.h"
+#include "SDL_x11video.h"
 #include "SDL_x11xfixes.h"
+#include "SDL_x11xinput2.h"
 #include "SDL_x11messagebox.h"
+#include "SDL_x11shape.h"
 
 #ifdef SDL_VIDEO_OPENGL_EGL
 #include "SDL_x11opengles.h"
 #endif
 
-#include "SDL_x11vulkan.h"
+// Initialization/Query functions
+static bool X11_VideoInit(SDL_VideoDevice *_this);
+static void X11_VideoQuit(SDL_VideoDevice *_this);
 
-/* Initialization/Query functions */
-static int X11_VideoInit(_THIS);
-static void X11_VideoQuit(_THIS);
-
-/* Find out what class name we should use */
-static char *get_classname()
-{
-    char *spot;
-#if defined(__LINUX__) || defined(__FREEBSD__)
-    char procfile[1024];
-    char linkfile[1024];
-    int linksize;
-#endif
-
-    /* First allow environment variable override */
-    spot = SDL_getenv("SDL_VIDEO_X11_WMCLASS");
-    if (spot) {
-        return SDL_strdup(spot);
-    }
-
-    /* Next look at the application's executable name */
-#if defined(__LINUX__) || defined(__FREEBSD__)
-#if defined(__LINUX__)
-    (void)SDL_snprintf(procfile, SDL_arraysize(procfile), "/proc/%d/exe", getpid());
-#elif defined(__FREEBSD__)
-    (void)SDL_snprintf(procfile, SDL_arraysize(procfile), "/proc/%d/file", getpid());
-#else
-#error Where can we find the executable name?
-#endif
-    linksize = readlink(procfile, linkfile, sizeof(linkfile) - 1);
-    if (linksize > 0) {
-        linkfile[linksize] = '\0';
-        spot = SDL_strrchr(linkfile, '/');
-        if (spot) {
-            return SDL_strdup(spot + 1);
-        } else {
-            return SDL_strdup(linkfile);
-        }
-    }
-#endif /* __LINUX__ || __FREEBSD__ */
-
-    /* Finally use the default we've used forever */
-    return SDL_strdup("SDL_App");
-}
-
-/* X11 driver bootstrap functions */
-
-static int (*orig_x11_errhandler)(Display *, XErrorEvent *) = NULL;
+// X11 driver bootstrap functions
 
 static void X11_DeleteDevice(SDL_VideoDevice *device)
 {
-    SDL_VideoData *data = (SDL_VideoData *)device->driverdata;
+    SDL_VideoData *data = device->internal;
     if (device->vulkan_config.loader_handle) {
         device->Vulkan_UnloadLibrary(device);
     }
     if (data->display) {
-        X11_XSetErrorHandler(orig_x11_errhandler);
         X11_XCloseDisplay(data->display);
     }
     if (data->request_display) {
@@ -111,41 +65,13 @@ static void X11_DeleteDevice(SDL_VideoDevice *device)
     if (device->wakeup_lock) {
         SDL_DestroyMutex(device->wakeup_lock);
     }
-    SDL_free(device->driverdata);
+    SDL_free(device->internal);
     SDL_free(device);
 
     SDL_X11_UnloadSymbols();
 }
 
-/* An error handler to reset the vidmode and then call the default handler. */
-static SDL_bool safety_net_triggered = SDL_FALSE;
-static int X11_SafetyNetErrHandler(Display *d, XErrorEvent *e)
-{
-    SDL_VideoDevice *device = NULL;
-    /* if we trigger an error in our error handler, don't try again. */
-    if (!safety_net_triggered) {
-        safety_net_triggered = SDL_TRUE;
-        device = SDL_GetVideoDevice();
-        if (device) {
-            int i;
-            for (i = 0; i < device->num_displays; i++) {
-                SDL_VideoDisplay *display = &device->displays[i];
-                if (SDL_memcmp(&display->current_mode, &display->desktop_mode,
-                               sizeof(SDL_DisplayMode)) != 0) {
-                    X11_SetDisplayMode(device, display, &display->desktop_mode);
-                }
-            }
-        }
-    }
-
-    if (orig_x11_errhandler) {
-        return orig_x11_errhandler(d, e); /* probably terminate. */
-    }
-
-    return 0;
-}
-
-static SDL_bool X11_IsXWayland(Display *d)
+static bool X11_IsXWayland(Display *d)
 {
     int opcode, event, error;
     return X11_XQueryExtension(d, "XWAYLAND", &opcode, &event, &error) == True;
@@ -155,7 +81,7 @@ static SDL_VideoDevice *X11_CreateDevice(void)
 {
     SDL_VideoDevice *device;
     SDL_VideoData *data;
-    const char *display = NULL; /* Use the DISPLAY environment variable */
+    const char *display = NULL; // Use the DISPLAY environment variable
     Display *x11_display = NULL;
 
     if (!SDL_X11_LoadSymbols()) {
@@ -166,7 +92,7 @@ static SDL_VideoDevice *X11_CreateDevice(void)
         nVidia driver to be threaded. */
     X11_XInitThreads();
 
-    /* Open the display first to be sure that X11 is available */
+    // Open the display first to be sure that X11 is available
     x11_display = X11_XOpenDisplay(display);
 
     if (!x11_display) {
@@ -174,31 +100,29 @@ static SDL_VideoDevice *X11_CreateDevice(void)
         return NULL;
     }
 
-    /* Initialize all variables that we clean on shutdown */
+    // Initialize all variables that we clean on shutdown
     device = (SDL_VideoDevice *)SDL_calloc(1, sizeof(SDL_VideoDevice));
     if (!device) {
-        SDL_OutOfMemory();
         return NULL;
     }
     data = (struct SDL_VideoData *)SDL_calloc(1, sizeof(SDL_VideoData));
     if (!data) {
         SDL_free(device);
-        SDL_OutOfMemory();
         return NULL;
     }
-    device->driverdata = data;
+    device->internal = data;
 
-    data->global_mouse_changed = SDL_TRUE;
+    data->global_mouse_changed = true;
 
 #ifdef SDL_VIDEO_DRIVER_X11_XFIXES
     data->active_cursor_confined_window = NULL;
-#endif /* SDL_VIDEO_DRIVER_X11_XFIXES */
+#endif // SDL_VIDEO_DRIVER_X11_XFIXES
 
     data->display = x11_display;
     data->request_display = X11_XOpenDisplay(display);
     if (!data->request_display) {
         X11_XCloseDisplay(data->display);
-        SDL_free(device->driverdata);
+        SDL_free(device->internal);
         SDL_free(device);
         SDL_X11_UnloadSymbols();
         return NULL;
@@ -210,23 +134,18 @@ static SDL_VideoDevice *X11_CreateDevice(void)
     X11_XSynchronize(data->display, True);
 #endif
 
-    /* Hook up an X11 error handler to recover the desktop resolution. */
-    safety_net_triggered = SDL_FALSE;
-    orig_x11_errhandler = X11_XSetErrorHandler(X11_SafetyNetErrHandler);
-
     /* Steam Deck will have an on-screen keyboard, so check their environment
      * variable so we can make use of SDL_StartTextInput.
      */
-    data->is_steam_deck = SDL_GetHintBoolean("SteamDeck", SDL_FALSE);
+    data->is_steam_deck = SDL_GetHintBoolean("SteamDeck", false);
 
-    /* Set the function pointers */
+    // Set the function pointers
     device->VideoInit = X11_VideoInit;
     device->VideoQuit = X11_VideoQuit;
     device->ResetTouch = X11_ResetTouch;
     device->GetDisplayModes = X11_GetDisplayModes;
     device->GetDisplayBounds = X11_GetDisplayBounds;
     device->GetDisplayUsableBounds = X11_GetDisplayUsableBounds;
-    device->GetDisplayDPI = X11_GetDisplayDPI;
     device->GetWindowICCProfile = X11_GetWindowICCProfile;
     device->SetDisplayMode = X11_SetDisplayMode;
     device->SuspendScreenSaver = X11_SuspendScreenSaver;
@@ -235,17 +154,17 @@ static SDL_VideoDevice *X11_CreateDevice(void)
     device->SendWakeupEvent = X11_SendWakeupEvent;
 
     device->CreateSDLWindow = X11_CreateWindow;
-    device->CreateSDLWindowFrom = X11_CreateWindowFrom;
     device->SetWindowTitle = X11_SetWindowTitle;
     device->SetWindowIcon = X11_SetWindowIcon;
     device->SetWindowPosition = X11_SetWindowPosition;
     device->SetWindowSize = X11_SetWindowSize;
     device->SetWindowMinimumSize = X11_SetWindowMinimumSize;
     device->SetWindowMaximumSize = X11_SetWindowMaximumSize;
+    device->SetWindowAspectRatio = X11_SetWindowAspectRatio;
     device->GetWindowBordersSize = X11_GetWindowBordersSize;
     device->SetWindowOpacity = X11_SetWindowOpacity;
-    device->SetWindowModalFor = X11_SetWindowModalFor;
-    device->SetWindowInputFocus = X11_SetWindowInputFocus;
+    device->SetWindowParent = X11_SetWindowParent;
+    device->SetWindowModal = X11_SetWindowModal;
     device->ShowWindow = X11_ShowWindow;
     device->HideWindow = X11_HideWindow;
     device->RaiseWindow = X11_RaiseWindow;
@@ -256,25 +175,23 @@ static SDL_VideoDevice *X11_CreateDevice(void)
     device->SetWindowResizable = X11_SetWindowResizable;
     device->SetWindowAlwaysOnTop = X11_SetWindowAlwaysOnTop;
     device->SetWindowFullscreen = X11_SetWindowFullscreen;
-    device->SetWindowGammaRamp = X11_SetWindowGammaRamp;
     device->SetWindowMouseGrab = X11_SetWindowMouseGrab;
     device->SetWindowKeyboardGrab = X11_SetWindowKeyboardGrab;
     device->DestroyWindow = X11_DestroyWindow;
     device->CreateWindowFramebuffer = X11_CreateWindowFramebuffer;
     device->UpdateWindowFramebuffer = X11_UpdateWindowFramebuffer;
     device->DestroyWindowFramebuffer = X11_DestroyWindowFramebuffer;
-    device->GetWindowWMInfo = X11_GetWindowWMInfo;
     device->SetWindowHitTest = X11_SetWindowHitTest;
     device->AcceptDragAndDrop = X11_AcceptDragAndDrop;
+    device->UpdateWindowShape = X11_UpdateWindowShape;
     device->FlashWindow = X11_FlashWindow;
+    device->ShowWindowSystemMenu = X11_ShowWindowSystemMenu;
+    device->SetWindowFocusable = X11_SetWindowFocusable;
+    device->SyncWindow = X11_SyncWindow;
 
 #ifdef SDL_VIDEO_DRIVER_X11_XFIXES
     device->SetWindowMouseRect = X11_SetWindowMouseRect;
-#endif /* SDL_VIDEO_DRIVER_X11_XFIXES */
-
-    device->shape_driver.CreateShaper = X11_CreateShaper;
-    device->shape_driver.SetWindowShape = X11_SetWindowShape;
-    device->shape_driver.ResizeWindowShape = X11_ResizeWindowShape;
+#endif // SDL_VIDEO_DRIVER_X11_XFIXES
 
 #ifdef SDL_VIDEO_OPENGL_GLX
     device->GL_LoadLibrary = X11_GL_LoadLibrary;
@@ -285,11 +202,12 @@ static SDL_VideoDevice *X11_CreateDevice(void)
     device->GL_SetSwapInterval = X11_GL_SetSwapInterval;
     device->GL_GetSwapInterval = X11_GL_GetSwapInterval;
     device->GL_SwapWindow = X11_GL_SwapWindow;
-    device->GL_DeleteContext = X11_GL_DeleteContext;
+    device->GL_DestroyContext = X11_GL_DestroyContext;
+    device->GL_GetEGLSurface = NULL;
 #endif
 #ifdef SDL_VIDEO_OPENGL_EGL
 #ifdef SDL_VIDEO_OPENGL_GLX
-    if (SDL_GetHintBoolean(SDL_HINT_VIDEO_X11_FORCE_EGL, SDL_FALSE)) {
+    if (SDL_GetHintBoolean(SDL_HINT_VIDEO_FORCE_EGL, false)) {
 #endif
         device->GL_LoadLibrary = X11_GLES_LoadLibrary;
         device->GL_GetProcAddress = X11_GLES_GetProcAddress;
@@ -299,21 +217,23 @@ static SDL_VideoDevice *X11_CreateDevice(void)
         device->GL_SetSwapInterval = X11_GLES_SetSwapInterval;
         device->GL_GetSwapInterval = X11_GLES_GetSwapInterval;
         device->GL_SwapWindow = X11_GLES_SwapWindow;
-        device->GL_DeleteContext = X11_GLES_DeleteContext;
+        device->GL_DestroyContext = X11_GLES_DestroyContext;
+        device->GL_GetEGLSurface = X11_GLES_GetEGLSurface;
 #ifdef SDL_VIDEO_OPENGL_GLX
     }
 #endif
 #endif
 
-    device->SetClipboardText = X11_SetClipboardText;
-    device->GetClipboardText = X11_GetClipboardText;
-    device->HasClipboardText = X11_HasClipboardText;
+    device->GetTextMimeTypes = X11_GetTextMimeTypes;
+    device->SetClipboardData = X11_SetClipboardData;
+    device->GetClipboardData = X11_GetClipboardData;
+    device->HasClipboardData = X11_HasClipboardData;
     device->SetPrimarySelectionText = X11_SetPrimarySelectionText;
     device->GetPrimarySelectionText = X11_GetPrimarySelectionText;
     device->HasPrimarySelectionText = X11_HasPrimarySelectionText;
     device->StartTextInput = X11_StartTextInput;
     device->StopTextInput = X11_StopTextInput;
-    device->SetTextInputRect = X11_SetTextInputRect;
+    device->UpdateTextInputArea = X11_UpdateTextInputArea;
     device->HasScreenKeyboardSupport = X11_HasScreenKeyboardSupport;
     device->ShowScreenKeyboard = X11_ShowScreenKeyboard;
     device->HideScreenKeyboard = X11_HideScreenKeyboard;
@@ -326,9 +246,23 @@ static SDL_VideoDevice *X11_CreateDevice(void)
     device->Vulkan_UnloadLibrary = X11_Vulkan_UnloadLibrary;
     device->Vulkan_GetInstanceExtensions = X11_Vulkan_GetInstanceExtensions;
     device->Vulkan_CreateSurface = X11_Vulkan_CreateSurface;
+    device->Vulkan_DestroySurface = X11_Vulkan_DestroySurface;
+    device->Vulkan_GetPresentationSupport = X11_Vulkan_GetPresentationSupport;
 #endif
 
+#ifdef SDL_USE_LIBDBUS
+    if (SDL_SystemTheme_Init())
+        device->system_theme = SDL_SystemTheme_Get();
+#endif
+
+    device->device_caps = VIDEO_DEVICE_CAPS_HAS_POPUP_WINDOW_SUPPORT |
+                          VIDEO_DEVICE_CAPS_SENDS_FULLSCREEN_DIMENSIONS;
+
     data->is_xwayland = X11_IsXWayland(x11_display);
+    if (data->is_xwayland) {
+        device->device_caps |= VIDEO_DEVICE_CAPS_MODE_SWITCHING_EMULATED |
+                               VIDEO_DEVICE_CAPS_DISABLE_MOUSE_WARP_ON_FULLSCREEN_TRANSITIONS;
+    }
 
     return device;
 }
@@ -349,9 +283,9 @@ static int X11_CheckWindowManagerErrorHandler(Display *d, XErrorEvent *e)
     }
 }
 
-static void X11_CheckWindowManager(_THIS)
+static void X11_CheckWindowManager(SDL_VideoDevice *_this)
 {
-    SDL_VideoData *data = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *data = _this->internal;
     Display *display = data->display;
     Atom _NET_SUPPORTING_WM_CHECK;
     int status, real_format;
@@ -363,7 +297,7 @@ static void X11_CheckWindowManager(_THIS)
     char *wm_name;
 #endif
 
-    /* Set up a handler to gracefully catch errors */
+    // Set up a handler to gracefully catch errors
     X11_XSync(display, False);
     handler = X11_XSetErrorHandler(X11_CheckWindowManagerErrorHandler);
 
@@ -390,7 +324,7 @@ static void X11_CheckWindowManager(_THIS)
         }
     }
 
-    /* Reset the error handler, we're done checking */
+    // Reset the error handler, we're done checking
     X11_XSync(display, False);
     X11_XSetErrorHandler(handler);
 
@@ -400,7 +334,7 @@ static void X11_CheckWindowManager(_THIS)
 #endif
         return;
     }
-    data->net_wm = SDL_TRUE;
+    data->net_wm = true;
 
 #ifdef DEBUG_WINDOW_MANAGER
     wm_name = X11_GetWindowTitle(_this, wm_window);
@@ -409,25 +343,23 @@ static void X11_CheckWindowManager(_THIS)
 #endif
 }
 
-int X11_VideoInit(_THIS)
+static bool X11_VideoInit(SDL_VideoDevice *_this)
 {
-    SDL_VideoData *data = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *data = _this->internal;
 
-    /* Get the window class name, usually the name of the application */
-    data->classname = get_classname();
-
-    /* Get the process PID to be associated to the window */
+    // Get the process PID to be associated to the window
     data->pid = getpid();
 
-    /* I have no idea how random this actually is, or has to be. */
+    // I have no idea how random this actually is, or has to be.
     data->window_group = (XID)(((size_t)data->pid) ^ ((size_t)_this));
 
-    /* Look up some useful Atoms */
-#define GET_ATOM(X) data->X = X11_XInternAtom(data->display, #X, False)
+    // Look up some useful Atoms
+#define GET_ATOM(X) data->atoms.X = X11_XInternAtom(data->display, #X, False)
     GET_ATOM(WM_PROTOCOLS);
     GET_ATOM(WM_DELETE_WINDOW);
     GET_ATOM(WM_TAKE_FOCUS);
     GET_ATOM(WM_NAME);
+    GET_ATOM(WM_TRANSIENT_FOR);
     GET_ATOM(_NET_WM_STATE);
     GET_ATOM(_NET_WM_STATE_HIDDEN);
     GET_ATOM(_NET_WM_STATE_FOCUSED);
@@ -437,6 +369,8 @@ int X11_VideoInit(_THIS)
     GET_ATOM(_NET_WM_STATE_ABOVE);
     GET_ATOM(_NET_WM_STATE_SKIP_TASKBAR);
     GET_ATOM(_NET_WM_STATE_SKIP_PAGER);
+    GET_ATOM(_NET_WM_MOVERESIZE);
+    GET_ATOM(_NET_WM_STATE_MODAL);
     GET_ATOM(_NET_WM_ALLOWED_ACTIONS);
     GET_ATOM(_NET_WM_ACTION_FULLSCREEN);
     GET_ATOM(_NET_WM_NAME);
@@ -450,7 +384,14 @@ int X11_VideoInit(_THIS)
     GET_ATOM(_SDL_WAKEUP);
     GET_ATOM(UTF8_STRING);
     GET_ATOM(PRIMARY);
+    GET_ATOM(CLIPBOARD);
+    GET_ATOM(INCR);
+    GET_ATOM(SDL_SELECTION);
+    GET_ATOM(TARGETS);
+    GET_ATOM(SDL_FORMATS);
+    GET_ATOM(XdndAware);
     GET_ATOM(XdndEnter);
+    GET_ATOM(XdndLeave);
     GET_ATOM(XdndPosition);
     GET_ATOM(XdndStatus);
     GET_ATOM(XdndTypeList);
@@ -460,42 +401,53 @@ int X11_VideoInit(_THIS)
     GET_ATOM(XdndSelection);
     GET_ATOM(XKLAVIER_STATE);
 
-    /* Detect the window manager */
+    // Detect the window manager
     X11_CheckWindowManager(_this);
 
-    if (X11_InitModes(_this) < 0) {
-        return -1;
+    if (!X11_InitModes(_this)) {
+        return false;
     }
 
-    X11_InitXinput2(_this);
+    if (!X11_InitXinput2(_this)) {
+        // Assume a mouse and keyboard are attached
+        SDL_AddKeyboard(SDL_DEFAULT_KEYBOARD_ID, NULL, false);
+        SDL_AddMouse(SDL_DEFAULT_MOUSE_ID, NULL, false);
+    }
 
 #ifdef SDL_VIDEO_DRIVER_X11_XFIXES
     X11_InitXfixes(_this);
-#endif /* SDL_VIDEO_DRIVER_X11_XFIXES */
+#endif // SDL_VIDEO_DRIVER_X11_XFIXES
+
+    X11_InitXsettings(_this);
 
 #ifndef X_HAVE_UTF8_STRING
-#warning X server does not support UTF8_STRING, a feature introduced in 2000! This is likely to become a hard error in a future libSDL2.
+#warning X server does not support UTF8_STRING, a feature introduced in 2000! This is likely to become a hard error in a future libSDL3.
 #endif
 
-    if (X11_InitKeyboard(_this) != 0) {
-        return -1;
+    if (!X11_InitKeyboard(_this)) {
+        return false;
     }
     X11_InitMouse(_this);
 
     X11_InitTouch(_this);
 
-    return 0;
+    X11_InitPen(_this);
+
+    return true;
 }
 
-void X11_VideoQuit(_THIS)
+void X11_VideoQuit(SDL_VideoDevice *_this)
 {
-    SDL_VideoData *data = (SDL_VideoData *)_this->driverdata;
+    SDL_VideoData *data = _this->internal;
 
     if (data->clipboard_window) {
         X11_XDestroyWindow(data->display, data->clipboard_window);
     }
 
-    SDL_free(data->classname);
+    if (data->xsettings_window) {
+        X11_XDestroyWindow(data->display, data->xsettings_window);
+    }
+
 #ifdef X_HAVE_UTF8_STRING
     if (data->im) {
         X11_XCloseIM(data->im);
@@ -506,13 +458,17 @@ void X11_VideoQuit(_THIS)
     X11_QuitKeyboard(_this);
     X11_QuitMouse(_this);
     X11_QuitTouch(_this);
+    X11_QuitPen(_this);
+    X11_QuitClipboard(_this);
+    X11_QuitXsettings(_this);
 }
 
-SDL_bool X11_UseDirectColorVisuals(void)
+bool X11_UseDirectColorVisuals(void)
 {
-    return SDL_getenv("SDL_VIDEO_X11_NODIRECTCOLOR") ? SDL_FALSE : SDL_TRUE;
+    if (SDL_GetHintBoolean(SDL_HINT_VIDEO_X11_NODIRECTCOLOR, false)) {
+        return false;
+    }
+    return true;
 }
 
-#endif /* SDL_VIDEO_DRIVER_X11 */
-
-/* vim: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_X11
