@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -12,17 +12,21 @@
 
 /* Simple test of the SDL MessageBox API */
 
-#include <stdio.h>
 #include <stdlib.h>
 
-#include "SDL.h"
+#include <SDL3/SDL.h>
+#include <SDL3/SDL_main.h>
+#include <SDL3/SDL_test.h>
 
 /* Call this instead of exit(), so we can clean up SDL: atexit() is evil. */
 static void
 quit(int rc)
 {
     SDL_Quit();
-    exit(rc);
+    /* Let 'main()' return normally */
+    if (rc != 0) {
+        exit(rc);
+    }
 }
 
 static int SDLCALL
@@ -81,15 +85,24 @@ button_messagebox(void *eventNumber)
 int main(int argc, char *argv[])
 {
     int success;
+    SDLTest_CommonState *state;
 
-    /* Enable standard application logging */
-    SDL_LogSetPriority(SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_PRIORITY_INFO);
+    /* Initialize test framework */
+    state = SDLTest_CommonCreateState(argv, 0);
+    if (!state) {
+        return 1;
+    }
+
+    /* Parse commandline */
+    if (!SDLTest_CommonDefaultArgs(state, argc, argv)) {
+        return 1;
+    }
 
     success = SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR,
                                        "Simple MessageBox",
                                        "This is a simple error MessageBox",
                                        NULL);
-    if (success == -1) {
+    if (!success) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Error Presenting MessageBox: %s\n", SDL_GetError());
         quit(1);
     }
@@ -98,7 +111,7 @@ int main(int argc, char *argv[])
                                        "Simple MessageBox",
                                        "This is a simple MessageBox with a newline:\r\nHello world!",
                                        NULL);
-    if (success == -1) {
+    if (!success) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Error Presenting MessageBox: %s\n", SDL_GetError());
         quit(1);
     }
@@ -107,7 +120,7 @@ int main(int argc, char *argv[])
                                        NULL,
                                        "NULL Title",
                                        NULL);
-    if (success == -1) {
+    if (!success) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Error Presenting MessageBox: %s\n", SDL_GetError());
         quit(1);
     }
@@ -116,7 +129,7 @@ int main(int argc, char *argv[])
                                        "NULL Message",
                                        NULL,
                                        NULL);
-    if (success == -1) {
+    if (!success) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Error Presenting MessageBox: %s\n", SDL_GetError());
         quit(1);
     }
@@ -126,7 +139,7 @@ int main(int argc, char *argv[])
                                        "UTF-8 Simple MessageBox",
                                        "Unicode text: '牛肉西蘭花' ...",
                                        NULL);
-    if (success == -1) {
+    if (!success) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Error Presenting MessageBox: %s\n", SDL_GetError());
         quit(1);
     }
@@ -136,7 +149,7 @@ int main(int argc, char *argv[])
                                        "UTF-8 Simple MessageBox",
                                        "Unicode text and newline:\r\n'牛肉西蘭花'\n'牛肉西蘭花'",
                                        NULL);
-    if (success == -1) {
+    if (!success) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Error Presenting MessageBox: %s\n", SDL_GetError());
         quit(1);
     }
@@ -146,7 +159,7 @@ int main(int argc, char *argv[])
                                        "牛肉西蘭花",
                                        "Unicode text in the title.",
                                        NULL);
-    if (success == -1) {
+    if (!success) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Error Presenting MessageBox: %s\n", SDL_GetError());
         quit(1);
     }
@@ -155,19 +168,19 @@ int main(int argc, char *argv[])
 
     /* Test showing a message box from a background thread.
 
-       On Mac OS X, the video subsystem needs to be initialized for this
+       On macOS, the video subsystem needs to be initialized for this
        to work, since the message box events are dispatched by the Cocoa
        subsystem on the main thread.
      */
-    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
+    if (!SDL_Init(SDL_INIT_VIDEO)) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't initialize SDL video subsystem: %s\n", SDL_GetError());
         return 1;
     }
     {
         int status = 0;
         SDL_Event event;
-        intptr_t eventNumber = SDL_RegisterEvents(1);
-        SDL_Thread *thread = SDL_CreateThread(&button_messagebox, "MessageBox", (void *)eventNumber);
+        Uint32 eventNumber = SDL_RegisterEvents(1);
+        SDL_Thread *thread = SDL_CreateThread(&button_messagebox, "MessageBox", (void *)(uintptr_t)eventNumber);
 
         while (SDL_WaitEvent(&event)) {
             if (event.type == eventNumber) {
@@ -183,30 +196,31 @@ int main(int argc, char *argv[])
     /* Test showing a message box with a parent window */
     {
         SDL_Event event;
-        SDL_Window *window = SDL_CreateWindow("Test", SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, 640, 480, 0);
+        SDL_Window *window = SDL_CreateWindow("Test", 640, 480, 0);
 
         /* On wayland, no window will actually show until something has
            actually been displayed.
         */
-        SDL_Renderer *renderer = SDL_CreateRenderer(window, -1, 0);
+        SDL_Renderer *renderer = SDL_CreateRenderer(window, NULL);
         SDL_RenderPresent(renderer);
 
         success = SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR,
                                            "Simple MessageBox",
                                            "This is a simple error MessageBox with a parent window. Press a key or close the window after dismissing this messagebox.",
                                            window);
-        if (success == -1) {
+        if (!success) {
             SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Error Presenting MessageBox: %s\n", SDL_GetError());
             quit(1);
         }
 
         while (SDL_WaitEvent(&event)) {
-            if (event.type == SDL_QUIT || event.type == SDL_KEYUP) {
+            if (event.type == SDL_EVENT_QUIT || event.type == SDL_EVENT_KEY_UP) {
                 break;
             }
         }
     }
 
     SDL_Quit();
+    SDLTest_CommonDestroyState(state);
     return 0;
 }
