@@ -11,14 +11,19 @@
 */
 /* This is a simple example of using GLSL shaders with SDL */
 
+#include <SDL3/SDL.h>
+#include <SDL3/SDL_main.h>
+#include <SDL3/SDL_test.h>
+
+#include "testutils.h"
+
 #include <stdlib.h>
-#include "SDL.h"
 
 #ifdef HAVE_OPENGL
 
-#include "SDL_opengl.h"
+#include <SDL3/SDL_opengl.h>
 
-static SDL_bool shaders_supported;
+static bool shaders_supported;
 static int current_shader = 0;
 
 enum
@@ -123,7 +128,7 @@ static PFNGLSHADERSOURCEARBPROC pglShaderSourceARB;
 static PFNGLUNIFORM1IARBPROC pglUniform1iARB;
 static PFNGLUSEPROGRAMOBJECTARBPROC pglUseProgramObjectARB;
 
-static SDL_bool CompileShader(GLhandleARB shader, const char *source)
+static bool CompileShader(GLhandleARB shader, const char *source)
 {
     GLint status = 0;
 
@@ -143,13 +148,13 @@ static SDL_bool CompileShader(GLhandleARB shader, const char *source)
             SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Failed to compile shader:\n%s\n%s", source, info);
             SDL_free(info);
         }
-        return SDL_FALSE;
+        return false;
     } else {
-        return SDL_TRUE;
+        return true;
     }
 }
 
-static SDL_bool LinkProgram(ShaderData *data)
+static bool LinkProgram(ShaderData *data)
 {
     GLint status = 0;
 
@@ -171,13 +176,13 @@ static SDL_bool LinkProgram(ShaderData *data)
             SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Failed to link program:\n%s", info);
             SDL_free(info);
         }
-        return SDL_FALSE;
+        return false;
     } else {
-        return SDL_TRUE;
+        return true;
     }
 }
 
-static SDL_bool CompileShaderProgram(ShaderData *data)
+static bool CompileShaderProgram(ShaderData *data)
 {
     const int num_tmus_bound = 4;
     int i;
@@ -191,18 +196,18 @@ static SDL_bool CompileShaderProgram(ShaderData *data)
     /* Create the vertex shader */
     data->vert_shader = pglCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
     if (!CompileShader(data->vert_shader, data->vert_source)) {
-        return SDL_FALSE;
+        return false;
     }
 
     /* Create the fragment shader */
     data->frag_shader = pglCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
     if (!CompileShader(data->frag_shader, data->frag_source)) {
-        return SDL_FALSE;
+        return false;
     }
 
     /* ... and in the darkness bind them */
     if (!LinkProgram(data)) {
-        return SDL_FALSE;
+        return false;
     }
 
     /* Set up some uniform variables */
@@ -217,7 +222,7 @@ static SDL_bool CompileShaderProgram(ShaderData *data)
     }
     pglUseProgramObjectARB(0);
 
-    return (glGetError() == GL_NO_ERROR) ? SDL_TRUE : SDL_FALSE;
+    return (glGetError() == GL_NO_ERROR);
 }
 
 static void DestroyShaderProgram(ShaderData *data)
@@ -229,12 +234,12 @@ static void DestroyShaderProgram(ShaderData *data)
     }
 }
 
-static SDL_bool InitShaders(void)
+static bool InitShaders(void)
 {
     int i;
 
     /* Check for shader support */
-    shaders_supported = SDL_FALSE;
+    shaders_supported = false;
     if (SDL_GL_ExtensionSupported("GL_ARB_shader_objects") &&
         SDL_GL_ExtensionSupported("GL_ARB_shading_language_100") &&
         SDL_GL_ExtensionSupported("GL_ARB_vertex_shader") &&
@@ -263,24 +268,24 @@ static SDL_bool InitShaders(void)
             pglShaderSourceARB &&
             pglUniform1iARB &&
             pglUseProgramObjectARB) {
-            shaders_supported = SDL_TRUE;
+            shaders_supported = true;
         }
     }
 
     if (!shaders_supported) {
-        return SDL_FALSE;
+        return false;
     }
 
     /* Compile all the shaders */
     for (i = 0; i < NUM_SHADERS; ++i) {
         if (!CompileShaderProgram(&shaders[i])) {
             SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Unable to compile shader!\n");
-            return SDL_FALSE;
+            return false;
         }
     }
 
     /* We're done! */
-    return SDL_TRUE;
+    return true;
 }
 
 static void QuitShaders(void)
@@ -304,7 +309,7 @@ power_of_two(int input)
     return value;
 }
 
-GLuint
+static GLuint
 SDL_GL_LoadTexture(SDL_Surface *surface, GLfloat *texcoord)
 {
     GLuint texture;
@@ -321,15 +326,7 @@ SDL_GL_LoadTexture(SDL_Surface *surface, GLfloat *texcoord)
     texcoord[2] = (GLfloat)surface->w / w; /* Max X */
     texcoord[3] = (GLfloat)surface->h / h; /* Max Y */
 
-    image = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32,
-#if SDL_BYTEORDER == SDL_LIL_ENDIAN     /* OpenGL RGBA masks */
-                                 0x000000FF,
-                                 0x0000FF00, 0x00FF0000, 0xFF000000
-#else
-                                 0xFF000000,
-                                 0x00FF0000, 0x0000FF00, 0x000000FF
-#endif
-    );
+    image = SDL_CreateSurface(w, h, SDL_PIXELFORMAT_RGBA32);
     if (!image) {
         return 0;
     }
@@ -356,13 +353,13 @@ SDL_GL_LoadTexture(SDL_Surface *surface, GLfloat *texcoord)
     glTexImage2D(GL_TEXTURE_2D,
                  0,
                  GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);
-    SDL_FreeSurface(image); /* No longer needed */
+    SDL_DestroySurface(image); /* No longer needed */
 
     return texture;
 }
 
 /* A general OpenGL initialization function.    Sets all of the initial parameters. */
-void InitGL(int Width, int Height) /* We call this right after our OpenGL window is created. */
+static void InitGL(int Width, int Height) /* We call this right after our OpenGL window is created. */
 {
     GLdouble aspect;
 
@@ -383,7 +380,7 @@ void InitGL(int Width, int Height) /* We call this right after our OpenGL window
 }
 
 /* The main drawing function. */
-void DrawGLScene(SDL_Window *window, GLuint texture, GLfloat *texcoord)
+static void DrawGLScene(SDL_Window *window, GLuint texture, GLfloat *texcoord)
 {
     /* Texture coordinate lookup, to make it simple */
     enum
@@ -446,23 +443,49 @@ void DrawGLScene(SDL_Window *window, GLuint texture, GLfloat *texcoord)
 
 int main(int argc, char **argv)
 {
+    int i;
     int done;
     SDL_Window *window;
+    char *filename = NULL;
     SDL_Surface *surface;
     GLuint texture;
     GLfloat texcoords[4];
+    SDLTest_CommonState *state;
 
-    /* Enable standard application logging */
-    SDL_LogSetPriority(SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_PRIORITY_INFO);
+    /* Initialize test framework */
+    state = SDLTest_CommonCreateState(argv, 0);
+    if (!state) {
+        return 1;
+    }
+
+    /* Parse commandline */
+    for (i = 1; i < argc;) {
+        int consumed;
+
+        consumed = SDLTest_CommonArg(state, i);
+        if (!consumed) {
+            if (!filename) {
+                filename = argv[i];
+                consumed = 1;
+            }
+        }
+        if (consumed <= 0) {
+            static const char *options[] = { "[icon.bmp]", NULL };
+            SDLTest_CommonLogUsage(state, argv[0], options);
+            exit(1);
+        }
+
+        i += consumed;
+    }
 
     /* Initialize SDL for video output */
-    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
+    if (!SDL_Init(SDL_INIT_VIDEO)) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Unable to initialize SDL: %s\n", SDL_GetError());
         exit(1);
     }
 
     /* Create a 640x480 OpenGL screen */
-    window = SDL_CreateWindow("Shader Demo", SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, 640, 480, SDL_WINDOW_OPENGL);
+    window = SDL_CreateWindow("Shader Demo", 640, 480, SDL_WINDOW_OPENGL);
     if (!window) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Unable to create OpenGL window: %s\n", SDL_GetError());
         SDL_Quit();
@@ -475,14 +498,17 @@ int main(int argc, char **argv)
         exit(2);
     }
 
-    surface = SDL_LoadBMP("icon.bmp");
+    filename = GetResourceFilename(NULL, "icon.bmp");
+    surface = SDL_LoadBMP(filename);
+    SDL_free(filename);
+
     if (!surface) {
         SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Unable to load icon.bmp: %s\n", SDL_GetError());
         SDL_Quit();
         exit(3);
     }
     texture = SDL_GL_LoadTexture(surface, texcoords);
-    SDL_FreeSurface(surface);
+    SDL_DestroySurface(surface);
 
     /* Loop, drawing and checking events */
     InitGL(640, 480);
@@ -499,14 +525,14 @@ int main(int argc, char **argv)
         {
             SDL_Event event;
             while (SDL_PollEvent(&event)) {
-                if (event.type == SDL_QUIT) {
+                if (event.type == SDL_EVENT_QUIT) {
                     done = 1;
                 }
-                if (event.type == SDL_KEYDOWN) {
-                    if (event.key.keysym.sym == SDLK_SPACE) {
+                if (event.type == SDL_EVENT_KEY_DOWN) {
+                    if (event.key.key == SDLK_SPACE) {
                         current_shader = (current_shader + 1) % NUM_SHADERS;
                     }
-                    if (event.key.keysym.sym == SDLK_ESCAPE) {
+                    if (event.key.key == SDLK_ESCAPE) {
                         done = 1;
                     }
                 }
@@ -515,6 +541,7 @@ int main(int argc, char **argv)
     }
     QuitShaders();
     SDL_Quit();
+    SDLTest_CommonDestroyState(state);
     return 1;
 }
 
@@ -527,5 +554,3 @@ int main(int argc, char *argv[])
 }
 
 #endif /* HAVE_OPENGL */
-
-/* vi: set ts=4 sw=4 expandtab: */
