@@ -12,39 +12,43 @@
 
 /* Simple test of the SDL threading code */
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
-#include <string.h>
 
-#include "SDL.h"
+#include <SDL3/SDL.h>
+#include <SDL3/SDL_main.h>
+#include <SDL3/SDL_test.h>
 
 #define NUMTHREADS 10
 
-static SDL_atomic_t time_for_threads_to_die[NUMTHREADS];
+static SDL_AtomicInt time_for_threads_to_die[NUMTHREADS];
 
 /* Call this instead of exit(), so we can clean up SDL: atexit() is evil. */
 static void
 quit(int rc)
 {
     SDL_Quit();
-    exit(rc);
+    /* Let 'main()' return normally */
+    if (rc != 0) {
+        exit(rc);
+    }
 }
 
-int SDLCALL
+static int SDLCALL
 SubThreadFunc(void *data)
 {
-    while (!*(int volatile *)data) {
-        ; /* SDL_Delay(10); */ /* do nothing */
+    SDL_AtomicInt *flag = (SDL_AtomicInt *)data;
+    while (!SDL_GetAtomicInt(flag)) {
+        SDL_Delay(10);
     }
     return 0;
 }
 
-int SDLCALL
+static int SDLCALL
 ThreadFunc(void *data)
 {
     SDL_Thread *sub_threads[NUMTHREADS];
-    int flags[NUMTHREADS];
+    SDL_AtomicInt flags[NUMTHREADS];
     int i;
     int tid = (int)(uintptr_t)data;
 
@@ -53,18 +57,18 @@ ThreadFunc(void *data)
     for (i = 0; i < NUMTHREADS; i++) {
         char name[64];
         (void)SDL_snprintf(name, sizeof(name), "Child%d_%d", tid, i);
-        flags[i] = 0;
+        SDL_SetAtomicInt(&flags[i], 0);
         sub_threads[i] = SDL_CreateThread(SubThreadFunc, name, &flags[i]);
     }
 
     SDL_Log("Thread '%d' waiting for signal\n", tid);
-    while (SDL_AtomicGet(&time_for_threads_to_die[tid]) != 1) {
+    while (SDL_GetAtomicInt(&time_for_threads_to_die[tid]) != 1) {
         ; /* do nothing */
     }
 
     SDL_Log("Thread '%d' sending signals to subthreads\n", tid);
     for (i = 0; i < NUMTHREADS; i++) {
-        flags[i] = 1;
+        SDL_SetAtomicInt(&flags[i], 1);
         SDL_WaitThread(sub_threads[i], NULL);
     }
 
@@ -77,13 +81,17 @@ int main(int argc, char *argv[])
 {
     SDL_Thread *threads[NUMTHREADS];
     int i;
+    SDLTest_CommonState *state;
 
-    /* Enable standard application logging */
-    SDL_LogSetPriority(SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_PRIORITY_INFO);
+    /* Initialize test framework */
+    state = SDLTest_CommonCreateState(argv, 0);
+    if (!state) {
+        return 1;
+    }
 
-    /* Load the SDL library */
-    if (SDL_Init(0) < 0) {
-        SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't initialize SDL: %s\n", SDL_GetError());
+    if (!SDLTest_CommonDefaultArgs(state, argc, argv)) {
+        SDL_Quit();
+        SDLTest_CommonDestroyState(state);
         return 1;
     }
 
@@ -91,7 +99,7 @@ int main(int argc, char *argv[])
     for (i = 0; i < NUMTHREADS; i++) {
         char name[64];
         (void)SDL_snprintf(name, sizeof(name), "Parent%d", i);
-        SDL_AtomicSet(&time_for_threads_to_die[i], 0);
+        SDL_SetAtomicInt(&time_for_threads_to_die[i], 0);
         threads[i] = SDL_CreateThread(ThreadFunc, name, (void *)(uintptr_t)i);
 
         if (threads[i] == NULL) {
@@ -101,12 +109,13 @@ int main(int argc, char *argv[])
     }
 
     for (i = 0; i < NUMTHREADS; i++) {
-        SDL_AtomicSet(&time_for_threads_to_die[i], 1);
+        SDL_SetAtomicInt(&time_for_threads_to_die[i], 1);
     }
 
     for (i = 0; i < NUMTHREADS; i++) {
         SDL_WaitThread(threads[i], NULL);
     }
     SDL_Quit();
+    SDLTest_CommonDestroyState(state);
     return 0;
 }
